# C720.tcl---LNL/Padova Silicon Shaper module control frame


set frame C720
set frametitle "Silicon Shaper Analyser Setup & Monitor"
set icontitle "C720"


set mname "c720"

set maxmodule 159

set maxmodule [inform camaccontrol inquire [set mname]count]
set modulelist {}
for {set i 1} {$i <= $maxmodule} {incr i} {lappend modulelist $i}

#    tables which give hardware register name connected to a widget
#      wrname (for write)  and  rrname  (for read)

set wrname(STOPENABLE)     Stop_Enable
set wrname(STARTENABLE)    Start_Enable
set wrname(LINEARGATE)     Linear_Gate
set wrname(ENABLE)         Module_Enable
set wrname(PUR)            PUR_Enable

set wrname(THRESH)         Master_Thresh
set wrname(POLEZERO)       PoleZero
set wrname(COTHRESH)       CO_Thresh
set wrname(GAIN)           Gain
set wrname(WALKTHRESH)     Walk_Thresh
set wrname(CFDDELAY)       CFD_Delay
set wrname(CFDWIDTH)       CFD_Width
set wrname(CFDTACDELAY)    CFD_TAC_Delay

set wrname(AIMUX)           AI_MUX
set wrname(DIMUX)           DI_MUX


#    make the two tables the same
foreach name [array names wrname] {set rrname($name) $wrname($name)}


for {set sid [array startsearch wrname]} {[array anymore wrname $sid]} { } {
	set element [array nextelement wrname $sid]
	set wname($wrname($element)) $element
}
for {set sid [array startsearch rrname]} {[array anymore rrname $sid]} { } {
	set element [array nextelement rrname $sid]
	set wname($rrname($element)) $element
}



loadsource common.camac

set reset_cmd Flags

proc repaint {} {
  global module
  :modify MODULE-NUM :value $module
}

proc redisplay-module {} {
  global rrname

   camac-address

   foreach w [array names rrname] {
      set register $rrname($w)
      update-widget $register [read-reg $register]
   }
}

proc click-mpx {w v} {
  global wrname rrname allmodules

#    first any existing signal must be deselected
  set all $allmodules
  set allmodules 1
  set register $wrname($w)
  write-reg $register 0

#    for setting multiplexer the all flag is ignored
   set allmodules 0
   set register $wrname($w)
   write-reg $register $v
   set allmodules $all

   set register $rrname($w)
   update-widget $register [read-reg $register]
}


proc click arg {
   global debug allmodules
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {

      MODULE-SEL      -
      MODULE-NUM {click-select-module $v}
      ALL        {set allmodules $v}

      THRESH  -
      POLEZERO  -
      COTHRESH  -
      WALKTHRESH  -
      CFDDELAY  -
      CFDWIDTH  -
      CFDTACDELAY  {click-slider $w $v}

      GAIN      {click-box $w $v}
      
      STOPENABLE  -
      STARTENABLE  -
      LINEARGATE  -
      PUR  -
      ENABLE    {click-box $w $v}

      *MUX      {click-mpx $w $v}

      REDISPLAY	{click-redisplay}
      default   {puts stdout $arg}
   }
}

proc click-select-adc cargs {}


set aimpxmenu [list Unselected "Input Monitor" "Gaussian Output" "Linear Gated Output" "Peak Detect Monitor" \
    "Clamped Signal" "BLR Monitor" "TAC Output"]

set dimpxmenu [list Unselected "Master TH Discriminator" "CFD TAC Output"  "CFD Out" "CO TH Discriminator"]

proc make-frame {} {
  global env maxmodule modulelist allmodules
  global camcrate camstation aimpxmenu dimpxmenu

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/camac_module" \
      :icon-name "[self icon-title]" :resize F

  :panel PANEL1  :layout T :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 10

     :number MODULE-NUM  :label "Module number:" :min 0 :max $maxmodule :w 3 
     :menubutton MODULE-SEL :label "" :strings $modulelist
     :text CAM-CRATE   :label "CAMAC Crate:"   :w 3 :value $camcrate :deaf T
     :text CAM-STATION :label "CAMAC Station:" :w 3 :value $camstation :deaf T
     :next-row T
     :hskip 300
     :checkbox ALL :label "Act on all modules?" :value $allmodules

  :panel PANEL2  :layout T :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 10

      :checkbox ENABLE       :label "Module Enable" 
      :checkbox STOPENABLE   :label "Stop TAC Enable" 
      :checkbox STARTENABLE  :label "Start TAC Enable"
     :next-row T
      :checkbox LINEARGATE   :label "Linear Gate Enable"
      :checkbox PUR          :label "PUR Enable"

   :panel PANEL3 :layout T :padx 10
     :layout V; :row-gap 5

       :slider THRESH    :label "Master Threshold:"     :min 0 :max 255  :valuex 25c
       :slider POLEZERO  :label "Pole Zero:"            :min 0 :max 255  :valuex 25c
       :slider COTHRESH  :label "CrossOver Threshold:"  :min 0 :max 255  :valuex 25c
       :slider WALKTHRESH  :label "Walk Threshold:"     :min 0 :max 255  :valuex 25c
       :slider CFDDELAY  :label "CFD Delay:"            :min 0 :max 255  :valuex 25c
       :slider CFDWIDTH  :label "CFD Width:"            :min 0 :max 255  :valuex 25c
       :slider CFDTACDELAY :label "CFD TAC Delay:"      :min 0 :max 255  :valuex 25c

       :vskip 10
       :choice GAIN      :label "Coarse Gain" :strings [list 0 1 2 3] :valuex 25c

       :vskip 10
       :choice-stack AIMUX :label "Analog Multiplexer" :strings $aimpxmenu   :valuex 25c
       :choice-stack DIMUX :label "Digital Multiplexer" :strings $dimpxmenu  :valuex 25c

   :panel CLICK :layout T :padx 10 :pady 10
     :layout H
       :button REDISPLAY 

  :show T

}

#proc EG {args} {insert-debug "$args"; return 2}
