# CAEN469.tcl---CAEN C469 gate and delay generator module control frame



set frame "CAEN469"
set frametitle "CAEN C469 gate and delay generator module setup and control frame "
set icontitle "CAEN469"

set mname "caen469"

set maxmodule 32
set maxadc 15

set wrname(RESET)	reset
set wrname(TEST)        test
set wrname(SET)         set

set wrname(DELAY)	delay
set wrname(WIDTH)	width
set wrname(SELECT)	select

set rrname(RESET)	reset
set rrname(TEST)        test
set rrname(SET)         set

set rrname(DELAY)	delay
set rrname(WIDTH)	width
set rrname(SELECT)	select

for {set sid [array startsearch wrname]} {[array anymore wrname $sid]} { } {
	set element [array nextelement wrname $sid]
	set wname($wrname($element)) $element
}
for {set sid [array startsearch rrname]} {[array anymore rrname $sid]} { } {
	set element [array nextelement rrname $sid]
	set wname($rrname($element)) $element
}


loadsource common.camac

set adcsarecalled "c"

proc redisplay-module {} {
   global debug
   if {$debug} then {puts stderr "redisplay-module with arguments"}
   camac-address
}

proc read-channel-reg {r} {
   global resource moduleprefix module channel mname adc
   EG "register :name '/$resource/$mname$moduleprefix$module.c$adc.$r' ?value"
}

proc redisplay-adc {} {
   global debug
   if {$debug} then {puts stderr "redisplay-channel"}
   set register delay
   set value [read-channel-reg $register]
   update-widget $register [expr $value]
   set register width
   set value [read-channel-reg $register]
   update-widget $register [expr $value]
}

proc redisplay-channels {} {}

proc click arg {
   global debug wrname
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch $w {
      ADC-SEL      -
      ADC-SEL-MENU -
      ADC-NUM {click-select-adc $v}

      MODULE-SEL      -
      MODULE-SEL-MENU -
      MODULE-NUM {click-select-module $v}

      ALL-MODULES     {click-all-modules $v}
      ALL-ADCS        {click-all-adcs $v}

      DELAY     -
      WIDTH	{click-adc-slider $w $v}
   
      SELECT    {write-adc-reg $wrname($w) 0}

      SET       {write-reg $wrname($w) 0}

      RESET 	-
      TEST 	{click-action $w}

      REDISPLAY	{click-redisplay}
      default   {puts stdout $arg}
   }
}


set selectmodulemenu {1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16}
set selectchannelmenu {0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15}


proc make-frame {} {
  global env
  global maxmodule maxadc camcrate camstation
  global selectmodulemenu selectchannelmenu

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/camac_module" \
      :icon-name "[self icon-title]" :resize F

  :panel MODULE  :layout T :padx 10 :pady 10
     :layout H; :row-gap 10

     :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5 :valuex 16c
     :menubutton MODULE-SEL :label "select:" :strings $selectmodulemenu
     :hskip 50
     :text CAM-CRATE   :label "CAMAC Crate:"   :w 5 :value $camcrate :deaf T
     :text CAM-STATION :label "CAMAC Station:" :w 5 :value $camstation :deaf T

   :panel ADC :layout T :padx 10
     :layout H; :row-gap 10; :col-gap 10

     :number ADC-NUM :label "Channel number:" :min 0 :max $maxadc :w 5 :valuex 16c
     :menubutton ADC-SEL :label "select:" :strings $selectchannelmenu
     :checkbox ALL-MODULES :label "Act on ALL modules?"
     :checkbox ALL-ADCS :label "Act on ALL channels?"

     :next-row T
     :slider DELAY  :label "Delay time:" :min 0 :max 255 :valuex 250
     :next-row T
     :slider WIDTH  :label "Gate width:" :min 0 :max 255 :valuex 250
     :next-row T
     :button SELECT    :label "Connect channel to MUX output"

  :panel CLICK :layout T :padx 10 :pady 10
     :layout H; :col-gap 15
     :button REDISPLAY :label "Redisplay"
     :hskip 40
     :button SET       :label "Setup module"
     :hskip 40
     :button RESET     :label "Reset Module"
     :button TEST      :label "Generate Test Signal"


  :show T
}
