# CAEN671.tcl-- CAEN 671 CFD module control frame


set frame CAEN671
set frametitle "CAEN 671 CFD Setup & Monitor"
set icontitle "CAEN671"

set mname "caen671"

set nchannels 16

set alladcs 0
set alladcs1 0
set alladcs2 0

set maxmodule [inform camaccontrol inquire [set mname]count]
set modulelist {}
for {set i 1} {$i <= $maxmodule} {incr i} {lappend modulelist $i}

set wrname(DWIDTH1)         Dwidth1
set wrname(DWIDTH2)         Dwidth2
set wrname(IMAJ)         Imaj
set wrname(EMAJ)         Emaj
set wrname(SUM)         Sum
set wrname(DEADTIME1)        Deadtime1
set wrname(DEADTIME2)        Deadtime2
set wrname(OWIDTH1)        Owidth1
set wrname(OWIDTH2)        Owidth2
set wrname(MINPUT) Minput
set wrname(MOUTPUT) Moutput
set wrname(MDELAYED) Mdelayed

for {set i 0} {$i < $nchannels} {incr i} {
	set wrname(THRESHOLD$i) Threshold
	set wrname(DELAY$i) Delay
	set wrname(ENABLE$i) Enable
	}

set rrname(DWIDTH1)         Dwidth1
set rrname(DWIDTH2)         Dwidth2
set rrname(IMAJ)         Imaj
set rrname(EMAJ)         Emaj
set rrname(SUM)         Sum
set rrname(DEADTIME1)        Deadtime1
set rrname(DEADTIME2)        Deadtime2
set rrname(OWIDTH1)        Owidth1
set rrname(OWIDTH2)        Owidth2
set rrname(MINPUT) Minput
set rrname(MOUTPUT) Moutput
set rrname(MDELAYED) Mdelayed

for {set i 0} {$i < $nchannels} {incr i} {
	set rrname(THRESHOLD$i) Threshold
	set rrname(DELAY$i) Delay
	set rrname(ENABLE$i) Enable
	}

for {set sid [array startsearch wrname]} {[array anymore wrname $sid]} { } {
    set element [array nextelement wrname $sid]
    set wname($wrname($element)) $element
}
for {set sid [array startsearch rrname]} {[array anymore rrname $sid]} { } {
    set element [array nextelement rrname $sid]
    set wname($rrname($element)) $element
}

for {set i 0} {$i < $nchannels} {incr i} {
	set wname(Threshold$i) THRESHOLD$i
	set wname(Delay$i) DELAY$i
	set wname(Enable$i) ENABLE$i
	}

loadsource common.camac

set reset_cmd "test"

proc repaint {} {
  global module adc
  :modify MODULE-NUM :value $module
}

proc redisplay-module {} {
   global debug nchannels adc
   if {$debug} then {puts stderr "redisplay-module"}
   camac-address
   foreach register {Dwidth1 Dwidth2 Deadtime1 Deadtime2 Imaj Emaj Sum Owidth1 Owidth2} {
       update-widget $register [read-reg $register]
   }
   foreach register {Minput Moutput Mdelayed} {
       update-widget $register [read-multiplexer $register]
   }
   set register Enable
   for {set adc 0} {$adc < $nchannels} {incr adc} {
	set v [expr [read-adc-reg $register]]
        update-widget $register$adc $v
	   if { $v==0 } { set t T } {set t F}
	   :modify THRESHOLD$adc :inactive $t
	   :modify DELAY$adc :inactive $t
   }

}

proc redisplay-channels {} {
   global debug adc nchannels
   if {$debug} then {puts stderr "redisplay-channels"}
   set register Threshold
   for {set adc 0} {$adc < $nchannels} {incr adc} {
      update-widget $register$adc [expr [read-adc-reg $register]]
   }
   set register Delay
   for {set adc 0} {$adc < $nchannels} {incr adc} {
      update-widget $register$adc [expr [read-adc-reg $register]]
   }
}

#    define dummy proc to replace version in common.camac
proc click-select-adc {i} {}

proc click-all-adcs1 {v} {global alladcs1; set alladcs1 $v}
proc click-all-adcs2 {v} {global alladcs2; set alladcs2 $v}

proc read-adc-reg {r} {
   global resource module adc mname
   global moduleprefix 
   EG "register :name '/$resource/$mname$moduleprefix$module.$r$adc'  ?value"
}

proc write-adc-reg {r v} {
   global resource module adc allmodules alladcs mname
   global moduleprefix 
   set opt :name
   if {$allmodules == 1} then {set lmodule \[?,??,???\]; set opt :pattern} \
                    else {set lmodule $module}
   if {$alladcs == 1}    then {set ladc \[?,??,???\]; set opt :pattern} \
                    else {set ladc $adc}
   EG  "register $opt '/$resource/$mname$moduleprefix$lmodule.$r$ladc' :value '$v'"
}

proc click-threshold {w v} {
   global adc alladcs alladcs1 wrname rrname 
   set alladcs $alladcs1
   set adc [string range $w 9 end]
   write-adc-reg $wrname($w) $v
   :modify $w :value [expr [read-adc-reg $rrname($w)]]
   if {$alladcs} {redisplay-channels}
}

proc click-delay {w v} {
   global adc alladcs alladcs2 wrname rrname
   set alladcs $alladcs2
   set adc [string range $w 5 end]
   write-adc-reg $wrname($w) $v
   :modify $w :value [expr [read-adc-reg $rrname($w)]]
   if {$alladcs} {redisplay-channels}
}

proc click-enable {w v} {
   global adc wrname rrname
   set adc [string range $w 6 end]
   if { $v==0 } { set t T } {set t F}
   :modify THRESHOLD$adc :inactive $t
   :modify DELAY$adc :inactive $t
   :modify ENABLE$adc :value $v
   write-adc-reg $wrname($w) $v
   :modify $w :value [expr [read-adc-reg $rrname($w)]]
}

proc click-multiplexer {w v} {
   global adc wrname rrname nchannels
	   for {set i 0} {$i < $nchannels} {incr i} {
		set adc  $i
		write-adc-reg $wrname($w) 0
	}
	set adc $v
        write-adc-reg $wrname($w) 1
}

proc read-multiplexer {w} {
   global adc rrname nchannels
	   set out 0
	   for {set i 0} {$i < $nchannels} {incr i} {
		set adc $i
		if {[expr [read-adc-reg $w]] == 1} {set out $adc}
	}
	return $out
}

proc click-enable-all {w v} {
   global nchannels
   for {set i 0} {$i < $nchannels} {incr i} {
	click-enable ENABLE$i $v
	}
   redisplay-channels
}

proc click arg {
   global debug rrname
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {

      MODULE-SEL      -
      MODULE-NUM      {click-select-module $v}

      ALL-MODULES {click-all-modules $v}
      ALL-ADCS1    {click-all-adcs1 $v}
      ALL-ADCS2    {click-all-adcs2 $v}

      ENABLE-ALL    {click-enable-all $w $v}
      ENABLE*    {click-enable $w $v}

      DWIDTH1 -
      DWIDTH2 -
      IMAJ -
      EMAJ -
      OWIDTH1 -
      OWIDTH2 -
      DEADTIME1 -
      DEADTIME2 -
      TTC -
      DEADTIME   {click-slider $w $v}

      MINPUT -
      MOUTPUT -
      MDELAYED  {click-multiplexer $w $v}
      SUM        {click-box $w $v}

      THRESHOLD*  {click-threshold $w $v}
      DELAY*  {click-delay $w $v}

      REDISPLAY {click-redisplay}
      default   {puts stdout $arg}
   }
}


proc make-frame {} {
  global env maxmodule camcrate camstation nchannels
  global modulelist

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/camac_module" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE :layout T :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 20

      :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5
      :menubutton MODULE-SEL :label "select:" :strings $modulelist
      :hskip 50
      :text CAM-CRATE   :label "CAMAC Crate:"    :w 5 :value $camcrate   :deaf T
      :text CAM-STATION :label "CAMAC Station:"  :w 5 :value $camstation :deaf T
      :hskip 50
      :checkbox ALL-MODULES :label "Act on ALL modules?"

      :next-row T
      :hskip 25
      :message M1 :label "Majority"  :bold T
      :next-row T
      :slider IMAJ :label "Internal Majority Threshold" :min 0 :max 255
      :slider EMAJ :label "External Majority Threshold" :min 0 :max 255
      :checkbox SUM :label "Current Sum Output" 

      :next-row T
      :hskip 25
      :message M2 :label "Multiplexers"  :bold T
      :next-row T
      :number MINPUT  :label "Input Channel and Threshold:" :min 0 :max 15 :w 5
      :number MOUTPUT  :label "Output Channel and Threshold:" :min 0 :max 15 :w 5
      :number MDELAYED :label "Delayed Output Channel:" :min 0 :max 15 :w 5

      :next-row T
      :hskip 25
      :message M3 :label "Channels 0-7: "  :bold T
      :next-row T
      :slider OWIDTH1 :label "Output Width" :min 0 :max 255
      :slider DWIDTH1 :label "Delayed Output Width" :min 0 :max 255
      :slider DEADTIME1    :label "Dead Time"     :min 0 :max 255

      :next-row T
      :hskip 25
      :message M4 :label "Channels 8-15:"  :bold T
      :next-row T
      :slider OWIDTH2 :label "Output Width" :min 0 :max 255
      :slider DWIDTH2 :label "Delayed Output Width" :min 0 :max 255
      :slider DEADTIME2    :label "Dead Time"     :min 0 :max 255

      :next-row T
      :hskip 25
      :message M7 :label "Enable Channels"  :bold T
      :next-row T
      for {set i 0} {$i < $nchannels} {incr i} {
        :checkbox ENABLE$i :label "$i" 
	}
      :checkbox ENABLE-ALL :label "All" 

      :next-row T
      :hskip 25
      :message M5 :label "Channel Thresholds"  :bold T
      :hskip 100
      :checkbox ALL-ADCS1 :label "Act on ALL channels?" 

      for {set i 0} {$i < $nchannels} {incr i} {
        if {[expr $i/4*4] == $i} then {:next-row T}
	:slider THRESHOLD$i :label $i: :valuex 2c :w 3 :min 0 :max 255
      }

      :next-row T
      :hskip 25
      :message M6 :label "Channel Delays    "  :bold T
      :hskip 100
      :checkbox ALL-ADCS2 :label "Act on ALL channels?" 

      for {set i 0} {$i < $nchannels} {incr i} {
        if {[expr $i/4*4] == $i} then {:next-row T}
        :slider DELAY$i :label $i: :valuex 2c :w 3 :min 0 :max 255
      }

   :panel CLICK :layout T :padx 10
      :layout H; :col-gap 30

      :button REDISPLAY :label Redisplay 

   :show T
}


