# CC.tcl---Parallel Crate Controller module control frame


set debug 0
set started 0


set frame CC
set frametitle "Parallel Crate Controller Setup & Monitor"
set icontitle "CC"


if {[info exists env(CamacServer)] == 0} then {set resource camac}  \
   else {set resource $env(CamacServer)}

set cchoice {1 2 3 4 5 6 7}
set camcrate "1"
set mname "CC#1"

proc define {c} {
    global debug resource mname
    set mname "CC#$c"
    EG  "register :pattern '/$resource/CC#$c.*' !delete"
    define-register CC#$c cCAMAC Z               "-c$c -n28 -a8 -f26 -q1"
    define-register CC#$c cCAMAC C               "-c$c -n28 -a9 -f26 -q1"
    define-register CC#$c cCAMAC SetI            "-c$c -n30 -a9 -f26 -q1"
    define-register CC#$c cCAMAC UnSetI          "-c$c -n30 -a9 -f24 -q1"
    define-register CC#$c cCAMAC TestI           "-c$c -n30 -a9 -f27 -q1"
    define-register CC#$c cCAMAC SetD            "-c$c -n30 -a10 -f26 -q1"
    define-register CC#$c cCAMAC UnSetD          "-c$c -n30 -a10 -f24 -q1"
    define-register CC#$c cCAMAC TestD           "-c$c -n30 -a10 -f27 -q1"
    define-register CC#$c cCAMAC TestLAM         "-c$c -n30 -a11 -f27 -q1"
 }

proc define-register {mname class rname attributes} {
   global debug resource
   set register /$resource/$mname.$rname
   set-footer "defining register $register"
   EG "register :name '$register' :class '$class' !new"
   EG "register :name '$register' :object '$attributes'"
   clear-footer
}

proc {no-QX} {module} {

   display-error-report self "no QX accessing $module - module probably missing or broken" \
        "check crate and station supplied in configuration are correct"
}

proc write-reg {r v} {
   global resource mname
   EG  "register :name '/$resource/$mname.$r' :value '$v'"
   }

proc read-reg {r} {
   global resource  mname
   EG "register :name '/$resource/$mname.$r' ?value"
   }

proc redisplay-status {} {
   global debug
   if {$debug} then {puts stderr "redisplay-status"}
   if {[read-reg TestI] == "%11"} then {:modify I :value 1} else {:modify I :value 0}
   if {[read-reg TestD] == "%11"} then {:modify D :value 1} else {:modify D :value 0}
   if {[read-reg TestLAM] == "%11"} then {:modify DMD :value 1} else {:modify DMD :value 0}

}

proc select-crate {v} {
   global debug cchoice camcrate
   set camcrate [lindex $cchoice $v]
   define $camcrate
}

proc click arg {
   global debug cnafdata camcrate
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch $w  {

      CAM-CRATE  {select-crate $v}

      Z       {read-reg Z}
      C       {read-reg C}
      SETI    {read-reg SetI}
      UNSETI  {read-reg UnSetI}
      SETD    {read-reg SetD}
      UNSETD  {read-reg UnSetD}

      DEFINE {define $camcrate}
      REDISPLAY	{}

      default   {puts stdout $arg}
      }
   redisplay-status
   }


proc start args {
   global started camcrate
   if {$started == 0} then {make-frame} else {open-frame}
   set started 1
   define $camcrate
   redisplay-status
}

proc click-quit {}   {close-frame}
proc finish     args {destroy-frame}

proc make-frame {} {
  global env  cchoice

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/camac_module" \
      :icon-name "[self icon-title]" :resize F

  :panel MODULE  :layout T :padx 10 :pady 10
    :layout H; :row-gap 10; :col-gap 20

    :hskip 200
    :choice-stack CAM-CRATE :label "CAMAC Crate:" :exclusive T :strings $cchoice :value 0

  :panel CLICK :layout T :padx 30
    :layout H; :col-gap 10; :row-gap 10

    :button Z   :label Z
    :button C   :label C
    :checkbox DMD  :label "Demand Present" :deaf T

    :next-row T
    :button SETI   :label "Set Inhibit"
    :button UNSETI :label "Unset Inhibit"
    :checkbox I    :label Inhibit :deaf T

    :next-row T
    :button SETD   :label "Enable Demand"
    :button UNSETD :label "Disable Demand"
    :checkbox D    :label Demand :deaf T

    :next-row T
    :button REDISPLAY    :label "Redisplay"
    :hskip 100
    :button DEFINE       :label "Define Registers"


   :show T
}
