# CC2405.tcl--- CAMAC crate controller

set frame CC2405
set frametitle "CC2405 CAMAC crate controller"
set icontitle "CC2405"

set mname "CC2405"

set maxmodule 0
set selectmodulemenu {}

loadsource common.camac


proc define {c} {
    global debug resource mname moduleprefix
    EG  "register :pattern '/$resource/$mname${moduleprefix}$c.*' !delete"
    define-register $mname${moduleprefix}$c cCAMAC Z               "-c$c -n28 -a8 -f26 -q1"
    define-register $mname${moduleprefix}$c cCAMAC genz            "-c$c -n28 -a8 -f26 -q1"

    define-register $mname${moduleprefix}$c cCAMAC C               "-c$c -n28 -a9 -f26 -q1"
    define-register $mname${moduleprefix}$c cCAMAC genc            "-c$c -n28 -a9 -f26 -q1"

    define-register $mname${moduleprefix}$c cCAMAC SetI            "-c$c -n30 -a9 -f26 -q1"
    define-register $mname${moduleprefix}$c cCAMAC geni            "-c$c -n30 -a9 -f26 -q1"

    define-register $mname${moduleprefix}$c cCAMAC UnSetI          "-c$c -n30 -a9 -f24 -q1"
    define-register $mname${moduleprefix}$c cCAMAC remi            "-c$c -n30 -a9 -f24 -q1"

    define-register $mname${moduleprefix}$c cCAMAC TestI           "-c$c -n30 -a9 -f27 -q1"
    define-register $mname${moduleprefix}$c cCAMAC tsti            "-c$c -n30 -a9 -f27 -q1"

    define-register $mname${moduleprefix}$c cCAMAC SetD            "-c$c -n30 -a10 -f26 -q1"
    define-register $mname${moduleprefix}$c cCAMAC UnSetD          "-c$c -n30 -a10 -f24 -q1"
    define-register $mname${moduleprefix}$c cCAMAC TestD           "-c$c -n30 -a10 -f27 -q1"
    define-register $mname${moduleprefix}$c cCAMAC TestLAM         "-c$c -n30 -a11 -f27 -q1"
 }

proc define-register {mname class rname attributes} {
   global debug resource
   set register /$resource/$mname.$rname
   set-footer "defining register $register"
   EG "register :name '$register' :class '$class' !new"
   EG "register :name '$register' :object '$attributes'"
   clear-footer
}

proc read-modules {} {
    global resource mname moduleprefix maxmodule selectmodulemenu crates
    set R [ EG "register :pattern '/$resource/$mname${moduleprefix}?.tsti' ?names-and-objects" ]
    set crates {} 
    set selectmodulemenu {}
    set maxmodule 0
    foreach r $R {
        set crate [string index $r 8]
	lappend crates $crate
	incr maxmodule
	lappend selectmodulemenu $maxmodule
    }
    return $crates
}

proc repaint {} {
    global module crates
    read-modules
    :modify MODULE-NUM :value $module
    :modify CAM-CRATE  :value [lindex $crates [expr $module-1]]
}

proc redisplay-module {} {
   global debug allmodules
   if {$debug} then {puts stderr "redisplay-module with arguments"}
   :modify ALL-MODULES :value $allmodules
   refresh-i
}

proc click-z {} {
    write-reg genz 0
}

proc click-c {} {
    write-reg genc 0
}

set inhb(0) remi
set inhb(1) geni

proc click-i {v} {
    global inhb
    write-reg $inhb($v) 0
}

proc refresh-i {} {
    set v1 [read-reg tsti]
    set v [string index $v1 1]
    :modify I :value $v
}

proc click arg {
   global debug module
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {

      MODULE-SEL -
      MODULE-NUM {click-select-module $v}

      Z		{ click-z }
      C		{ click-c }
      I		{ click-i $v }

      ALL-MODULES  {click-all-modules $v}

      DEFINE {define $module}
      REDISPLAY	{click-redisplay}
      default   {puts stdout $arg}
   }
}


#    tk version

proc make-frame {} {
  global env
  global maxmodule camcrate
  global selectmodulemenu

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/camac_module" \
      :icon-name "[self icon-title]" :resize F

  :panel MODULE  :layout T :padx 10 :pady 10
     :layout H; :row-gap 20; :col-gap 20

     :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5 
     :menubutton MODULE-SEL :label "select:" :strings $selectmodulemenu
     :hskip 50
     :text CAM-CRATE   :label "CAMAC Crate:"   :w 5 :value $camcrate :deaf T

     :next-row T
     :button Z :label "Z (CAMAC RESET)"
     :button C :label "C (CAMAC CLEAR)"
     :choice I :label "Inhibit" :strings { OFF ON }

  :panel CLICK :layout T :padx 10
     :layout H; :col-gap 20
     :button REDISPLAY :label "Redisplay"
     :hskip 60
     :button DEFINE       :label "Define Registers"
     :checkbox ALL-MODULES :label "Act on ALL modules?"

  :show T
}
