# FRC.tcl---LNL/Padova Fera Readout Controller control frame


set frame FRC
set frametitle "Fera Readout Controller Setup & Monitor"
set icontitle "FRC"

set mname "frc"

set maxmodule 3
set  vsn 0

#    tables which give hardware register name connected to a widget
#      wrname (for write)  and  rrname  (for read)

set wrname(RESETALL)       reset
set wrname(RESETEC)        resetEC
set wrname(RESETFIFOs)     resetFIFOs
set wrname(CLEARFERA)      ClearFera

set wrname(SVSN)           VSN

set wrname(FTSAMPLE)       FTSample
set wrname(FTENABLE)       FTSampleEnable
set wrname(VALSAMPLE)      ValSample
set wrname(VALSAMPLEENABLE) ValSampleEnable
set wrname(BUSYWIDTH)      Busy.Width
set wrname(BUSYDELAY)      Busy.Delay
set wrname(GATEDELAY)      Gate.Delay
set wrname(GATEWIDTH)      Gate.Width
set wrname(GATEENABLE)     GateEnable
set wrname(CLEARDELAY)     Clear.Delay
set wrname(CLEARWIDTH)     Clear.Width
set wrname(CLEARENABLE)    EndClearEnable

set wrname(WATCHDOG)       WdDelay
set wrname(WDENABLE)       WdEnable

set wrname(DMX1)           MPX0
set wrname(DMX2)           MPX1

set wrname(STARTSTOP)      StartStop
set wrname(INHIBIT)        FeraInhibit
set wrname(DREQENABLE)     DreqEnable
set wrname(VALENABLE)      ValEnable
set wrname(ECADJENABLE)    ECAdjustEnable

set wrname(COUNTERLSB)     Counter.LSB
set wrname(COUNTERMSB)     Counter.MSB


#    make the two tables the same
foreach name [array names wrname] {set rrname($name) $wrname($name)}


for {set sid [array startsearch wrname]} {[array anymore wrname $sid]} { } {
	set element [array nextelement wrname $sid]
	set wname($wrname($element)) $element
}
for {set sid [array startsearch rrname]} {[array anymore rrname $sid]} { } {
	set element [array nextelement rrname $sid]
	set wname($rrname($element)) $element
}



loadsource common.camac

 
proc repaint {} {
  global module
  :modify MODULE-NUM :value $module
}

proc redisplay-module {} {
  global rrname

   camac-address

   set register $rrname(SVSN)
   update-widget $register [read-reg $register]

   foreach w {STARTSTOP INHIBIT DREQENABLE VALENABLE ECADJENABLE} {
      set register $rrname($w)
      update-widget $register [read-reg $register]
   }

   foreach w {FTSAMPLE VALSAMPLE BUSYWIDTH BUSYDELAY GATEDELAY GATEWIDTH CLEARDELAY CLEARWIDTH WATCHDOG} {
      set register $rrname($w)
      update-widget $register [read-reg $register]
   }

   foreach w {VALSAMPLEENABLE FTENABLE WDENABLE CLEARENABLE GATEENABLE} {
      set register $rrname($w)
      update-widget $register [read-reg $register]
   }

   foreach w {DMX1 DMX2} {
      set register $rrname($w)
      update-widget $register [read-reg $register]
   }

   click-read-counter
}

proc click-read-counter {} {
  global rrname
  foreach w {COUNTERMSB COUNTERLSB} {
     set register $rrname($w)
     update-widget $register [read-reg $register]
  }
}

proc click-mpx {w v} {
  global wrname rrname
   set register $wrname($w)
   write-reg $register $v
   set register $rrname($w)
   update-widget $register [read-reg $register]
}


proc click arg {
   global debug chan allchans
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {

      MODULE-SEL-MENU -
      MODULE-SEL      -
      MODULE-NUM {click-select-module $v}

      RESETALL  -
      RESETFIFOS  -
      RESETEC  -
      CLEARFERA  {click-action $w}

      SVSN      {click-text $w $v}

      FTSAMPLE  -
      VALSAMPLE  -
      BUSYDELAY  -
      BUSYWIDTH  -
      WATCHDOG  -
      GATEDELAY  -
      GATEWIDTH  -
      CLEARDELAY  -
      CLEARWIDTH  {click-slider $w $v}

      FTENABLE  -
      VALSAMPLEENABLE  -
      WDENABLE  -
      CLEARENABLE  -
      GATEENABLE   {click-box $w $v}
      
      STARTSTOP  -
      INHIBIT  -
      DREQENABLE  -
      VALENABLE  -
      ECADJENABLE   {click-box $w $v}

      DMX*     {click-mpx $w $v}

      READ     {click-read-counter}

      REDISPLAY	{click-redisplay}
      default   {puts stdout $arg}
   }
}

set selectmodulemenu {1 2 3}

set mpxmenu [list Event "Start process" Busy WST WAk "Data request" STb Ack "End data" \
   Validation "Val sample" "Fast Trigger" "FT Sample" Gate Clear "Inhibit Out"]

proc make-frame {} {
  global env maxmodule
  global selectmodulemenu
  global camcrate camstation vsn mpxmenu

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/camac_module" \
      :icon-name "[self icon-title]" :resize F

  :panel PANEL1  :layout T :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 10

     :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5 
     :menubutton MODULE-SEL :label "select:" :strings $selectmodulemenu
     :hskip 20
     :text CAM-CRATE   :label "CAMAC Crate:"   :w 5 :value $camcrate :deaf T
     :text CAM-STATION :label "CAMAC Station:" :w 5 :value $camstation :deaf T
     :hskip 20
     :text SVSN        :label VSN: :w 5 :value $vsn

     :next-row T

     :button RESETEC    :label "Reset EC:"
     :button CLEARFERA  :label "Clear FERA"
     :button RESETFIFOS :label "Reset FiFos"
     :button RESETALL   :label "Reset All"
     :text COUNTERMSB :label "" :w 8 :deaf T 
     :col-gap 0
     :text COUNTERLSB :label "" :w 8 :deaf T
     :button READ  :label "Read Counter"

  :panel PANEL4  :layout T :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 20

      :checkbox STARTSTOP  :label "Start/Stop" 
      :checkbox INHIBIT    :label "Fera Inhibit"  
      :checkbox DREQENABLE :label "Dreq Enable"
      :checkbox VALENABLE  :label "Val Enable"  
      :checkbox ECADJENABLE  :label "EC Adjust Enable" 

     :next-row T
       :col-gap 50

      :choice-stack DMX1 :label "DMX #1" :strings $mpxmenu
      :choice-stack DMX2 :label "DMX #2" :strings $mpxmenu


   :panel PANEL2 :layout L :padx 10

     :layout H; :row-gap 5; :col-gap 0
       :checkbox FTENABLE :label ""
       :slider FTSAMPLE  :label "FT Sample:"  :min 0 :max 255  :valuex 15c
       :next-row T
       :checkbox VALSAMPLEENABLE  :label ""
       :slider VALSAMPLE :label "Val Sample:" :min 0 :max 255  :valuex 15c
       :next-row T
       :slider BUSYDELAY :label "Busy Delay:" :min 0 :max 255  :valuex 20c
       :next-row T
       :slider BUSYWIDTH :label "Busy Width:" :min 0 :max 255  :valuex 20c
       :next-row T
       :checkbox WDENABLE :label ""
       :slider WATCHDOG  :label "Watchdog Delay:" :min 0 :max 15    :valuex 15c

   :panel PANEL3 :layout L :padx 10

     :layout H; :row-gap 5; :col-gap 0
       :checkbox GATEENABLE :label ""
       :slider GATEDELAY :label "Gate Delay:" :min 0 :max 255   :valuex 15c
       :next-row T
       :slider GATEWIDTH :label "Gate Width:" :min 0 :max 255  :valuex 20c
       :next-row T
       :checkbox CLEARENABLE :label ""
       :slider CLEARDELAY :label "Clear Delay:" :min 0 :max 255  :valuex 15c
       :next-row T
       :slider CLEARWIDTH :label "Clear Width:" :min 0 :max 255  :valuex 20c


  :panel CLICK :layout B :after FOOTER
     :hskip 40; :vskip 10
     :button REDISPLAY :label "Redisplay"

  :show T


}
