# LC2277.tcl---LeCroy 2277 TDC module control frame
#   see  CDOC219


set frame LC2277
set frametitle "LeCroy 2277 TDC Setup & Monitor"
set icontitle "LC2277"

set mname "lc2277"

set maxmodule [inform camaccontrol inquire [set mname]count]
set modulelist {}
for {set i 1} {$i <= $maxmodule} {incr i} {lappend modulelist $i}

set module [first $modulelist]

set adcdata "0x0000"

set wrname(CLEAR)       clear
set wrname(TESTLAM)     testlam
set wrname(CLEARLAM)    clearlam
set wrname(DISABLE)     disable
set wrname(ENABLE)      enable
set wrname(TEST)        test
set wrname(TESTBUFFERING)  testbuffering
set wrname(TESTBUSY)    testbusy

set wrname(DATA)	data

set wrname(SMODE)	status.mode
set wrname(STIM)	status.timeout
set wrname(SLEAD)	status.leading
set wrname(STRAIL)	status.trailing
set wrname(SETEST)	status.eventest
set wrname(SOTEST)	status.oddtest
set wrname(STMODE)	status.trigmode
set wrname(STHIT)	status.testhit
set wrname(STHITL)	status.testhitlen
set wrname(STATUS)      status

set rrname(CLEAR)       clear
set rrname(TESTLAM)     testlam
set rrname(CLEARLAM)    clearlam
set rrname(DISABLE)     disable
set rrname(ENABLE)      enable
set rrname(TEST)        test
set rrname(TESTBUFFERING)  testbuffering
set rrname(TESTBUSY)    testbusy

set rrname(SMODE)	status.mode
set rrname(STIM)	status.timeout
set rrname(SLEAD)	status.leading
set rrname(STRAIL)	status.trailing
set rrname(SETEST)	status.eventest
set rrname(SOTEST)	status.oddtest
set rrname(STMODE)	status.trigmode
set rrname(STHIT)	status.testhit
set rrname(STHITL)	status.testhitlen
set rrname(STATUS)        status



for {set sid [array startsearch wrname]} {[array anymore wrname $sid]} { } {
	set element [array nextelement wrname $sid]
	set wname($wrname($element)) $element
}
for {set sid [array startsearch rrname]} {[array anymore rrname $sid]} { } {
	set element [array nextelement rrname $sid]
	set wname($rrname($element)) $element
}

loadsource common.camac


proc repaint {} {
  global module
  :modify MODULE-NUM :value $module
}

proc redisplay-module {} {
   global debug
   if {$debug} then {puts stderr "redisplay-module with arguments"}
   camac-address
   set register status.mode
   update-text-widget $register [read-reg $register]
   set register status.timeout
   update-widget $register [read-reg $register]
   set register status.leading
   update-widget $register [read-reg $register]
   set register status.trailing
   update-widget $register [read-reg $register]
   set register status.eventest
   update-widget $register [read-reg $register]
   set register status.oddtest
   update-widget $register [read-reg $register]
   set register status.trigmode
   update-widget $register [read-reg $register]
   set register status.testhit
   update-widget $register [read-reg $register]
   set register status.testhitlen
   update-widget $register [read-reg $register]
}

proc click-select-adc {i} {}

proc click arg {
   global debug rrname
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$f=="ER"} then {cleanup-frame; return}
   if {$c==":quit"} then {click-quit; return}
   switch $w {
      MODULE-SEL-MENU -
      MODULE-SEL      -
      MODULE-NUM      {click-select-module $v}

      ALL-MODULES {click-all-modules $v}
 
      READADC   {click-read-data $w}

      SMODE	-
      STIM      -
      SLEAD	-
      STRAIL	-
      SETEST	-
      SOTEST	-
      STMODE	-
      STHIT     -
      STHITL    {click-box $w $v}

      CLEAR	    -
      TEST          -
      CLEARLAM      -
      DISABLE	    -
      ENABLE	    {write-reg $rrname($w) 0}

      TESTLAM	    -
      TESTBUFFERING -
      TESTBUSY	    {read-reg $rrname($w)}

      REDISPLAY	{click-redisplay}
      default   {puts stdout $arg}
   }
}


proc make-frame {} {
  global env camcrate camstation maxmodule adcdata
  global modulelist

set smodechoice {"common stop" "common start"}
set sleadchoice {"disable" "enable"}
set strailchoice {"disable" "enable"}
set setestchoice {"disable" "enable"}
set sotestchoice {"disable" "enable"}
set stmodechoice {"pulse" "latch"}
set stimchoice {"front panel" "1024 nsec" "2048 nsec" "4096 nsec" "8192 nsec" "16384 nsec" \
                    "32768 nsec" "65536 nsec"}
set sthitchoice {"1" "2" "4" "8"}
set sthitlchoice {"128 nsec" "256 nsec" "1024 nsec" "2048 nsec"}


  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/camac_module" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE :layout T :padx 10 :pady 10
     :layout H; :row-gap 10

      :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5
      :menubutton MODULE-SEL :label "select:" :strings $modulelist
      :hskip 100
      :text CAM-CRATE   :label "CAMAC Crate:"   :w 5 :value $camcrate :deaf T
      :text CAM-STATION :label "CAMAC Station:" :w 5 :value $camstation :deaf T

      :next-row T; :hskip 10
      :message M1 :label "Status Register:" :bold T

    :panel P1 :layout L :padx 10
      :layout V; :row-gap 15

      :choice-stack SMODE  :label "Mode:" :exclusive T  \
         :strings $smodechoice :value 0 
      :choice-stack SLEAD  :label "Leading Edge:" :exclusive T    \
         :strings $sleadchoice  :value 0 
      :choice-stack STRAIL  :label "Trailing Edge:" :exclusive T   \
         :strings $strailchoice  :value 0

    :panel P2 :layout L :padx 10
      :layout V; :row-gap 15

      :choice-stack SETEST  :label "Even Channel Test Input:" :exclusive T  \
         :strings $setestchoice :value 0
      :choice-stack SOTEST  :label "Odd Channel Test Input:" :exclusive T  \
         :strings $sotestchoice :value 0
      :choice-stack STMODE  :label "Trigger Mode:" :exclusive T  \
         :strings $stmodechoice :value 0

    :panel P3 :layout L :padx 10
      :layout V; :row-gap 15

      :choice-stack STIM  :label "Common start timeout:" :exclusive T   \
         :strings $stimchoice  :value 1
      :choice-stack STHIT  :label "Tester hit burst:" :exclusive T   \
         :strings $sthitchoice :value 0
      :choice-stack STHITL  :label "Tester hit duration:" :exclusive T  \
         :strings $sthitlchoice  :value 0

    :panel P4 :layout B :padx 10 :pady 10 :after FOOTER
      :layout H; :hskip 500
      :checkbox ALL-MODULES :label "Act on ALL modules?"

      :row-gap 15; :next-row T; :col-gap 50
      :checkbox TESTLAM :label "LAM present" :deaf T
      :checkbox TESTBUSY  :label "Busy"    :deaf T
      :checkbox TESTBUFFERING  :label "buffering in progress"  :deaf T
      :text ADCDATA :label "Data:" :w 10 :value $adcdata :deaf T
      :col-gap 0
      :button READADC :label "read"
   
   :panel CLICK :layout B :padx 10 :after FOOTER
      :layout H; :col-gap 20
      :button REDISPLAY :label "Redisplay"
      :hskip 25
      :button CLEAR     :label "Clear Module"
      :button TEST      :label "Test"
      :button ENABLE    :label "Enable LAM"
      :button DISABLE   :label "Disable LAM"
      :button CLEARLAM  :label "Clear LAM"

      :show T

}
