# LC2551.tcl---LeCroy 2551 Scaler module control frame
#   see CDOC218


set frame LC2551
set frametitle "LeCroy 2551 Scaler Setup & Monitor"
set icontitle "LC2551"

set mname "lc2551"

set nchannels 12

set maxmodule [inform camaccontrol inquire [set mname]count]
set modulelist {}
for {set i 1} {$i <= $maxmodule} {incr i} {lappend modulelist $i}

set module [first $modulelist]

set wrname(RESET)       reset
set wrname(TESTLAM)     testlam
set wrname(DISABLE)     disable
set wrname(ENABLE)      enable
set wrname(INCREMENT)   increment

set rrname(RESET)       reset
set rrname(TESTLAM)     testlam
set rrname(DISABLE)     disable
set rrname(ENABLE)      enable
set rrname(INCREMENT)   increment

set rrname(READDATA)     data
set rrname(READDATACLR)  data+clr

set DATA DATA
for {set i 0} {$i < $nchannels} {incr i} {set wname(data$i)     C$i$DATA}
for {set i 0} {$i < $nchannels} {incr i} {set wname(data+clr$i) C$i$DATA}

for {set sid [array startsearch wrname]} {[array anymore wrname $sid]} { } {
    set element [array nextelement wrname $sid]
    set wname($wrname($element)) $element
}
for {set sid [array startsearch rrname]} {[array anymore rrname $sid]} { } {
    set element [array nextelement rrname $sid]
    set wname($rrname($element)) $element
}

loadsource common.camac

proc repaint {} {
  global module adc
  :modify MODULE-NUM :value $module
}

proc redisplay-module {} {
   global debug
   if {$debug} then {puts stderr "redisplay-module"}
   camac-address
}

proc redisplay-channels {} {
   global debug channel nchannels
   if {$debug} then {puts stderr "redisplay-channels"}
   set register data
   for {set i 0} {$i < $nchannels} {incr i} {
      set channel $i
      update-text-widget $register$channel [expr [read-channel-reg $register]]
   }
}

#    define dummy proc to replace version in common.camac
proc click-select-adc {i} {}

proc click-clear {} {click-read-channels READDATACLR; click-redisplay}

proc click-clear-all {}  {
  global modulelist
   foreach module $modulelist {
      click-select-module $module
      click-clear
   }
}

proc click arg {
   global debug rrname
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch $w {

      MODULE-SEL      -
      MODULE-SEL-MENU -
      MODULE-NUM      {click-select-module $v}

      READDATA    -
      READDATACLR {click-read-channels $w; click-redisplay}

      RESET     -
      DISABLE    -
      ENABLE    -
      INCREMENT {read-reg $rrname($w)}
    

      REDISPLAY    {click-redisplay}
      default   {puts stdout $arg}
   }
}


set gap 100
set col1 50
set col2  [expr $col1+$gap]
set col3  [expr $col2+$gap]


proc make-frame {} {
  global env maxmodule camcrate camstation nchannels
  global modulelist

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/camac_module" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE :layout T :padx 10 :pady 10
     :layout H; :row-gap 10

      :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5
      :menubutton MODULE-SEL :label "select:" :strings $modulelist
      :hskip 100
      :text CAM-CRATE   :label "CAMAC Crate:"    :w 5 :value $camcrate   :deaf T
      :text CAM-STATION :label "CAMAC Station:"  :w 5 :value $camstation :deaf T

      :next-row T; :col-gap 20
      :hskip 25
      :message M1 :label "Channel values"  :bold T
      :button READDATA :label "read"
      :button READDATACLR :label "read & clear" 

      :hskip 200
      :checkbox LAM  :label "LAM present"   :deaf T

      for {set i 0} {$i < $nchannels} {incr i} {
        if {[expr $i/6*6] == $i} then {
          :next-row T
          :text C[set i]DATA :label $i: :w 10 :valuex 2c :value 0 :deaf T
        } else {
          :text C[set i]DATA :label $i: :w 10 :valuex 2c :value 0 :deaf T
        }
      }

   :panel CLICK :layout T :padx 10
      :layout H; :col-gap 10

      :button REDISPLAY :label Redisplay
      :button CLEAR     :label "Clear all scalers"  :command click-clear
      :button RESET     :label "Reset Module"
      :button ENABLE    :label "Enable LAM"
      :button DISABLE   :label "Disable LAM"
      :button INCREMENT :label "Increment Scalers"

   :show T
}

