# LC3420.tcl---LeCroy 3420 CFD module control frame


set frame LC3420
set frametitle "LeCroy 3420 CFD Setup & Monitor"
set icontitle "LC3420"

set mname "lc3420"

set nchannels 16

set maxmodule [inform camaccontrol inquire [set mname]count]
set modulelist {}
for {set i 1} {$i <= $maxmodule} {incr i} {lappend modulelist $i}
set module [first $modulelist]

set wrname(TEST)        test
set wrname(DEADTIME)    DeadTime
set wrname(WIDTH)   Width
set wrname(TTC)         TimeToCharge
set wrname(MMASK)        MMask
for {set i 0} {$i < $nchannels} {incr i} {
	set wrname(THRESHOLD$i) Threshold
}

set rrname(TEST)        test
set rrname(DEADTIME)    DeadTime
set rrname(WIDTH)   Width
set rrname(TTC)         TimeToCharge
set rrname(MMASK)        MMask
for {set i 0} {$i < $nchannels} {incr i} {
	set rrname(THRESHOLD$i) Threshold
}

for {set sid [array startsearch wrname]} {[array anymore wrname $sid]} { } {
    set element [array nextelement wrname $sid]
    set wname($wrname($element)) $element
}
for {set sid [array startsearch rrname]} {[array anymore rrname $sid]} { } {
    set element [array nextelement rrname $sid]
    set wname($rrname($element)) $element
}

for {set i 0} {$i < $nchannels} {incr i} {
	set wname(Threshold$i) THRESHOLD$i
}

loadsource common.camac

set reset_cmd "test"


proc repaint {} {
  global module adc
  :modify MODULE-NUM :value $module
}

proc redisplay-module {} {
   global debug
   if {$debug} then {puts stderr "redisplay-module"}
   camac-address
   foreach register {DeadTime Width TimeToCharge MMask} {
       update-widget $register [read-reg $register]
   }
}

proc redisplay-channels {} {
   global debug adc nchannels
   if {$debug} then {puts stderr "redisplay-channels"}
   set register Threshold
   for {set adc 0} {$adc < $nchannels} {incr adc} {
      update-widget $register$adc [expr [read-adc-reg $register]]
   }
   set register MMask
   set v [read-reg $register]
   update-widget $register $v
   if {[expr $v] != 65535} {:modify MASK-ALL :value 0} {:modify MASK-ALL :value 1}
   set register Mask
   for {set adc 0} {$adc < $nchannels} {incr adc} {
      [:path MASK].bit$adc delete 0 end
      [:path MASK].bit$adc insert 0 [expr [read-adc-reg $register]]
   }
}

#    define dummy proc to replace version in common.camac
proc click-select-adc {i} {}

proc read-adc-reg {r} {
   global resource module adc mname
   global moduleprefix 
   EG "register :name '/$resource/$mname$moduleprefix$module.$r$adc'  ?value"
}

proc write-adc-reg {r v} {
   global resource module adc allmodules alladcs mname
   global moduleprefix 
   set opt :name
   if {$allmodules == 1} then {set lmodule \[?,??,???\]; set opt :pattern} \
                    else {set lmodule $module}
   if {$alladcs == 1}    then {set ladc \[?,??,???\]; set opt :pattern} \
                    else {set ladc $adc}
   EG  "register $opt '/$resource/$mname$moduleprefix$lmodule.$r$ladc' :value '$v'"
}

proc click-threshold {w v} {
   global adc alladcs wrname rrname
   set adc [string range $w 9 end]
   write-adc-reg $wrname($w) $v
   :modify $w :value [expr [read-adc-reg $rrname($w)]]
   if {$alladcs} {redisplay-channels}
}

proc click-mask {i v} {
   global adc wrname rrname
   set adc $i
   set vl $v
   if {$vl == 2} then {
       set vl [[:path MASK].bit$adc get]
       if {$vl == 0} then {set vl 1} else {set vl 0}
   }
#   if { $vl==1 } { set t T } {set t F}
#   :modify THRESHOLD$adc :inactive $t
   [:path MASK].bit$adc delete 0 end
   [:path MASK].bit$adc insert 0 $vl
   write-adc-reg Mask $vl
   set vl [expr [read-adc-reg Mask]]
   [:path MASK].bit$adc delete 0 end
   [:path MASK].bit$adc insert 0 $vl
   set register MMask
   set vl [read-reg $register]
   update-widget $register $vl
   if {[expr $vl] != 255} {:modify MASK-ALL :value 0} {:modify MASK-ALL :value 1}
}

proc click-mask-all {v} {
   global nchannels
   for {set i 0} {$i < $nchannels} {incr i} {
	click-mask $i $v
   }
   redisplay-channels
}

proc click-mmask {v} {
   global nchannels
   for {set i 0} {$i < $nchannels} {incr i} {click-mask $i 0}
   foreach i [bits-in $v] {
	click-mask $i 1
   }
   redisplay-channels
}

proc click arg {
   global debug rrname
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {

      MODULE-SEL      -
      MODULE-NUM   {click-select-module $v; click-redisplay}

      ALL-MODULES  {click-all-modules $v}
      ALL-ADCS     {click-all-adcs $v}

      WIDTH -
      TTC -
      DEADTIME     {click-slider $w $v}

      THRESHOLD*   {click-threshold $w $v}

      MASK-ALL     {click-mask-all $v}
      MMASK        {click-mmask $v}

      TEST      {read-reg $rrname($w)}
    
      REDISPLAY {click-redisplay}
      default   {puts stdout $arg}
   }
}


proc make-frame {} {
  global env maxmodule camcrate camstation nchannels
  global modulelist

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/camac_module" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE :layout T :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 20

      :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5
      :menubutton MODULE-SEL :label "select:" :strings $modulelist
      :hskip 50
      :text CAM-CRATE   :label "CAMAC Crate:"    :w 5 :value $camcrate   :deaf T
      :text CAM-STATION :label "CAMAC Station:"  :w 5 :value $camstation :deaf T
      :hskip 50
      :checkbox ALL-MODULES :label "Act on ALL modules?"

      :next-row T
      :slider DEADTIME :label "Dead Time" :min 0 :max 15
      :slider WIDTH    :label "Width"     :min 0 :max 15
      :slider TTC      :label "Time-to-Charge" :min 0 :max 255

      :next-row T
      :hskip 25
      :message M1 :label "Mask channels"  :bold T
      :next-row T
      :col-gap 0

       :message MASK0  :label 15 :bold T

        :canvas MASK  :w 370 :h 28
        [:path MASK] config -background $env(tkframebackground)

        set x 15
        set y 15
        for {set i 15} {$i >= 0} {incr i -1} {
           entry [:path MASK].bit$i -relief flat -width 1 -justify right
           [:path MASK].bit$i insert 0 " "
           :draw window :tag MASKbit$i :value $x $y -window [:path MASK].bit$i 
           bind [:path MASK].bit$i <ButtonPress-1>  "click-mask $i 2"
           incr x 20
           if {[expr ($i)/4*4] == [expr ($i)]} then {incr x 15}
        }

       :message MASK15  :label 0 :bold T

      :col-gap 20

      :checkbox MASK-ALL :label "All" 
      :hskip 25
      :text MMASK       :label "Mask"  :w 8

      :next-row T
      :hskip 25
      :message M2 :label "Channel Thresholds"  :bold T
      :hskip 200
      :checkbox ALL-ADCS :label "Act on ALL channels?" 

      for {set i 0} {$i < $nchannels} {incr i} {
        if {[expr $i/4*4] == $i} then {:next-row T}
        :slider THRESHOLD$i :label $i: :valuex 2c :w 3 :min 0 :max 255
      }

   :panel CLICK :layout T :padx 10
      :layout H; :col-gap 30

      :button REDISPLAY :label Redisplay 
      :hskip 40
      :button TEST     :label "Trigger Test Function"
      :button SAVERESTOTE     :label "Save/Restore"  :command "inform camacsaverestore start"
      :menubutton MPX  :label "Connect to Scope Channel" :inactive T :command click_mpx :strings [list 1 2 3 4]

   do_mpx

   :show T
}

proc do_mpx {} {}

