# LC4208.tcl---LeCroy 4208 TDC module control frame
#   see CDOC216


set frame "LC4208"
set frametitle "LeCroy 4208 TDC Setup & Monitor"
set icontitle "LC4208"

set mname "lc4208"

set maxmodule 32
set maxadc 7

set adcdata "0x0000"

set wrname(RESET)	reset

set rrname(RESET)	reset

set rrname(READADC)     data
set rrname(READADCCLR)  data+clr


for {set sid [array startsearch wrname]} {[array anymore wrname $sid]} { } {
	set element [array nextelement wrname $sid]
	set wname($wrname($element)) $element
}
for {set sid [array startsearch rrname]} {[array anymore rrname $sid]} { } {
	set element [array nextelement rrname $sid]
	set wname($rrname($element)) $element
}

loadsource common.camac
 
proc redisplay-module {} {
   global debug
   if {$debug} then {puts stderr "redisplay-module with arguments"}
   camac-address
}
 
proc click-reset {w} {
   global rrname
   set register $rrname($w)
   read-reg $register
}

proc click arg {
   global debug
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$f=="ER"} then {cleanup-frame; return}
   if {$c==":quit"} then {click-quit; return}
   switch $w {
      ADC-SEL-MENU -
      ADC-SEL      -
      ADC-NUM      {click-select-adc $v}

      MODULE-SEL-MENU -
      MODULE-SEL      -
      MODULE-NUM      {click-select-module $v}

      READADC    -
      READADCCLR {click-read-adc $w}

      RESET 	{click-reset $w}
      default   {puts stdout $arg}
   }
}

set selectmodulemenu {1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16}
set selectadcmenu {0 1 2 3 4 5 6 7}


proc make-frame {} {
  global env maxmodule maxadc camcrate camstation adcdata
  global selectmodulemenu selectadcmenu


  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/camac_module" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE :layout T :padx 10
      :layout H; :row-gap 10
      :vskip 10

      :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5
      :menubutton MODULE-SEL :label "select:" :strings $selectmodulemenu 
      :hskip 100
      :text CAM-CRATE   :label "CAMAC Crate:"     :w 5 :value $camcrate :deaf T
      :text CAM-STATION :label "CAMAC Station:"   :w 5 :value $camstation :deaf T
   
   :panel ADC :layout T :padx 10 :pady 10
      :layout H; :row-gap  10

      :number ADC-NUM :label "ADC number:" :min 0 :max $maxadc :w 5 :valuex 100
      :menubutton ADC-SEL :label "select:" :strings $selectadcmenu
      :hskip 50
      :text ADCDATA :label "ADC Data:" :w 10 :value "$adcdata" :deaf T
      :button READADC    :label "read"
      :button READADCCLR :label "read & clear"
    
   :panel CLICK :layout T :padx 10
      :hskip 50
      :button RESET :label "Reset Module"


   :show T

}
