# LC4300B.tcl---LeCroy 4300B ADC module control frame
#   see EDOC212


set frame "LC4300B"
set frametitle "LeCroy 4300B ADC Setup & Monitor"
set icontitle "LC4300B"

set mname "lc4300b"

set maxmodule 32
set maxadc 15

set maxmodule [inform camaccontrol inquire [set mname]count]
set modulelist {}
for {set i 1} {$i <= $maxmodule} {incr i} {lappend modulelist $i}

set module [first $modulelist]

set vsn "0x00"
set adcdata "0x0000"

set wrname(RESET)	reset
set wrname(TEST)        test

set wrname(PEDESTAL)	pedestal

set wrname(SVSN)	status.vsn
set wrname(SMODE)	status.mode
set wrname(SEPS)	status.eps
set wrname(SECE)	status.ece
set wrname(SEEN)	status.een
set wrname(SCPS)	status.cps
set wrname(SCCE)	status.cce
set wrname(SCSR)	status.csr
set wrname(SCLE)	status.cle
set wrname(SOFS)        status.ofs

set rrname(RESET)	reset
set rrname(TEST)        test

set rrname(PEDESTAL)	pedestal
set rrname(READADC)	data

set rrname(SVSN)	status.vsn
set rrname(SMODE)	status.mode
set rrname(SEPS)	status.eps
set rrname(SECE)	status.ece
set rrname(SEEN)	status.een
set rrname(SCPS)	status.cps
set rrname(SCCE)	status.cce
set rrname(SCSR)	status.csr
set rrname(SCLE)	status.cle
set rrname(SOFS)	status.ofs

for {set sid [array startsearch wrname]} {[array anymore wrname $sid]} { } {
	set element [array nextelement wrname $sid]
	set wname($wrname($element)) $element
}
for {set sid [array startsearch rrname]} {[array anymore rrname $sid]} { } {
	set element [array nextelement rrname $sid]
	set wname($rrname($element)) $element
}

set wname(data) ADCDATA

loadsource common.camac

proc redisplay-module {} {
   global debug
   if {$debug} then {puts stderr "redisplay-module with arguments"}
   camac-address
   set register status.vsn
   update-text-widget $register [read-reg $register]
   set register status.eps
   update-widget $register [read-reg $register]
   set register status.ece
   update-widget $register [read-reg $register]
   set register status.een
   update-widget $register [read-reg $register]
   set register status.cps
   update-widget $register [read-reg $register]
   set register status.cce
   update-widget $register [read-reg $register]
   set register status.csr
   update-widget $register [read-reg $register]
   set register status.cle
   update-widget $register [read-reg $register]
   set register status.ofs
   update-widget $register [read-reg $register]
}

proc redisplay-adc {} {
   global debug
   if {$debug} then {puts stderr "redisplay-adc"}
   set register pedestal
   set value [read-adc-reg $register]
   update-widget $register [expr $value]
}

proc click arg {
   global debug
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$f=="ER"} then {cleanup-frame; return}
   if {$c==":quit"} then {click-quit; return}
   switch $w {
      ADC-SEL      -
      ADC-SEL-MENU -
      ADC-NUM {click-select-adc $v}

      MODULE-SEL      -
      MODULE-SEL-MENU -
      MODULE-NUM {click-select-module $v}

      ALL-MODULES {click-all-modules $v}
      ALL-ADCS    {click-all-adcs $v}

      PEDESTAL	{click-adc-slider $w $v}
      READADC   {click-read-adc $w}

      SVSN      {click-text $w $v}

      SEPS	-
      SECE      -
      SEEN	-
      SCPS	-
      SCCE	-
      SCSR	-
      SCLE	-
      SOFS      {click-box $w $v}

      RESET 	-
      TEST 	{click-action $w}

      REDISPLAY	{click-redisplay}
      default   {puts stdout $arg}
      }
   }


set selectadcmenu {0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15}

proc make-frame {} {
  global env
  global width maxmodule maxadc camcrate camstation vsn adcdata
  global modulelist selectadcmenu

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/camac_module" \
      :icon-name "[self icon-title]" :resize F

  :panel MODULE  :layout T :padx 10 :pady 10
     :layout H; :row-gap 10

     :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5
     :menubutton MODULE-SEL :label "select:" :strings $modulelist
     :hskip 50
     :text CAM-CRATE   :label "CAMAC Crate:"   :w 5 :value $camcrate :deaf T
     :text CAM-STATION :label "CAMAC Station:" :w 5 :value $camstation :deaf T

     :next-row T
     :message M1 :label "Status Register:" :bold T 

     :row-gap 5; :next-row T
     :hskip 40
     :text SVSN :label VSN: :w 5 :value $vsn
     :hskip 300
     :checkbox ALL-MODULES :label "Act on ALL modules?"

     :next-row T; :col-gap 40
     :hskip 30
     :checkbox SEPS  :label EPS:
     :checkbox SECE  :label ECE:
     :checkbox SEEN  :label EEN:
     :checkbox SCPS  :label CPS:
     :checkbox SCCE  :label CCE:
     :checkbox SCSR  :label CSR:

     :next-row T
     :hskip 30
     :checkbox SCLE  :label CLE:
     :checkbox SOFS  :label OFS:
  
   :panel ADC :layout T :padx 10
     :layout H; :row-gap 10
     :hskip 10

     :number ADC-NUM :label "ADC number:" :min 0 :max $maxadc :w 5
     :menubutton ADC-SEL :label "select:" :strings $selectadcmenu
     :hskip 200
     :checkbox ALL-ADCS :label "Act on ALL adcs?"

     :row-gap 10; :next-row T
     :slider PEDESTAL :label "Pedestal Memory:" :min 0 :max 255 :valuex 250

     :row-gap 5; :next-row T
     :hskip 100
     :text ADCDATA :label "ADC Data:" :w 10 :value "$adcdata"  :deaf T
     :button READADC :label read
  
  :panel CLICK :layout T :padx 10
     :layout H; :col-gap 15
     :hskip 40; :vskip 10
     :button REDISPLAY :label "Redisplay"
     :hskip 100
     :button RESET     :label "Reset Module"
     :button TEST      :label "Test Module"


  :show T


}

