# LC4302.tcl---LeCroy 4302 FERA dual port memory control frame



set frame LC4302
set frametitle "LeCroy 4302 dual port memory"
set icontitle "LC4302"

set mname "lc4302"

set maxmodule [inform camaccontrol inquire [set mname]count]
set modulelist {}
for {set i 1} {$i <= $maxmodule} {incr i} {lappend modulelist $i}

set module [first $modulelist]

set maxadc 0
set nchannels 0

set memAddr 0
set memAddrInc 0

set rrname(DATA)	      data
set rrname(ADDR)	      addr
set rrname(ENABLE)	      enable
set rrname(CLEAR)        clear

set wrname(DATA)	      data
set wrname(ADDR)	      addr
set wrname(ENABLE)	      enable
set wrname(CLEAR)        clear


for {set sid [array startsearch wrname]} {[array anymore wrname $sid]} { } {
	set element [array nextelement wrname $sid]
	set wname($wrname($element)) $element
}
for {set sid [array startsearch rrname]} {[array anymore rrname $sid]} { } {
	set element [array nextelement rrname $sid]
	set wname($rrname($element)) $element
}

loadsource common.camac
set reset_cmd clear

proc repaint {} {
  global module
  :modify MODULE-NUM :value $module
}

proc redisplay-module {} {
   global debug
   global memAddr memAddrInc
   if {$debug} then {puts stderr "redisplay-module"}
   camac-address
   update-widget enable [read-reg enable]
   update-widget addr $memAddr
}

proc do_memRead args {
   global memAddr memAddrInc

  set words 64

#    first must enable the CAMAC port - or disable the ECL port
  write-reg enable 2
  update-widget enable [read-reg enable]

  write-reg addr $memAddr
  :modify MEMDISPLAY :strings "\n    Memory Offset $memAddr\n"

  for {set i 1} {$i <= $words} {incr i} {
      set data [string trimleft [read-reg data]]
      :modify MEMDISPLAY :strings "  $data"
      if {[expr ($i / 8) * 8] == $i} then {:modify MEMDISPLAY :strings "\n"}
  }

  if {$memAddrInc != 0} then {set memAddr [expr $memAddr + 64]}
  redisplay-module
}

proc click-enable {v} {
    global memAddr

#    set the memory address for data capture
    write-reg addr $memAddr

    write-reg enable $v
}


proc click arg {
   global debug wrname
   global memAddr memAddrInc
   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {

      MODULE-SEL-MENU -
      MODULE-SEL      -
      MODULE-NUM {click-select-module $v}

      ENABLE    {click-enable $v}

      INCR      {set memAddrInc $v}
      ADDR      {set memAddr $v}

      EMPTY     {:modify MEMDISPLAY :empty T}

      default   {puts stdout $arg}
      }
   click-redisplay
}


proc make-frame {} {
  global env maxmodule modulelist camcrate camstation 
  global selectmodulemenu
  global memAddr memAddrInc

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/camac_module" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE :layout T :padx 10
      :layout H; :row-gap 10; :col-gap 10
      :vskip 10

      :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5
      :menubutton MODULE-SEL :label "select:" :strings $modulelist
      :hskip 50
      :text CAM-CRATE   :label "CAMAC Crate:"    :w 5 :value $camcrate   :deaf T
      :text CAM-STATION :label "CAMAC Station:"  :w 5 :value $camstation :deaf T

      :next-row T
      :choice ENABLE :label "Port enable" :strings [list Disabled Reserved "CAMAC enabled" "ECL enabled"] :w 16

      :next-row T
      :text ADDR :label "Memory address:" :w 8  :value $memAddr  :valuex 18c
      :button UPDATE  :label read  :command do_memRead
      :checkbox INCR :label "increment address" :value $memAddrInc
      :button EMPTY     :label "empty memory display"

      :next-row T
      :galley MEMDISPLAY :rows 15 :cols 80 :font fixed

   :show T

}

#proc EG args {puts stdout "$args"; return 7}
