# LC4434.tcl---LeCroy 4434 Scaler module control frame
#   see DOC227


set frame LC4434
set frametitle "LeCroy 4434 Scaler Setup & Monitor"
set icontitle "LC4434"

set mname "lc4434"

set maxmodule 8
set maxadc 0
set nchannels 32



set wrname(TESTLAM)     testlam
set wrname(TESTCLRLAM)	test+clrlam

set wrname(FA)          command.FA
set wrname(RN)          command.RN
set wrname(T)           command.T
set wrname(LD)          command.LD
set wrname(CL)          command.CL
set wrname(RD)          command.RD

set rrname(RESET)	command
set rrname(TESTLAM)     testlam
set rrname(TESTCLRLAM)	test+clrlam

set rrname(READDATA)     data
set rrname(READDATA++)   data++


for {set sid [array startsearch wrname]} {[array anymore wrname $sid]} { } {
	set element [array nextelement wrname $sid]
	set wname($wrname($element)) $element
}
for {set sid [array startsearch rrname]} {[array anymore rrname $sid]} { } {
	set element [array nextelement rrname $sid]
	set wname($rrname($element)) $element
}

loadsource common.camac
set reset_cmd command

proc repaint {} {
  global module
  :modify MODULE-NUM :value $module
}

set scaler(0) 0

proc redisplay-channels {} {
   global debug channel nchannels scaler
   if {$debug} then {puts stderr "redisplay-channels"}
   global EGreport
   set saveEGreport $EGreport
   set EGreport -1
#   write-reg command.LD 1
#   write-reg command.RD 1
#   write-reg command.FA 0
#   write-reg command.RN [expr $nchannels - 1]
   write-reg command 0x1fa0
   set register data++
   for {set i 0} {$i < $nchannels} {incr i} {
      set channel $i
      set value [read-reg $register]
      set scaler($i) [expr $value]
      :modify C[set i]DATA  :value $scaler($i)
   }
   write-reg command.RD 0
   set EGreport $saveEGreport
   set-footer "scaler display [date]"
}

set rateperiod 2
proc show-rates {} {
    global scaler nchannels rateperiod
    set-footer "reading scalers"
    redisplay-channels
    for {set i 0} {$i < $nchannels} {incr i} {set saved($i) $scaler($i)}
    set-footer "waiting for $rateperiod seconds"
    after [expr $rateperiod * 1000]
    set-footer "rereading scalers"
    redisplay-channels
    for {set i 0} {$i < $nchannels} {incr i} {
        set rate [expr [expr $scaler($i) - $saved($i)] / $rateperiod]
        :modify C[set i]DATA  :value $rate
    }
    set-footer "rate display (/sec) [date]"
}

proc redisplay-module {} {
   camac-address
   :modify LAM :value [string range [read-reg testlam] 1 1]
}

proc click-increment {} {
   write-reg command.T 1
   write-reg command.T 0
}

proc click arg {
   global debug rrname
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch $w {

      MODULE-SEL-MENU -
      MODULE-SEL      -
      MODULE-NUM {click-select-module $v}

      CLEAR      {write-reg command.CL 1; click-redisplay}
      CLRLAM     {read-reg test+clrlam; redisplay-module}
	
      INCREMENT  {click-increment; click-redisplay}
      
      default   {puts stdout $arg}
      }
}

set selectmodulemenu {1 2 3 4 5 6 7 8}


proc make-frame {} {
  global env maxmodule camcrate camstation nchannels
  global selectmodulemenu

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/camac_module" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE :layout T :padx 10
      :layout H; :row-gap 10
      :vskip 10

      :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5
      :menubutton MODULE-SEL :label "select:" :strings $selectmodulemenu
      :hskip 50
      :text CAM-CRATE   :label "CAMAC Crate:"     :w 5 :value $camcrate   :deaf T
      :text CAM-STATION :label "CAMAC Station:"   :w 5 :value $camstation :deaf T

      :next-row T
      :message M1 :label "Channel values"  :bold T

      for {set i 0} {$i < $nchannels} {incr i} {
        if {[expr $i/6*6] == $i} then {:next-row T}
        :text C[set i]DATA :label $i: :w 10 :value 0 :deaf T :valuex 40
      }

      :next-row T; :col-gap 25
      :hskip 25
      :button READDATA :label "Read Scalers" :command redisplay-channels
      :button CLEAR    :label "Clear Scalers" 
#      :button INCREMENT :label "Increment Scalers"
      :button RATES  :label "Scaler rates" :command show-rates

      :hskip 50
      :checkbox LAM  :label "LAM present"  :deaf T
      :button CLRLAM :label "Clear LAM"

      :next-row T
      :hskip 50
      :button REDISPLAY :label "Redisplay" :command click-redisplay

   :show T

}

