# LC4504.tcl---LeCroy 4504 Scaler module control frame
#   see DOC231


set frame LC4504
set frametitle "LeCroy 4504 Flash ADC Setup & Monitor"
set icontitle "LC4504"

set mname "lc4504"

set maxmodule 8
set maxadc 0
set nchannels 4


set wrname(VL)          VL
set wrname(VH)          VH
set wrname(C)           C

set rrname(ADC0)	adc0.data
set rrname(ADC1)        adc1.data
set rrname(ADC2)	adc2.data
set rrname(ADC3)        adc3.data

set rrname(OFLOW0)	adc0.oflow
set rrname(OFLOW1)      adc1.oflow
set rrname(OFLOW2)	adc2.oflow
set rrname(OFLOW3)      adc3.oflow

set rrname(C)           C


for {set sid [array startsearch wrname]} {[array anymore wrname $sid]} { } {
	set element [array nextelement wrname $sid]
	set wname($wrname($element)) $element
}
for {set sid [array startsearch rrname]} {[array anymore rrname $sid]} { } {
	set element [array nextelement rrname $sid]
	set wname($rrname($element)) $element
}

loadsource common.camac
set reset_cmd C

proc repaint {} {
  global module
  :modify MODULE-NUM :value $module
}

proc redisplay-module {} {
   camac-address
}

proc redisplay-channels {} {
   global debug channel nchannels
   if {$debug} then {puts stderr "redisplay-channels"}

   for {set i 0} {$i < $nchannels} {incr i} {
      set register adc[set i].data
      update-widget $register [read-reg $register]
      set register adc[set i].oflow
      update-widget $register [read-reg $register]
   }
}

proc click arg {
   global debug rrname
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$f=="ER"} then {cleanup-frame; return}
   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {

      MODULE-SEL-MENU -
      MODULE-SEL      -
      MODULE-NUM {click-select-module $v}

      VH    -
      VL    {click-write-data $w $v}

      C    {read-reg $rrname($w)}

      REDISPLAY	{click-redisplay}
      default   {puts stdout $arg}
      }
}

set selectmodulemenu {1 2 3 4 5 6 7 8}


proc make-frame {} {
  global env maxmodule camcrate camstation nchannels
  global selectmodulemenu

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/camac_module" \
      :icon-name "[self icon-title]"  :resize F

   :panel MODULE :layout T :padx 10
      :layout H; :row-gap 10
      :vskip 10

      :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5
      :menubutton MODULE-SEL :label "select:" :strings $selectmodulemenu
      :hskip 50
      :text CAM-CRATE   :label "CAMAC Crate:"    :w 5 :value $camcrate   :deaf T
      :text CAM-STATION :label "CAMAC Station:"  :w 5 :value $camstation :deaf T

      :next-row T
      :message M :label "Channel values"  :bold T

      for {set i 0} {$i < $nchannels} {incr i} {
        if {[expr $i/2*2] == $i} then {set j 0; :next-row T} else {:hskip 50}
         :message M$i :label "Channel $i:" :bold T
         :text ADC$i :label adc :w 4 :value 0 :deaf T 
         :checkbox OFLOW$i :label "overflow" :value 0 :deaf T
         incr j
      }

      :next-row T
      :slider VL :label "Low Reference Voltage:"  :min 0 :max 255  :valuex 150
      :next-row T
      :slider VH :label "High Reference Voltage:" :min 0 :max 255  :valuex 150

      :hskip 100
      :button C  :label "Module C"

      :next-row T
      :hskip 50
      :button REDISPLAY  :label "Redisplay"

   :show T

}
