# LC4516.tcl---LeCroy 4516 Programmable Logic Unit control frame
#   see DOC232


set frame LC4516
set frametitle "LeCroy 4516 Programmable Logic Unit Setup & Monitor"
set icontitle "LC4516"

set mname "lc4516"

set maxmodule 8
set maxadc 0
set nchannels 0

set rrname(C02AND)	C0toANDmode
set rrname(C02OR)       C0toORmode
set rrname(C12AND)	C1toANDmode
set rrname(C12OR)       C1toORmode
set rrname(C0INAND)	C0inANDmode
set rrname(C1INAND)     C1inANDmode

for {set sid [array startsearch rrname]} {[array anymore rrname $sid]} { } {
	set element [array nextelement rrname $sid]
	set wname($rrname($element)) $element
}

loadsource common.camac
set reset_cmd C0inANDmode

proc repaint {} {
  global module
  :modify MODULE-NUM :value $module
}

proc redisplay-module {} {
   global debug
   camac-address
}

proc redisplay-channels {} {
   global debug
   if {$debug} then {puts stderr "redisplay-channels"}
   if {[read-reg C0inANDmode] == "%11"} then {set vl 1} else {set vl 0}
   update-widget C0inANDmode $vl
   if {[read-reg C1inANDmode] == "%11"} then {set vl 1} else {set vl 0}
   update-widget C1inANDmode $vl

}

proc click arg {
   global debug rrname
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$f=="ER"} then {cleanup-frame; return}
   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {

      MODULE-SEL-MENU -
      MODULE-SEL      -
      MODULE-NUM {click-select-module $v}

      C*2AND  -
      C*2OR   {read-reg $rrname($w); redisplay-channels}

      C*INAND   {set register $rrname($w); update-widget $register [read-reg $register]}

      REDISPLAY	{click-redisplay}
      default   {puts stdout $arg}
      }
}

set selectmodulemenu {1 2 3 4 5 6 7 8}

proc make-frame {} {
  global env maxmodule camcrate camstation nchannels
  global selectmodulemenu

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/camac_module" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE :layout T :padx 10
      :layout H; :row-gap 10
      :vskip 10

      :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5
      :menubutton MODULE-SEL :label "select:" :strings $selectmodulemenu
      :hskip 100
      :text CAM-CRATE   :label "CAMAC Crate:"     :w 5 :value $camcrate   :deaf T
      :text CAM-STATION :label "CAMAC Station:"   :w 5 :value $camstation :deaf T

      :next-row T
      :button C02AND :label "set all C0s to AND mode"
      :button C02OR  :label "set all C0s to OR mode"
      :button C12AND :label "set all C1s to AND mode"
      :button C12OR  :label "set all C1s to OR mode"

      :next-row T
      :checkbox C0INAND :label "C0 switch in AND mode"
      :hskip 150
      :checkbox C1INAND :label "C1 switch in AND mode"

      :next-row T
      :hskip 50
      :button REDISPLAY     :label "Redisplay"

   :show T

}
