# LC4518.tcl---LeCroy 4518 Programmable Logic Delay module control frame
#   see EDOC2**


set frame LC4518
set frametitle "LeCroy 4518 Setup & Monitor"
set icontitle "LC4518"

set mname "lc4518"

set maxmodule 32
set maxadc 0
set maxchan 15

set chandata "0x0000"

set nchannels 16
for {set i 0} {$i < $nchannels} {incr i} {
set wrname(DELAY$i)	delay
}

for {set i 0} {$i < $nchannels} {incr i} {set wname(delay$i) DELAY$i}

loadsource common.camac

set reset_cmd chan0.delay
set wname(reset) $reset_cmd

set chan 0
 
proc repaint {} {
   global env module
  if {$env(TCLWM) == "xv"} then {
    xv "(MODULE-NUM :value $module)
  } else {
    :modify MODULE-NUM :value $module
  }
}

proc redisplay-module {} {
   global debug allmodules allchans
   if {$debug} then {puts stderr "redisplay-module with arguments"}
   camac-address
   :modify ALL-MODULES :value $allmodules
   :modify ALL-CHANS :value $allchans
}

proc redisplay-channels {} {
   global debug maxchan chan
   if {$debug} then {puts stderr "redisplay-channels"}

   set savechan $chan
   for {set chan 0} {$chan <= $maxchan} {incr chan} {
      update-widget delay$chan [read-chan-reg delay]
   }
   set chan $savechan
}

proc click-set-chan {w v} {
   global debug rrname wrname allchans
   if {$debug} then {puts stderr "click-set-chan with arguments \{$w\} \{$v\}"}
   clear-footer
   set register $wrname($w)
   write-chan-reg $register $v
   if {$allchans} then {redisplay-channels}
}

#    define dummy proc to replace version in common.camac
proc click-select-adc {i} {}

proc click arg {
   global debug chan allchans
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$f=="ER"} then {cleanup-frame; return}
   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {

      MODULE-SEL-MENU -
      MODULE-SEL -
      MODULE-NUM {click-select-module $v}

      ALL-MODULES  {click-all-modules $v}
      ALL-CHANS    {click-all-chans $v}

      DELAY*   {
        set chan [string trimleft $w DELAY]
        click-set-chan $w $v
      }

      REDISPLAY	{click-redisplay}
      default   {puts stdout $arg}
   }
}

set selectmodulemenu {1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16}
set selectchanmenu {0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15}


if {$env(TCLWM) == "xv"} then {

#    xv version

set width 700
set gap 100
set col1 150
set col2 [expr $col1+$gap]
set col3  [expr $col2+$gap]
set col4  [expr $col3+$gap]
set col5  [expr $col4+$gap]
set col6  [expr $col5+$gap]


set col10 250

proc make-frame {} {
  global width maxmodule maxchan camcrate camstation vsn chandata
  global selectmodulemenu selectchanmenu
  global col1 col2 col3 col4 col5 col6 col10

 xv "(:frame [self frame-name] :label '[self frame-title]'
      :shrink-wrap T :show-footer T  :resize F
      :icon (:icon :image eg-array :transparent T :label '[self icon-title]')
   (:panel MODULE  :x 0 :y 0 :layout H :w $width :h 160 :row-gap 10
      (:number MODULE-NUM  :y 20 :label 'Module number:' :min 1 :max $maxmodule :w 5 :valuex 150)
      (:button MODULE-SEL :label 'select:'
          :menu (:menu MODULE-SEL-MENU :strings ([quote-list $selectmodulemenu]) :rows 4 :cols 4))
      (:text CAM-CRATE :label 'CAMAC Crate:' :x 350 :w 5 :value '$camcrate' :deaf T)
      (:text CAM-STATION :label 'CAMAC Station:' :w 5 :value '$camstation' :deaf T)
      :next-row T
      :row-gap 5
      :next-row T
      (:checkbox ALL-MODULES :label 'Act on ALL modules?' :valuex $col6)
   )
   (:panel ADC :layout H :w $width :h 130 :row-gap  10
      (:number ADC-NUM :label 'ADC number:' :min 0 :max $maxchan :w 5 :valuex 150)
      (:button ADC-SEL :label 'select:'
          :menu (:menu ADC-SEL-MENU :strings ([quote-list $selectchanmenu]) :rows 4 :cols 4))
      :row-gap 10
      :next-row T
      (:slider PEDESTAL :label 'Pedestal Memory:' :min 0 :max 255 :valuex $col10)
      :row-gap 5
      :next-row T
      (:text ADCDATA :label 'ADC Data:' :w 10 :value '$chandata' :valuex $col10 :deaf T)
      (:button READADC :label 'read')
      :next-row T
      (:checkbox ALL-ADCS :label 'Act on ALL chans?' :valuex $col6)
   )
   (:panel CLICK :layout H :w $width :h 40 :row-gap 15 :col-gap 15
      :next-row T
      (:button TEST :label 'Test Module')
      (:button REDISPLAY :label 'Redisplay' :valuex $col4)
   )
)"

}

}  else {

#    tk version


proc make-frame {} {
  global env
  global width maxmodule maxchan camcrate camstation vsn chandata
  global selectmodulemenu selectchanmenu

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/camac_module" \
      :icon-name "[self icon-title]" :resize F

  :panel MODULE  :layout T :padx 10
     :layout H; :row-gap 10; :col-gap 20
     :vskip 10

     :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5 
     :menubutton MODULE-SEL :label "select:" :strings $selectmodulemenu :rows 4 :cols 4
     :hskip 50
     :text CAM-CRATE   :label "CAMAC Crate:"   :w 5 :value $camcrate :deaf T
     :text CAM-STATION :label "CAMAC Station:" :w 5 :value $camstation :deaf T

     :row-gap 5; :next-row T
     :hskip 400
     :checkbox ALL-MODULES :label "Act on ALL modules?"
     :next-row T
     :hskip 400
     :checkbox ALL-CHANS   :label "Act on ALL channels?"

   :panel CHAN1 :layout L :padx 20
     :layout V; :row-gap 5
     for {set i 0} {$i < 8} {incr i} {
       :slider DELAY$i :label "Delay Channel $i :" :min 0 :max 15
     }

   :panel CHAN2 :layout L :padx 20

     :layout V; :row-gap 5
     for {set i 8} {$i < 16} {incr i} {
       :slider DELAY$i :label "Delay Channel $i :" :min 0 :max 15
     }
  
  :panel CLICK :layout B :after MODULE
     :hskip 40; :vskip 10
     :button REDISPLAY :label "Redisplay"

  :show T


}

}
