# MILANDELAY.tcl---Milano CAMAC Delay module control frame
# J. Sampson, Thu Mar 14 10:22:57 1996.

set frame milandelay
set frametitle "Milano Delay Module: Setup"
set icontitle "MilanDelay"

set mname "milandelay"

set maxmodule 6
set maxadc 0
set maxchan 15

set selectmodulemenu {}
for {set i 1} {$i <= $maxmodule} {incr i} {
    lappend selectmodulemenu $i
}

for {set i 0} {$i <= $maxchan} {incr i} {
    set wrname(CDELAY$i) cdelay
    set wrname(FDELAY$i) fdelay
    set wrname(WIDTH$i)  width
}

for {set i 0} {$i < 8} {incr i} {
    set cwrangename(CRANGE$i)   range0
}
for {set i 8} {$i <= $maxchan} {incr i} {
    set cwrangename(CRANGE$i)   range1
}

for {set sid [array startsearch wrname]} {[array anymore wrname $sid]} { } {
	set element [array nextelement wrname $sid]
	set wname($wrname($element)) $element
}
for {set sid [array startsearch cwrangename]} {[array anymore cwrangename $sid]} { } {
	set element [array nextelement cwrangename $sid]
	set wname($cwrangename($element)) $element
}

set rrname(RESET)  reset

loadsource common.camac
set reset_cmd enable
 
proc repaint {} {
   global module
   :modify MODULE-NUM :value $module
}

proc redisplay-module {} {
   global debug
   if {$debug} then {puts stderr "redisplay-module"}
   camac-address
}

proc redisplay-channels {} {
   global debug
   if {$debug} then {puts stderr "redisplay-channels"}
   redisplay-cdelays
   redisplay-range
   redisplay-fdelays
   redisplay-widths
}

proc do-loadRAM {} {
   global resource moduleprefix module mname
   EG  "register :pattern '/$resource/$mname$moduleprefix$module.loadRAM*' :value '0'"
}

proc redisplay-cdelays {} {
   global debug maxchan chan
   if {$debug} then {puts stderr "redisplay-cdelays"}

     for {set chan 0} {$chan <= $maxchan} {incr chan} {
        set vl [expr ~[read-chan-reg cdelay] & 255]
#    convert to physical value
        set vl [expr $vl * 4]
        :modify cdelay$chan :value $vl
     }
 }

proc redisplay-fdelays {} {
   global debug maxchan chan
   if {$debug} then {puts stderr "redisplay-fdelays"}

     for {set chan 0} {$chan <= $maxchan} {incr chan} {
        set vl [expr [read-chan-reg fdelay] & 255]
#    convert to physical value
        set vl [expr $vl * 32]
        :modify fdelay$chan :value $vl
     }
 }

proc redisplay-widths {} {
   global debug maxchan chan
   if {$debug} then {puts stderr "redisplay-widths"}

     for {set chan 0} {$chan <= $maxchan} {incr chan} {
        set vl [expr [read-chan-reg width] & 255]
#    convert to physical value
        set vl [widthtophys $vl]
        :modify width$chan :value $vl
     }
 }

proc redisplay-range {} {
    global debug maxchan
    set r0 [expr (~[read-reg range0])&0xff]
    set r1 [expr (~[read-reg range1])&0xff]
    set r [expr ($r1 << 8) | ($r0 & 0xff)]
    for {set i 0} {$i <= $maxchan} {incr i} {
	:modify crange$i :value [expr ($r >> $i) & 1]
    }
}


#    setting course delay
proc click-set-cchan {w v} {
   global debug wrname allchans
   if {$debug} then {puts stderr "click-set-cchan with arguments \{$w\} \{$v\}"}
   clear-footer
   set register $wrname($w)
#    convert physical units to hardware value
   set vl [expr $v/4]
#    data inverted (8 bits)
   set vl [expr 255 - $vl]
   write-chan-reg $register $vl
   do-loadRAM 
   if {$allchans} {redisplay-cdelays}
}


#    set range
proc click-set-crange {w v} {
    global debug allchans chan
    clear-footer
    if {$chan > 15} {return}

#   Inverted data; bitvalue 0=10uS; 1= 1uS

    if {$allchans} {
	if {$v} {set wval 0} {set wval 0xff}
	write-reg range0 $wval
	write-reg range1 $wval
    } else {
	if {$chan < 8} then {
	    set register range0
	} else {
	    set register range1
	}
	set wval [read-reg $register]
	set bit [expr 1<<($chan&7)]
	if {!$v} then {set wval  [expr $wval  | $bit]}  \
	else          {set wval  [expr $wval  & ~$bit]}
	write-reg $register $wval
    }
    do-loadRAM 
    if {$allchans} {redisplay-range}
}


#   set fine delay
proc click-set-fchan {w v} {
   global debug wrname allchans
   if {$debug} then {puts stderr "click-set-fchan with arguments \{$w\} \{$v\}"}
   clear-footer
   set register $wrname($w)
#    convert physical units to hardware value
   set vl [expr $v/32]
#    data normal (8 bits)
   write-chan-reg $register $vl
   do-loadRAM 
   if {$allchans} {redisplay-fdelays}
}


#   set width
proc click-set-width {w v} {
   global debug wrname allchans
   if {$debug} then {puts stderr "click-set-width with arguments \{$w\} \{$v\}"}
   clear-footer
   set register $wrname($w)
#    convert physical units to hardware value
   set vl [widthtohrdwre $v]
#    data normal (8 bits)
   write-chan-reg $register $vl
   do-loadRAM
   if {$allchans} {redisplay-widths}
}

proc widthtohrdwre {v} {
   if {$v <= 125} then {return [expr [expr $v - 100] * 64 / 25 + 0]}
   if {$v <= 150} then {return [expr [expr $v - 125] * 36 / 25 + 64]}
   if {$v <= 200} then {return [expr [expr $v - 150] * 50 / 50 + 100]}
   if {$v <= 330} then {return [expr [expr $v - 200] * 25 / 130 + 150]}
   if {$v <= 550} then {return [expr [expr $v - 330] * 25 / 170 + 175]}
   if {$v <= 800} then {return [expr [expr $v - 500] * 25 / 300 + 200]}
}

proc widthtophys {v} {
   if {$v <= 64}  then {return [expr [expr $v - 0]   * 25 / 64 + 100]}
   if {$v <= 100} then {return [expr [expr $v - 64]  * 25 / 36 + 125]}
   if {$v <= 150} then {return [expr [expr $v - 100] * 50 / 50 + 150]}
   if {$v <= 175} then {return [expr [expr $v - 150] * 130 / 25 + 200]}
   if {$v <= 200} then {return [expr [expr $v - 175] * 170 / 25 + 330]}
   if {$v <= 225} then {return [expr [expr $v - 200] * 300 / 25 + 500]}
   return 800
}

proc click-loadram {} {
    global allmodules
    set temp $allmodules
    set allmodules 1
    write-regs loadRAM* 0
    set allmodules $temp
}


#    define dummy proc to replace version in common.camac
proc click-select-adc {i} {}

proc click arg {
   global debug chan
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$f=="ER"} then {cleanup-frame; return}
   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {

      MODULE-SEL-MENU -
      MODULE-SEL -
      MODULE-NUM {click-select-module $v; redisplay-channels}

      ALL-MODULES  {click-all-modules $v}
      ALL-CHANS    {click-all-chans $v}

      CDELAY*   {
        set chan [string trimleft $w CDELAY]
        click-set-cchan $w $v
      }

      CRANGE*   {
        set chan [string trimleft $w CRANGE]
        click-set-crange $w $v
      }

      FDELAY*   {
        set chan [string trimleft $w FDELAY]
        click-set-fchan $w $v
      }

      WIDTH*   {
        set chan [string trimleft $w WIDTH]
        click-set-width $w $v
      }

      REDISPLAY	{click-redisplay}
      LOADRAM   {click-loadram}
      default   {puts stdout $arg}
   }
}

proc make-frame {} {
  global env
  global maxmodule maxchan camcrate camstation 
  global selectmodulemenu 

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/camac_module" \
      :icon-name "[self icon-title]" :resize F

  :panel MODULE  :layout T :padx 10 :pady 10 
     :layout H; :row-gap 10; :col-gap 20

     :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5 
     :menubutton MODULE-SEL :label "select:" :strings $selectmodulemenu 
     :hskip 50
     :text CAM-CRATE   :label "CAMAC Crate:"   :w 5 :value $camcrate :deaf T
     :text CAM-STATION :label "CAMAC Station:" :w 5 :value $camstation :deaf T

     :row-gap 5; :next-row T
     :checkbox ALL-MODULES :label "Act on ALL modules?"  :valuex 500
     :next-row T
     :checkbox ALL-CHANS   :label "Act on ALL channels?" :valuex 500

   :panel CHAN1 :layout L :padx 10
     :row-gap 5
     :layout V
     :message CDELAY-HEADER   :label "    Coarse Delay (nsec)"  :bold T

     for {set i 0} {$i < 16} {incr i} {
     :slider CDELAY$i :label "$i :" :min 0 :max 1023 :valuex 30 :resolution 4
     }

   :panel CHAN1CHECK :layout L :padx 10
     :row-gap 5
     :layout V
     :message CDELAY-CHECKHEADER   :label "*10" :bold T

     :row-gap 8
     for {set i 0} {$i < 16} {incr i} {
     :checkbox CRANGE$i :value T :label ""
     }

# Leave extra column space after coarse delay panels to make column space even.
   :panel CHAN2 :layout L :padx 10
     :row-gap 5
     :layout V
     :message FDELAY-HEADER   :label "    Fine Delay (psec)"  :bold T

     for {set i 0} {$i < 16} {incr i} {
     :slider FDELAY$i :label "" :min 0 :max 8192 :valuex 20 :resolution 32
     }

   :panel CHAN3 :layout L :padx 10
     :row-gap 5
     :layout V
     :message WIDTH-HEADER   :label "       Width (nsec)"  :bold T

     for {set i 0} {$i < 16} {incr i} {
     :slider WIDTH$i :label "" :min 100 :max 800 :valuex 20
     }
  
  :panel CLICK :layout B :padx 10 :after FOOTER
     :layout H; :col-gap 15
     :vskip 10
     :button REDISPLAY :label "Redisplay"
     :button LOADRAM   :label "LoadRAM - Apply current settings in all modules"

  :show T


}
