# ORTEC413.tcl---Ortec 413 ADC module control frame
#   see EDOC226


set frame "ORTEC413"
set frametitle "Ortec 413 ADC Setup & Monitor"
set icontitle "ORTEC413"

set mname "ot413"

set maxmodule 32
set maxadc 3

set maxmodule [inform camaccontrol inquire [set mname]count]
set modulelist {}
for {set i 1} {$i <= $maxmodule} {incr i} {lappend modulelist $i}

set module [first $modulelist]

set vsn "0x00"
set adcdata "0x0000"

set wrname(CLEAR)	clear

set wrname(LLD)		lld

set wrname(SVSN)	control.vsn
set wrname(SMODE)	control.mode
set wrname(SZSE)	control.zse
set wrname(SEEN)	control.een
set wrname(SCME)	control.cme
set wrname(SCSR)	control.csr
set wrname(SCLE)	control.cle
set wrname(SOFS)        control.ofs

set wrname(GATE1)	gate1.enable
set wrname(GATE2)	gate2.enable
set wrname(GATE3)	gate3.enable
set wrname(GATE4)	gate4.enable
set wrname(GATEM)	gatem.enable

set rrname(CLEAR)	clear

set rrname(LLD) 	lld
set rrname(READADC)	data

set rrname(SVSN)	control.vsn
set rrname(SMODE)	control.mode
set rrname(SZSE)	control.zse
set rrname(SEEN)	control.een
set rrname(SCME)	control.cme
set rrname(SCSR)	control.csr
set rrname(SCLE)	control.cle
set rrname(SOFS)	control.ofs

set rrname(GATE1)	gate1.enable
set rrname(GATE2)	gate2.enable
set rrname(GATE3)	gate3.enable
set rrname(GATE4)	gate4.enable
set rrname(GATEM)	gatem.enable


for {set sid [array startsearch wrname]} {[array anymore wrname $sid]} { } {
	set element [array nextelement wrname $sid]
	set wname($wrname($element)) $element
}
for {set sid [array startsearch rrname]} {[array anymore rrname $sid]} { } {
	set element [array nextelement rrname $sid]
	set wname($rrname($element)) $element
}

set wname(data) ADCDATA

loadsource common.camac
 
proc redisplay-module {} {
   global debug
   if {$debug} then {puts stderr "redisplay-module with arguments"}
   camac-address
   set register control.vsn
   update-text-widget $register [read-reg $register]
   set register control.zse
   update-widget $register [read-reg $register]
   set register control.een
   update-widget $register [read-reg $register]
   set register control.cme
   update-widget $register [read-reg $register]
   set register control.csr
   update-widget $register [read-reg $register]
   set register control.cle
   update-widget $register [read-reg $register]
   set register control.ofs
   update-widget $register [read-reg $register]
   set register gate1.enable
   update-widget $register [read-reg $register]
   set register gate2.enable
   update-widget $register [read-reg $register]
   set register gate3.enable
   update-widget $register [read-reg $register]
   set register gate4.enable
   update-widget $register [read-reg $register]
   set register gatem.enable
   update-widget $register [read-reg $register]
}

proc redisplay-adc {} {
   global debug
   if {$debug} then {puts stderr "redisplay-adc"}
   set register lld
   set value [read-adc-reg $register]
   update-widget $register [expr $value]
}

proc click arg {
   global debug
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {
      ADC-SEL  -
      ADC-NUM {click-select-adc $v}

      MODULE-SEL  -
      MODULE-NUM {click-select-module $v}

      ALL-MODULES {click-all-modules $v}
      ALL-ADCS    {click-all-adcs $v}

      LLD	{click-adc-slider $w $v}
      READADC   {click-read-adc $w}

      SVSN      {click-text $w $v}

      GATE*     -

      SZSE	-
      SEEN	-
      SCME	-
      SCSR	-
      SCLE	-
      SOFS      {click-box $w $v}

      CLEAR 	{click-action $w}


      REDISPLAY	{click-redisplay}
      default   {puts stdout $arg}
   }
}


proc make-frame {} {
  global env
  global maxmodule maxadc camcrate camstation vsn adcdata modulelist

set selectadcmenu {0 1 2 3}


  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/camac_module" \
      :icon-name "[self icon-title]" :resize F

  :panel MODULE  :layout T :padx 10 :pady 10
      :layout H; :row-gap 10

     :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5
     :menubutton MODULE-SEL :label "select:" :strings $modulelist 
     :hskip 50
     :text CAM-CRATE   :label "CAMAC Crate:"   :w 5 :value $camcrate :deaf T
     :text CAM-STATION :label "CAMAC Station:" :w 5 :value $camstation :deaf T

     :next-row T
     :message M1 :label "Control Register 1:" :bold T 

     :row-gap 5; :next-row T
     :hskip 40
     :text SVSN :label VSN: :w 5 :value $vsn
     :hskip 300
     :checkbox ALL-MODULES :label "Act on ALL modules?"

     :next-row T; :col-gap 40
     :hskip 30
     :checkbox SZSE  :label ZSE:
     :checkbox SEEN  :label EEN:
     :checkbox SCME  :label CME:
     :checkbox SCSR  :label CSR:
     :checkbox SCLE  :label CLE:
     :checkbox SOFS  :label OFS:

     :row-gap 10; :next-row T
     :message M2 :label "Control Register 2:" :bold T 

     :row-gap 0; :next-row T
     :checkbox GATE1  :label "Gate1:" 
     :checkbox GATE2  :label "Gate2:" 
     :checkbox GATE3  :label "Gate3:" 
     :checkbox GATE4  :label "Gate4:"
     :checkbox GATEM  :label "Master Gate:"
  
   :panel ADC :layout T :padx 10
      :layout H; :row-gap 10
      :hskip 10
   
     :number ADC-NUM :label "ADC number:" :min 0 :max $maxadc :w 5
     :menubutton ADC-SEL :label "select:" :strings $selectadcmenu
     :hskip 200
     :checkbox ALL-ADCS :label "Act on ALL adcs?" 

     :next-row T
     :slider LLD :label "Lower Level Discriminator:" :min 0 :max 255 :valuex 250

     :next-row T; :col-gap 10
     :hskip 100
     :text ADCDATA :label "ADC Data:" :w 10 :value "$adcdata" :deaf T
     :button READADC :label read

   :panel CLICK :layout T :padx 10
      :layout H; :col-gap 15
      :hskip 40; :vskip 10
   
     :button REDISPLAY :label "Redisplay"
     :hskip 100
     :button CLEAR     :label "Clear Module"

  :show T


}
