# QDC1612F.tcl---GANELEC QDC1612F ADC module control frame
#   see EDOC2**



set frame "QDC1612F"
set frametitle "GANELEC QDC1612F ADC Setup & Monitor"
set icontitle "QDC1612F"

set mname "qdc1612f"

set maxmodule 32
set maxadc 15

set vsn1 "0x00"
set vsn2 "0x00"
set adcdata "0x0000"

set wrname(RESET)	reset
set wrname(TEST)        test

set wrname(PEDESTAL)	pedestal

set wrname(SVSN)	vsn1
set wrname(SVSN1)	vsn1
set wrname(SVSN2)	vsn2
set wrname(SMODE)	status.mode
set wrname(SEPS)	status.eps
set wrname(SECE)	status.ece
set wrname(SEEN)	status.een
set wrname(SCPS)	status.cps
set wrname(SCCE)	status.cce
set wrname(SCSR)	status.csr
set wrname(SCLE)	status.cle
set wrname(SOFS)        status.ofs
set wrname(SINDIV)      status.indiv

set rrname(RESET)	reset
set rrname(TEST)        test

set rrname(PEDESTAL)	pedestal
set rrname(READADC)	data

set rrname(SVSN)	vsn1
set rrname(SVSN1)	vsn1
set rrname(SVSN2)	vsn2
set rrname(SMODE)	status.mode
set rrname(SEPS)	status.eps
set rrname(SECE)	status.ece
set rrname(SEEN)	status.een
set rrname(SCPS)	status.cps
set rrname(SCCE)	status.cce
set rrname(SCSR)	status.csr
set rrname(SCLE)	status.cle
set rrname(SOFS)	status.ofs
set rrname(SINDIV)      status.indiv



for {set sid [array startsearch wrname]} {[array anymore wrname $sid]} { } {
	set element [array nextelement wrname $sid]
	set wname($wrname($element)) $element
}
for {set sid [array startsearch rrname]} {[array anymore rrname $sid]} { } {
	set element [array nextelement rrname $sid]
	set wname($rrname($element)) $element
}

set wname(data)  ADCDATA

loadsource common.camac

set adcsarecalled  "channel"

proc redisplay-module {} {
   global debug env
   if {$debug} then {puts stderr "redisplay-module with arguments"}
   camac-address
   set register vsn1
   update-text-widget $register [read-reg $register]
   set register vsn2
   update-text-widget $register [read-reg $register]
   set register status.eps
   update-widget $register [read-reg $register]
   set register status.ece
   update-widget $register [read-reg $register]
   set register status.een
   update-widget $register [read-reg $register]
   set register status.cps
   update-widget $register [read-reg $register]
   set register status.cce
   update-widget $register [read-reg $register]
   set register status.csr
   update-widget $register [read-reg $register]
   set register status.cle
   update-widget $register [read-reg $register]
   set register status.ofs
   update-widget $register [read-reg $register]
}

proc redisplay-adc {} {
   global debug
   if {$debug} then {puts stderr "redisplay-adc with arguments"}
   set register pedestal
   set value [read-adc-reg $register]
   update-widget $register [expr $value]
}

proc click arg {
   global debug
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$f=="ER"} then {cleanup-frame; return}
   if {$c==":quit"} then {click-quit; return}
   switch $w {
      ADC-SEL       -
      ADC-SEL-MENU  -
      ADC-NUM       {click-select-adc $v}

      MODULE-SEL      -
      MODULE-SEL-MENU -
      MODULE-NUM      {click-select-module $v}

      ALL-MODULES {click-all-modules $v}
      ALL-ADCS    {click-all-adcs $v}

      PEDESTAL	{click-adc-slider $w $v}
      READADC   {click-read-adc $w}

      SVSN      -
      SVSN1     -
      SVSN2     {click-text $w $v}


      SEPS	-
      SECE      -
      SEEN	-
      SCPS	-
      SCCE	-
      SCSR	-
      SCLE	-
      SOFS      -
      SINDIV    {click-box $w $v}

      RESET 	-
      TEST 	{click-action $w}

      REDISPLAY	{click-redisplay}
      default   {puts stdout $arg}
      }
   }

set selectmodulemenu {1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16}
set selectadcmenu {0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15}


if {$env(TCLWM) == "xv"} then {

#    xv version

set width 710
set gap 100
set col1 150
set col2 [expr $col1+$gap]
set col3  [expr $col2+$gap]
set col4  [expr $col3+$gap]
set col5  [expr $col4+$gap]
set col6  [expr $col5+$gap]


set col10 250

proc make-frame {} {
  global width maxmodule maxadc camcrate camstation vsn1 vsn2 adcdata
  global selectmodulemenu selectadcmenu
  global col1 col2 col3 col4 col5 col6 col10

 xv "(:frame [self frame-name] :label '[self frame-title]'
      :shrink-wrap T :show-footer T  :resize F
      :icon (:icon :image eg-array :transparent T :label '[self icon-title]')
   (:panel MODULE  :x 0 :y 0 :layout H :w $width :h 180 :row-gap 10
      (:number MODULE-NUM  :y 20 :label 'Module number:' :min 1 :max $maxmodule :w 5 :valuex 150)
      (:button MODULE-SEL :label 'select:'
          :menu (:menu MODULE-SEL-MENU :strings ([quote-list $selectmodulemenu]) :rows 4 :cols 4))
      (:text CAM-CRATE :label 'CAMAC Crate:' :x 350 :w 5 :value '$camcrate' :deaf T)
      (:text CAM-STATION :label 'CAMAC Station:' :w 5 :value '$camstation' :deaf T)
      :next-row T
      (:message :label 'Status Register:' :bold T :valuex $col1)
      :row-gap 5
      :next-row T
      (:text SVSN1 :label 'VSN1:' :w 5 :value '$vsn1' :valuex $col1)
      (:text SVSN2 :label 'VSN2:' :w 5 :value '$vsn2' :valuex $col2)

      :next-row T
      (:checkbox SEPS  :label 'EPS:' :valuex $col1)
      (:checkbox SECE  :label 'ECE:' :valuex $col2)
      (:checkbox SEEN  :label 'EEN:' :valuex $col3)
      (:checkbox SCPS  :label 'CPS:' :valuex $col4)
      (:checkbox SCCE  :label 'CCE:' :valuex $col5)
      (:checkbox SCSR  :label 'CSR:' :valuex $col6)
      :next-row T
      (:checkbox SCLE  :label 'CLE:' :valuex $col1)
      (:checkbox SOFS  :label 'OFS:' :valuex $col2)
      (:checkbox SINDIV  :label 'INDIV:' :valuex $col3)

      :next-row T
      (:slider MASTERGATE :label 'Master Gate (us):' :min 1 :max 16)
      (:slider CPEDESTALC :label 'Common Pedestal Current:' :min 0 :max 255 :x 320)

      :row-gap 5
      :next-row T
      (:checkbox ALL-MODULES :label 'Act on ALL modules?' :valuex $col6)
   )
   (:panel ADC :layout H :w $width :h 100 :row-gap  10
      (:number ADC-NUM :label 'ADC number:' :min 0 :max $maxadc :w 5 :valuex 150)
      (:button ADC-SEL :label 'select:'
          :menu (:menu ADC-SEL-MENU :strings ([quote-list $selectadcmenu]) :rows 4 :cols 4))
      :row-gap 10
      :next-row T
      (:slider PEDESTAL :label 'Pedestal Memory:' :min 0 :max 255 :valuex $col10)
      :row-gap 5
      :next-row T
      (:text ADCDATA :label 'ADC Data:' :w 10 :value '$adcdata' :valuex $col10 :deaf T)
      (:button READADC :label 'read')
      :next-row T
      (:checkbox ALL-ADCS :label 'Act on ALL adcs?' :valuex $col6)
   )
   (:panel CLICK :layout H :w $width :h 40 :row-gap 15 :col-gap 15
      :next-row T
      (:button RESET :label 'Reset Module' :valuex $col1)
      (:button TEST :label 'Test Module')
      (:button REDISPLAY :label 'Redisplay' :valuex $col4)
   )
)"

}

}  else {

#    tk version

proc make-frame {} {
  global env
  global width maxmodule maxadc camcrate camstation vsn1 vsn2 adcdata
  global selectmodulemenu selectadcmenu
  global col1 col2 col3 col4 col5 col6 col10


  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/camac_module" \
      :icon-name "[self icon-title]" :resize F

  :panel MODULE  :layout T :padx 10 :pady 10
     :layout H; :row-gap 10

     :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5
     :menubutton MODULE-SEL :label "select:" :strings $selectmodulemenu :rows 4 :cols 4
     :hskip 50
     :text CAM-CRATE   :label "CAMAC Crate:"   :w 5 :value $camcrate :deaf T
     :text CAM-STATION :label "CAMAC Station:" :w 5 :value $camstation :deaf T

     :next-row T
     :message M1 :label "Status Register:" :bold T 

     :row-gap 5; :next-row T; :col-gap 40
     :hskip 40
     :text SVSN1 :label VSN1: :w 5 :value $vsn1
     :text SVSN2 :label VSN2: :w 5 :value $vsn2
     :hskip 120
     :checkbox ALL-MODULES :label "Act on ALL modules?"

     :next-row T
     :hskip 30
     :checkbox SEPS  :label EPS:
     :checkbox SECE  :label ECE:
     :checkbox SEEN  :label EEN:
     :checkbox SCPS  :label CPS:
     :checkbox SCCE  :label CCE:
     :checkbox SCSR  :label CSR:

     :next-row T
     :hskip 30
     :checkbox SCLE  :label CLE:
     :checkbox SOFS  :label OFS:
     :checkbox SINDIV :label INDIV:

     :next-row T
     :slider MASTERGATE :label "Master Gate (us):" :min 1 :max 16 :valuex 250
     :next-row T
     :slider CPEDESTALC :label "Common Pedestal Current:" :min 0 :max 255 :valuex 250
  
   :panel ADC :layout T :padx 10
      :layout H; :row-gap 10
      :hskip 10

     :number ADC-NUM :label "ADC number:" :min 0 :max $maxadc :w 5
     :menubutton ADC-SEL :label "select:" :strings $selectadcmenu :rows 4 :cols 4
     :hskip 200
     :checkbox ALL-ADCS :label "Act on ALL adcs?"

     :row-gap 10; :next-row T
     :slider PEDESTAL :label "Pedestal Memory:" :min 0 :max 255 :valuex 250

     :next-row T; :col-gap 10
     :hskip 100
     :text ADCDATA :label "ADC Data:" :w 10 :value "$adcdata" :deaf T
     :button READADC :label read
  
   :panel CLICK :layout T :padx 10
      :layout H; :col-gap 15
      :hskip 40; :vskip 10
     :button REDISPLAY :label "Redisplay"
     :hskip 100
     :button RESET     :label "Reset Module"
     :button TEST      :label "Test Module"

  :show T


}

}
