# RS232CC.tcl---RS232 Crate Controller module control frame


set debug 0
set started 0


set frame RS232CC
set frametitle "RS232 Crate Controller Setup & Monitor"
set icontitle "RS232CC"


if {[info exists env(CamacServer)] == 0} then {set resource camac}  \
   else {set resource $env(CamacServer)}

set cchoice {1 2 3 4 5 6 7}
set camcrate "1"
set mname "RS232CC#1"

proc define {c} {
    global debug resource mname
    set mname "RS232CC#$c"
    EG  "register :pattern '/$resource/RS232CC#$c.*' !delete"
    define-register RS232CC#$c xCAMAC Z               "-c$c -n30 -a0 -f1 -prw -w24 -l1 -b7 -zd"
    define-register RS232CC#$c xCAMAC C               "-c$c -n30 -a0 -f1 -prw -w24 -l1 -b6 -zd"
    define-register RS232CC#$c xCAMAC I               "-c$c -n30 -a0 -f1 -prw -w24 -l1 -b5 -zd"
    define-register RS232CC#$c xCAMAC status           "-c$c -n30 -a0 -f1 -prw -w24 -l0 -b0 -zx -q1"
    define-register RS232CC#$c xCAMAC Bmask            "-c$c -n30 -a1 -f16 -pwo -w24 -l0 -b0 -zx"
    define-register RS232CC#$c xCAMAC LAMmask          "-c$c -n30 -a13 -f17 -pwo -w24 -l0 -b0 -zx"
    define-register RS232CC#$c xCAMAC Estatus          "-c$c -n30 -a1 -f1 -pro -w24 -l0 -b0 -zx"
}

proc define-register {mname class rname attributes} {
   global debug resource
   set register /$resource/$mname.$rname
   set-footer "defining register $register"
   EG "register :name '$register' :class '$class' !new"
   EG "register :name '$register' :object '$attributes'"
   clear-footer
}

proc {no-QX} {module} {

   display-error-report self "no QX accessing $module - module probably missing or broken" \
        "check crate and station supplied in configuration are correct"
}

proc write-reg {r v} {
   global resource mname
   EG  "register :name '/$resource/$mname.$r' :value '$v'"
   }

proc read-reg {r} {
   global resource  mname
   EG "register :name '/$resource/$mname.$r' ?value"
   }

proc redisplay-status {} {
   global debug
   if {$debug} then {puts stderr "redisplay-status"}
   :modify I :value [read-reg I]
   :modify STATUS :value [read-reg status]
}

proc click-initialise {} {
   global EGallow
   set EGallow 0x2000a
#    set control register:  enable status byte;  set 14 bit mode;  remove inhibit
   write-reg status 0x000400
#    clear break mask register
   write-reg Bmask  0x000000
#    set all bits in the LAM mask register  - we do not use LAMs
   write-reg LAMmask 0xffffff
#    perform crate Z
   write-reg status 0x000480
   set EGallow 0xffffff
}

proc initialise {c} {define $c; click-initialise}

proc select-crate {v} {
   global debug cchoice camcrate
   set camcrate [lindex $cchoice $v]
   define $camcrate
}

proc click arg {
   global debug cnafdata camcrate
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch $w  {

      CAM-CRATE  {select-crate $v}

      Z          {write-reg Z 1}
      C          {write-reg C 1}
      I          {write-reg I $v}

      INITIALISE {click-initialise}
      STATUS     {click-status $v}

      DEFINE     {define $camcrate}
      REDISPLAY	 {}

      default   {puts stdout $arg}
   }
   redisplay-status
}


proc start args {
   global started camcrate
   if {$started == 0} then {make-frame} else {open-frame}
   set started 1
   define $camcrate
   redisplay-status
}

proc click-quit {}   {close-frame}
proc finish     args {destroy-frame}

proc make-frame {} {
  global env  cchoice

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/camac_module" \
      :icon-name "[self icon-title]" :resize F

  :panel MODULE  :layout T :padx 10 :pady 10
    :layout H; :row-gap 10; :col-gap 20

    :hskip 180
    :choice-stack CAM-CRATE :label "CAMAC Crate:" :exclusive T :strings $cchoice :value 0

  :panel CLICK :layout T :padx 30
    :layout H; :col-gap 10; :row-gap 10

    :button Z;    :button C;   :checkbox I
    :button INITIALISE :label "Initialise Controller"

    :next-row T
    :text STATUS :label "Control/Status register" :w 8

    :next-row T
    :button REDISPLAY    :label "Redisplay"
    :hskip 100
    :button DEFINE       :label "Define Registers"


   :show T
}
