
#    This file is the common part of the S4418 family control
#    for the file header see the S4418/V and S4418/T files

set maxmodule 32
set maxadc 7

set maxmodule [inform camaccontrol inquire [set mname]count]
set modulelist {}
for {set i 1} {$i <= $maxmodule} {incr i} {lappend modulelist $i}

set module [first $modulelist]

set vsn "0x00"
set adcdata "0x0000"
set hdrdata "0x0000"
set patdata "0x0000"

set wrname(RESET)	reset
set wrname(TEST)        test

set wrname(ULD)		uld
set wrname(LLD)		lld
set wrname(OFFSET)	offset

if {$mname == "s4418v"} then {
   set wrname(THRESHOLD)	threshold
}

set wrname(SVSN)	status.vsn
set wrname(SMODE)	status.mode
set wrname(SSUB)	status.sub
set wrname(SEEN)	status.een
set wrname(SOVF)	status.ovf
set wrname(SCCE)	status.cce
set wrname(SCSR)	status.csr
set wrname(SCLE)	status.cle

set rrname(RESET)	reset
set rrname(TEST)	test

set rrname(ULD)		uld
set rrname(LLD)		lld
set rrname(OFFSET)	offset
set rrname(READADC)     data
set rrname(READADCCLR)  data+clr

set rrname(READHDR)     hdr.data
set rrname(READHDRCLR)  hdr.data+clr
set rrname(READPAT)     patt.data
set rrname(READPATCLR)  patt.data+clr

if {$mname == "s4418v"} then {
   set rrname(THRESHOLD)	threshold
}

set rrname(SVSN)	status.vsn
set rrname(SMODE)	status.mode
set rrname(SSUB)	status.sub
set rrname(SEEN)	status.een
set rrname(SOVF)	status.ovf
set rrname(SCCE)	status.cce
set rrname(SCSR)	status.csr
set rrname(SCLE)	status.cle



for {set sid [array startsearch wrname]} {[array anymore wrname $sid]} { } {
	set element [array nextelement wrname $sid]
	set wname($wrname($element)) $element
}
for {set sid [array startsearch rrname]} {[array anymore rrname $sid]} { } {
	set element [array nextelement rrname $sid]
	set wname($rrname($element)) $element
}

set wname(data)          ADCDATA
set wname(data+clr)      ADCDATA
set wname(hdr.data)      HDRDATA
set wname(hdr.data+clr)  HDRDATA
set wname(patt.data)     PATDATA
set wname(patt.data+clr) PATDATA

loadsource common.camac

proc redisplay-module {} {
   global debug mname
   if {$debug} then {puts stderr "redisplay-module"}
   camac-address
   set register status.vsn
   update-text-widget $register [read-reg $register]
   set register status.sub
   update-widget $register [read-reg $register]
   set register status.een
   update-widget $register [read-reg $register]
   set register status.ovf
   update-widget $register [read-reg $register]
   set register status.cce
   update-widget $register [read-reg $register]
   set register status.csr
   update-widget $register [read-reg $register]
   set register status.cle
   update-widget $register [read-reg $register]
   if {$mname == "s4418v"} then {
      set register threshold
      update-widget $register [read-reg $register]
   }
}

proc redisplay-adc {} {
   global debug
   if {$debug} then {puts stderr "redisplay-adc"}
   set register uld
   set value [read-adc-reg $register]
   update-widget $register [expr $value]
   set register lld
   set value [read-adc-reg $register]
   update-widget $register [expr $value]
   set register offset
   set value [read-adc-reg $register]
   update-widget $register [expr $value]

}

proc click arg {
   global debug mname
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch $w {
      ADC-SEL-MENU -
      ADC-NUM      -
      ADC-SEL      {click-select-adc $v}

      MODULE-SEL-MENU -
      MODULE-NUM      -
      MODULE-SEL      {click-select-module $v}

      ALL-MODULES {click-all-modules $v}
      ALL-ADCS    {click-all-adcs $v}

      ULD	-
      LLD	-
      OFFSET	{click-adc-slider $w $v}

      READADC    -
      READADCCLR {click-read-adc $w}

      READHDR     -
      READHDRCLR  -
      READPAT     -
      READPATCLR  {click-read-data $w}

      THRESHOLD	  {click-slider $w $v}

      SVSN        {click-text $w $v}

      SSUB	-
      SEEN	-
      SOVF	-
      SCCE	-
      SCSR	-
      SCLE	{click-box $w $v}

      RESET 	-
      TEST 	{click-action $w}

      REDISPLAY	{click-redisplay}
      default   {puts stdout $args}
      }
   }


set width 700
set gap 100
set col1 150
set col2 [expr $col1+$gap]
set col3  [expr $col2+$gap]
set col4  [expr $col3+$gap]
set col5  [expr $col4+$gap]
set col6  [expr $col5+$gap]


set col10 250

set selectadcmenu {0 1 2 3 4 5 6 7}


proc make-frame {} {
  global env mname
  global width maxmodule maxadc camcrate camstation vsn hdrdata patdata adcdata
  global modulelist selectadcmenu

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/camac_module" \
      :icon-name "[self icon-title]" :resize F

  :panel MODULE  :layout T :padx 10 :pady 10
      :layout H; :row-gap 10

      :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5 
      :menubutton MODULE-SEL :label "select:" :strings $modulelist
      :hskip 50
      :text CAM-CRATE   :label "CAMAC Crate:"   :w 5 :value "00" :deaf T
      :text CAM-STATION :label "CAMAC Station:" :w 5 :value "000" :deaf T

      :next-row T
      :message MSGSR :label "Status Register:" :bold T 
      :row-gap 5
      :next-row T
      :hskip 40
      :text SVSN :label "VSN:" :w 5 :value "$vsn"
      :hskip 300
      :checkbox ALL-MODULES :label "Act on ALL modules?"

      :next-row T; :col-gap 40
      :hskip 30
      :checkbox SSUB  :label SUB:
      :checkbox SEEN  :label EEN: 
      :checkbox SOVF  :label OVF: 
      :checkbox SCCE  :label CCE:
      :checkbox SCSR  :label CSR: 
      :checkbox SCLE  :label CLE: 
      :col-gap 10

      if {$mname == "s4418v"} then {
         :row-gap 10; :next-row T
         :slider THRESHOLD :label "Common Threshold:" :min 0 :max 255 :valuex 35c
      }

      :row-gap 5; :next-row T
      :text HDRDATA :label "Event Header Word:" :w 10 :value $hdrdata :deaf T :valuex 35c
      :button READHDR  :label "read"
      :button READHDRCLR :label "read & clear"

      :next-row T
      :text PATDATA :label "Event Pattern Word:" :w 10 :value $patdata :deaf T :valuex 35c
      :button READPAT  :label "read"
      :button READPATCLR  :label "read & clear"
 
   :panel ADC :layout T :padx 10
      :layout H; :row-gap 10
      :hskip 10

      :number ADC-NUM :label "ADC number:" :min 0 :max $maxadc :w 5
      :menubutton ADC-SEL :label "select:"  :strings $selectadcmenu
      :hskip 200
      :checkbox ALL-ADCS :label "Act on ALL adcs?" 

      :row-gap 10; :next-row T
      :slider ULD :label "Upper Level Discriminator:" :min 0 :max 255 :valuex 35c

      :row-gap 5; :next-row T
      :slider LLD :label "Lower Level Discriminator:" :min 0 :max 255 :valuex 35c

      :next-row T
      :slider OFFSET :label "Offset Memory:" :min 0 :max 255 :valuex 35c

      :next-row T; :col-gap 10
      :hskip 100
      :text ADCDATA :label "ADC Data:" :w 10 :value $adcdata :deaf T 
      :button READADC :label "read"
      :button READADCCLR :label "read & clear"

   :panel CLICK :layout T :padx 10
      :layout H; :col-gap 15

      :vskip 10; :hskip 40
      :button REDISPLAY :label "Redisplay"
      :hskip 100
      :button RESET :label "Reset Module" 
      :button TEST :label "Test Module"

   :show T

 }

