# SCC.tcl---Serial Crate Controller module control frame


set debug 0
set started 0


set frame SCC
set frametitle "Serial Crate Controller Setup & Monitor"
set icontitle "SCC"

if {[info exists env(CAMACServer)] == 0} then {set MIDASAccessServer CAMAC}  \
        else {set MIDASAccessServer $env(CAMACServer)}

set resource $MIDASAccessServer

set maxcrate 50
set cchoice {}
for {set i 1} {$i <= $maxcrate} {incr i} {lappend cchoice $i}

set camstation "30"
set camcrate "1"
set mname "SCC#1"

proc define {c} {
    global mname resource
    set mname "SCC#$c"

    EG  "register :pattern '/$resource/SCC#$c.*' !delete"

    define-register SCC#$c xCAMAC status          "-c $c -n 30 -a 0 -f 1  -p rw -l 0 -b 0 -z x -q 1"
    define-register SCC#$c xCAMAC set             "-c $c -n 30 -a 0 -f 19 -p wo -l 0 -b 0 "
    define-register SCC#$c xCAMAC clear           "-c $c -n 30 -a 0 -f 23 -p wo -l 0 -b 0 "
 }

proc define-register {mname class rname attributes} {
   global resource

       set register /$resource/$mname.$rname
       set-footer "defining register $register"
       EG "register :name '$register' :class '$class' !new"
       EG "register :name '$register' :object '$attributes'"

   clear-footer
}

proc {no-QX} {module} {

   display-error-report self "no QX accessing $module - module probably missing or broken" \
        "check crate and station supplied in configuration are correct"
}

proc write-reg {r v} {
   global resource mname

       EG  "register :name '/$resource/$mname.$r' :value '$v'"
}

proc read-reg {r} {
   global resource mname

       EG "register :name '/$resource/$mname.$r' ?value"
}

proc redisplay-status {} {

   set stat [read-reg status]
   set qx [second $stat]
   if {$qx == "%00"} then {no-QX SCC}
   set stat [first $stat]
   update-widget BIT1 [expr $stat & 1]
   update-widget BIT2 [expr $stat >> 1 & 1]
   update-widget BIT3 [expr $stat >> 2 & 1]
   update-widget BIT4 [expr $stat >> 3 & 1]
   update-widget BIT5 [expr $stat >> 4 & 1]
   update-widget BIT6 [expr $stat >> 5 & 1]
   update-widget BIT7 [expr $stat >> 6 & 1]
#   update-widget BIT8 [expr $stat >> 7 & 1]
   update-widget BIT9 [expr $stat >> 8 & 1]
   update-widget BIT10 [expr $stat >> 9 & 1]
   update-widget BIT11 [expr $stat >> 10 & 1]
   update-widget BIT13 [expr $stat >> 12 & 1]
   update-widget BIT14 [expr $stat >> 13 & 1]
#   update-widget BIT15 [expr $stat >> 14 & 1]
   update-widget BIT16 [expr $stat >> 15 & 1]
}

proc update-widget {widget value} {
   :modify $widget :value $value
}

proc click-control {w m v} {
   clear-footer
   if {$m==0} then {write-reg clear $v} else {write-reg set $v}
}

proc click-online {} {
    set-footer "Onlining Crate"
    write-reg status 0
    after 1000 write-reg status 1
    after 2000 write-reg status 0x0100
    after 3000 redisplay-status
    after 4000 clear-footer
}

proc click-offline {} {
    write-reg status 0x1C00
    after 1000 redisplay-status
}

proc select-crate {v} {
   global cchoice camcrate
   set camcrate [lindex $cchoice $v]
   define $camcrate
}

proc click-status {} {

    set rc [InterfaceStatus]

    if {[first $rc] != 0} then {
         midas-warning "The CAMAC Access Server has returned an error: code=[first $rc]; ,reason=[second $rc]"
         return
    }

    if {[second $rc] != "pci5992"} then {
         midas-warning "This operation is not supported for [second $rc]"
         return
    }

    set status1  [lindex $rc 2]
    set status2  [lindex $rc 3]
    set status3  [lindex $rc 4]

    insert-log "[second $rc] status is $status1 $status2 $status3"

}

proc click arg {
   global cnafdata camcrate
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch $w  {

      CAM-CRATE  {select-crate $v}

      BIT1   {click-control $w $v 0x0001}
      BIT2   {click-control $w $v 0x0002}
      BIT3   {click-control $w $v 0x0004}
      BIT9   {click-control $w $v 0x0100}
      BIT10  {click-control $w $v 0x0200}
      BIT11  {click-control $w $v 0x0400}
      BIT12  {click-control $w $v 0x0800}
      BIT13  {click-control $w $v 0x1000}


      DEFINE {define $camcrate}
      REDISPLAY	{}

      PUT-ONLINE  {click-online; return}
      PUT-OFFLINE {click-offline}
      STATUS      {click-status; return}

      default   {puts stdout $arg}
      }
   redisplay-status
   }


proc start args {
   global started camcrate
   if {$started == 0} then {make-frame} else {open-frame}
   set started 1
   define $camcrate
   redisplay-status
}


proc make-frame {} {
  global env camstation cchoice

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/camac_module" \
      :icon-name "[self icon-title]" :resize F

  :panel MODULE  :layout T :padx 10 :pady 10
    :layout H; :row-gap 10; :col-gap 20

    :hskip 200
    :choice-stack CAM-CRATE :label "CAMAC Crate:" :exclusive T :strings $cchoice :value 0
    :text CAM-STATION :label "CAMAC Station:" :w 5 :value $camstation :deaf T

    :next-row T
    :message M1 :label "Status Register (READ)" :bold T

    :row-gap 0; :col-gap 40; :next-row T
    :hskip 20
    :checkbox BIT4  :label "DERR"  :deaf T 
    :checkbox BIT5  :label "DSX"   :deaf T 
    :checkbox BIT6  :label "DSQ"   :deaf T
    :checkbox BIT7  :label "Dataway I" :deaf T

    :next-row T
    :hskip 20
    :checkbox BIT14 :label "Switch off-line" :deaf T 
    :checkbox BIT16 :label "Selected Ls present" :deaf T

    :next-row T
    :vskip 20
    :message M2 :label "Status Register (WRITE)" :bold T 

    :next-row T
    :hskip 20
    :checkbox BIT1  :label "Initiates Dataway Z"
    :checkbox BIT2  :label "Initiates Dataway C"
    :checkbox BIT12 :label "SCC bypass" 

    :next-row T
    :vskip 20
    :message M3 :label "Status Register (READ/WRITE)" :bold T

    :next-row T
    :hskip 20
    :checkbox BIT3  :label "SCC control-bit for Dataway I"
    :checkbox BIT9  :label "Demand Enable"
    :checkbox BIT10 :label "Internal demand (L24)" 

    :next-row T
    :hskip 20
    :checkbox BIT11 :label "Loop Collapse" 
    :checkbox BIT13 :label "Dataway off-line"

  :panel CLICK :layout T :padx 10
    :layout H; :col-gap 20
    :button REDISPLAY    :label "Redisplay"
    :button PUT-ONLINE   :label "Put Online"
    :button PUT-OFFLINE  :label "Put Offline"
    :button DEFINE       :label "Define Registers"
#    :button STATUS       :label "Interface Status"

   :show T
}
