# SEN4S2004.tcl---SEN 4S2004 Scaler module control frame


set frame SEN4S2004
set frametitle "SEN 4S2004 Scaler Setup & Monitor"
set icontitle "SEN4S2004"

set mname "sen4s2004"

set maxmodule 16
set nchannels 4

set maxmodule [inform camaccontrol inquire [set mname]count]
set modulelist {}
for {set i 1} {$i <= $maxmodule} {incr i} {lappend modulelist $i}

set module [first $modulelist]

set wrname(INCREMENT)	increment

set rrname(INCREMENT)	increment

set rrname(READDATA)     data
set rrname(READDATACLR)  data+clr

set DATA DATA
for {set i 0} {$i < $nchannels} {incr i} {set wname(data$i)     C$i$DATA}
for {set i 0} {$i < $nchannels} {incr i} {set wname(data+clr$i) C$i$DATA}

for {set sid [array startsearch wrname]} {[array anymore wrname $sid]} { } {
	set element [array nextelement wrname $sid]
	set wname($wrname($element)) $element
}
for {set sid [array startsearch rrname]} {[array anymore rrname $sid]} { } {
	set element [array nextelement rrname $sid]
	set wname($rrname($element)) $element
}

loadsource common.camac
set reset_cmd "increment"


proc repaint {} {
  global env module adc
  :modify MODULE-NUM :value $module
}

proc redisplay-module {} {
   global debug
   if {$debug} then {puts stderr "redisplay-module"}
   camac-address
}

proc redisplay-channels {} {
   global debug channel nchannels
   if {$debug} then {puts stderr "redisplay-channels"}
   set register data
   for {set i 0} {$i < $nchannels} {incr i} {
      set channel $i
      update-text-widget $register$channel [expr [read-channel-reg $register]]
   }
}

#    define dummy proc to replace version in common.camac
proc click-select-adc {i} {}

proc click-clear {} {
   global channel nchannels
   set register data+clr
   for {set i 0} {$i < $nchannels} {incr i} {
      set channel $i
      read-channel-reg $register
   }
}

proc click-clear-all {}  {
  global modulelist
   foreach module $modulelist {
      click-select-module $module
      click-clear
      click-redisplay
   }
}

proc click arg {
   global debug rrname
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch $w {

      MODULE-SEL      -
      MODULE-SEL-MENU -
      MODULE-NUM      {click-select-module $v}

      READDATA    -
      READDATACLR {click-read-channels $w}

      INCREMENT {read-reg $rrname($w)}
      CLEAR     {click-clear; click-redisplay}

      REDISPLAY	{click-redisplay}
      default   {puts stdout $arg}
   }
}


set gap 100
set col1 50
set col2  [expr $col1+$gap]
set col3  [expr $col2+$gap]


proc make-frame {} {
  global env maxmodule camcrate camstation nchannels
  global modulelist

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/camac_module" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE :layout T :padx 10 :pady 10
     :layout H; :row-gap 10

      :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5
      :menubutton MODULE-SEL :label "select:" :strings $modulelist
      :hskip 40
      :text CAM-CRATE   :label "CAMAC Crate:"    :w 5 :value $camcrate   :deaf T
      :text CAM-STATION :label "CAMAC Station:"  :w 5 :value $camstation :deaf T

      :next-row T; :col-gap 20
      :hskip 25
      :message M1 :label "Channel values"  :bold T
      :button READDATA :label "read"
      :button READDATACLR :label "read & clear" 

      :next-row T
      for {set i 0} {$i < $nchannels} {incr i} {
          :text C[set i]DATA :label $i: :w 10 :value 0 :deaf T
      }

   :panel CLICK :layout T :padx 10
      :layout H; :col-gap 30

      :button REDISPLAY :label Redisplay 
      :hskip 40
      :button CLEAR     :label "Clear all Scalers"
      :button INCREMENT :label "Increment Scalers"

   :show T

}
