# camaccontrol.tcl---CAMAC Configuration & Control Frame
#     default version 
#     RegisterServer


set debug 0
set EGreport -1
set started 0

set frame CAMAC
set frametitle "CAMAC Configuration & Control Frame"
set icontitle "CAMAC"


if {[info exists env(CamacController)]} then {set controller $env(CamacController)} else {set controller SCC}
set CAMACcontroller $controller

if {[info exists env(CAMACServer)] == 0} then {set MIDASAccessServer CAMAC}  \
        else {set MIDASAccessServer $env(CAMACServer)}

set RegisterServer $MIDASAccessServer
set resource $MIDASAccessServer

#    load moduledefinition list

loadsource camaccontrol.defn

foreach item $moduledefinition {set [second $item]count 0}

set modulechoice {}
set modulenames  {}
set tasknames    {}
set channels     {}
set databits     {}
set pattwords    {}

foreach item $moduledefinition {append modulechoice " " \"[first $item]\"}
foreach item $moduledefinition {append modulenames  " " [second $item]}
foreach item $moduledefinition {append tasknames    " " [third $item]}
foreach item $moduledefinition {append channels     " " [fourth $item]}
foreach item $moduledefinition {append databits     " " [fifth $item]}
foreach item $moduledefinition {append pattwords    " " [sixth $item]}


set moduletype     [first $modulechoice]
set modulename    [first $modulenames]
set camaccrate   1
set camacstation 1
set vsn      1
set subaddr  0
set numchans 0
set group    0
set item     0
set ipg      0
set iincr    1
set icomment ""

proc eleventh p {lindex $p 10}

set occupied {}

set Configdir "$env(MIDASEXPBASE)"
catch {set Configdir "$env(EXPERIMENTBASE)"}

set LUTdir "$env(MIDASEXPBASE)"
catch {set LUTdir "$env(EXPERIMENTBASE)"}
set LUTfile "LUT.fera"
set LUTmsg "\n Do you wish to create a new LUT file?\n\
       You will need to do this if you have changed the configuration of FERA data sources\n\n"

set selection -1

set m {
    {{}            {}             {  CAMAC} {   CAMAC}  {VSN}  {SubAddr} {Number of} {Group}   {Item}    {Items/}  {Item}  {Comment    } }
    {{Module Type} { Module Name} {  Crate} {  Station} {}     {}        {channels}  {Number}  {Number}  {group}   {Incr}  {}  }
      } 

set datalist {}

#    configuration list 
#      for each module need 
#        1.   module type
#        2.   camac crate
#        3.   camac station
#        4.   vsn
#        5.   subaddress of first channel
#        6.   number of channels
#        7.   group number for first channel
#        8.   item number for first channel
#        9.   number of items per group
#        10.  item increment per channel
#        11.  comment

set defaultlist {}

proc initialise {} {
    global moduledefinition

#    load module definitions and action procedures
   foreach litem $moduledefinition {
      if {[first $litem] != ""} then {
         set lmoduletype [first $litem]
         set ltaskname   [third $litem]
         catch {loadsource $ltaskname.defn}
         set lname "_registersavepatterns"
         global $lmoduletype$lname
         if {[info exists $lmoduletype$lname] == 0} then {set $lmoduletype$lname {}}
      }
   }
}

proc set-experiment dir {
   global env LUTdir Configdir
   set LUTdir $dir
   set Configdir $dir
}

proc {no-QX} {module} {

   display-error-report self "no QX accessing $module - module probably missing or broken" \
        "check crate and station supplied in configuration are correct"
}

proc DAQ-action {procname} {
   global env debug datalist occupied
   if {[catch {set x $env(EXPERIMENTDIR)}] != 0} then {midas-warning "No Current Experiment!"; return}
   set selection 0
   foreach data $datalist {
      set moduletype [first $data]
      set modulename [second $data]
      set camaccrate [third $data]
      set camacstation [fourth $data]
      set vsn [fifth $data]
      if {[set occupied(selection,c$camaccrate,n$camacstation)] == $selection} then {
#    first occurence of this specific module in the configuration
         if {$debug} then {insert-debug "$procname $moduletype $modulename <$moduletype$procname>"}
         if {[info procs $moduletype$procname] != ""} then {$moduletype$procname $modulename $vsn} 
      }
      incr selection
   }
#   send sys redisplay-electronics
}

proc DAQ-reset {} {DAQ-action "_DAQ-reset"}
proc DAQ-go {}    {DAQ-action "_DAQ-go"}
proc DAQ-halt {}  {DAQ-action "_DAQ-halt"}
proc DAQ-setup {} {DAQ-action "_DAQ-setup"}


proc define-register {mname class rname attributes} {
   global CAMACcontroller resource

      set register /$resource/$mname.$rname
      EG "register :name '$register' :class '$class' !new"
      if {$CAMACcontroller == "RS232CC" || $CAMACcontroller == "CC"} then {
         EG "register :name '$register' :object '$attributes -w 24'"
      } else {
         EG "register :name '$register' :object '$attributes'"
      }
}

if {$LUTpresent != 0} {

proc format-data l {
  set ll {}
  foreach f $l {lappend ll [format "%16s %12s %8s %8s %8s %8s %8s %8s %6s %6s %6s %20s" \
   [first $f] [second $f] [third $f] [fourth $f] [fifth $f] [sixth $f] [seventh $f] [eigth $f] [nineth $f] [tenth $f] [eleventh $f] [lindex $f 11]]}
  return $ll
}
} else {

proc format-data l {
  set ll {}
  foreach f $l {lappend ll [format "%16s %12s %8s %8s %8s %20s" \
   [first $f] [second $f] [third $f] [fourth $f] [fifth $f] [lindex $f 11]]}
  return $ll
}
}

proc make-datalist {} {
   global moduledefinition modulechoice channels modulenames
   global datalist
#     this computes the module name instance and channel number for each module
   foreach item $moduledefinition {
      if {[first $item] != ""} then {
        global [second $item]count
        set [second $item]count 0
      }
   }
#    delete the table occupied and recreate as a way of deleting all elements
   global occupied; catch {unset occupied}; global occupied
   set channelnum 1
   set selection 0
   foreach data $datalist {
#      load-data $selection
      set moduletype "[first $data]"
      set camaccrate [third $data]
      set camacstation [fourth $data]
      set vsn [fifth $data]
      set subaddr [sixth $data]
      set numchans [seventh $data]
      set group [eigth $data]
      set item [nineth $data]
      set ipg [tenth $data]
      set iincr [eleventh $data]
      set icom "[lindex $data 11]"
      set moduleindex [lsearch $modulechoice $moduletype]
      set modulename [lindex $modulenames $moduleindex]

      if {[info exists occupied(name,c$camaccrate,n$camacstation)] == 0} then {
#      first use of camac crate slot
         if {$moduleindex >= 0} {
            incr [lindex $modulenames $moduleindex]count
            append modulename  # [expr $[lindex $modulenames $moduleindex]count]
            set occupied(name,c$camaccrate,n$camacstation) [lindex $modulenames $moduleindex]
            set occupied(index,c$camaccrate,n$camacstation) [expr $[lindex $modulenames $moduleindex]count]
            set occupied(selection,c$camaccrate,n$camacstation) $selection
         } else {
            midas-warning "module $moduletype is in the configuration file but is not a defined module for this experiment"
         }
      } else {
#      camac crate slot already mentioned
         set modulename [set occupied(name,c$camaccrate,n$camacstation)]
         if {[lindex $modulenames $moduleindex] == $modulename} then {
#          same module type is probably OK
            set-footer "Warning - CAMAC crate $camaccrate station $camacstation previously mentioned"
            append modulename #  [set occupied(index,c$camaccrate,n$camacstation)]
         } else {
            display-error-report self "CAMAC crate $camaccrate station $camacstation used by $modulename and [lindex $modulenames $moduleindex]"
         }
      }

      set datalist [lreplace $datalist $selection $selection \
         [list $moduletype $modulename $camaccrate $camacstation $vsn $subaddr $numchans $group $item $ipg $iincr $icom]]
      set channelnum [expr $channelnum+[lindex $channels $moduleindex]]
      incr selection
   }
   paint-datalist
   repaint
}

proc makeLUT {} {
  global env debug datalist LUTdir LUTfile
  global modulechoice  databits pattwords channels

  set-footer "Making the LUT file"

  set z [catch {open [file join $LUTdir $LUTfile] w} file]
  if {$z != 0} then {set-footer "$file"; return}

  puts $file "# fera vsn/sa => group/item conversion"
  puts $file "#     vsn  :   subaddr  :  # data bits  :  max subaddr  :  # pattern words  :  group   :  item "

#    calculate LUT entry for each adc in each module
  foreach data $datalist {
    set moduletype [first $data]
    set nadcs [lindex $channels [lsearch $modulechoice $moduletype]]
    set dbits [lindex $databits [lsearch $modulechoice $moduletype]]
    set pwrds [lindex $pattwords [lsearch $modulechoice $moduletype]]
    set vsn [fifth $data]
    set sa [sixth $data]
    set numchans [seventh $data]
    set group [eigth $data]
    set item [nineth $data]
    set ipg [tenth $data]
    set iincr [eleventh $data]
    set items 0
    for {set i 0} {$i < $numchans} {incr i} {
      incr items
      puts $file "fera $vsn $sa $dbits $nadcs $pwrds $group $item"
      incr sa
      incr item $iincr
      if {$items >= $ipg} then {set item [nineth $data]; incr group; set items 0}
    }
  }

  close $file
  set-footer "LUT written to [file join $LUTdir $LUTfile]"
}

proc load-data n {
   global datalist moduletype modulename camaccrate camacstation vsn subaddr numchans group item ipg iincr icomment
   set data [lindex $datalist $n]

   set moduletype [first $data]
   set modulename [second $data]
   set camaccrate [third $data]
   set camacstation [fourth $data]
   set vsn [fifth $data]
   set subaddr [sixth $data]
   set numchans [seventh $data]
   set group [eigth $data]
   set item [nineth $data]
   set ipg [tenth $data]
   set iincr [eleventh $data]
   set icomment [lindex $data 11]
}

proc load-configuration {} {
   global resource
   global debug env Configdir
   global datalist occupied modulenames

   set-footer "deleting all CAMAC registers"
   foreach module $modulenames {
           EG  "register :pattern '/$resource/$module*' !delete"
   }

   set-footer "Creating Configuration"
   set selection 0
   foreach data $datalist {
     set moduletype [first $data]
     set modulename [second $data]
     set camaccrate [third $data]
     set camacstation [fourth $data]
     if {[set occupied(selection,c$camaccrate,n$camacstation)] == $selection} then {
#    first occurence of this specific module in the configuration
         set-footer "Creating module $modulename"
         if {$debug} then {insert-debug "load-configuration $moduletype $modulename $camaccrate $camacstation"}
         if {[info procs $moduletype] != ""} then {$moduletype $modulename $camaccrate $camacstation}
     }
     incr selection
   }

#    write the current configuration to disc

    set z [catch {open [file join $Configdir [ set resource]_current_configuration] w} file]
       
   if {$z != 0} then {set-footer "$file"; return}
   foreach data $datalist {puts $file "$data"}
   close $file

#    record that we have done it

   write-register NetVar_HasBeenConfigured 1

   clear-footer
}

proc load-configuration+LUT {} {
    global LUTmsg LUTpresent

    load-configuration

    if {$LUTpresent != 0} {
        if {[midas-confirmation "$LUTmsg"] != 0} then {makeLUT}
    }
}

proc HasBeenConfigured {} {

    read-register NetVar_HasBeenConfigured
}

proc restore_configuration {} {
   global env resource Configdir datalist

   set datalist ""

   set z [catch {open [file join $Configdir [ set resource]_current_configuration] r} file]

   if {$z != 0} then {set-footer "$file"; return}

   set configuration [split [read $file] \n]
   close $file
   set l [expr [llength $configuration] - 1]
   set datalist [lreplace $configuration $l $l]
   make-datalist
}

proc Configuration {} {global datalist; return $datalist}

proc paint-datalist {} {
  global env datalist selection
  global m
  set pointstrings [format-data $datalist]
  set strings [format-data $m]
    :modify M :empty T :strings "$strings"
    :modify CONFIG-DATALIST :show F :empty T :strings "$pointstrings" :show T
    :modify CONFIG-DATALIST :select $selection
    :modify CONFIG-DATALIST :see 0
}

proc repaint {} {
   global env debug moduletype modulename camaccrate camacstation vsn subaddr numchans group item ipg iincr icomment
   global modulechoice cratechoice stationchoice numchoice 
   global LUTpresent
   if {$debug} then {insert-debug "repaint"}
   set mt [lsearch $modulechoice $moduletype]
   if {$mt == -1} then {insert-debug "cannot find $moduletype"}
   set cc [lsearch $cratechoice $camaccrate]
   set cn [lsearch $stationchoice $camacstation]
   set lvsn [lsearch $numchoice $vsn]
   set lsa [lsearch $numchoice $subaddr]
   set lchans [lsearch $numchoice $numchans]
   set litem [lsearch $numchoice $item]
   set lipg [lsearch $numchoice $ipg]
   set liincr [lsearch $numchoice $iincr]
     :modify MODULE :value $mt
     :modify CRATE  :value $cc
     :modify STATION :value $cn
     :modify VSN :value $lvsn
if {$LUTpresent != 0} {
     :modify SUBADDR :value $lsa
     :modify CHANNELS :value $lchans
     :modify GROUP :value $group
     :modify ITEM :value $litem
     :modify IPG :value $lipg
     :modify IINCR :value $liincr
}
     :modify COMMENT :value $icomment
}

proc repaint-selection {} {
  global selection
    :modify CONFIG-DATALIST :select $selection
}

proc click-add {} {
   global datalist moduletype modulename camaccrate camacstation vsn subaddr numchans group item ipg iincr icomment
   lappend datalist  [list $moduletype $modulename $camaccrate $camacstation $vsn $subaddr $numchans $group $item $ipg $iincr $icomment]
   make-datalist
   clear-footer
}

proc click-replace {} {
   global selection 
   global datalist moduletype modulename camaccrate camacstation vsn subaddr numchans group item ipg iincr icomment
   if {$selection<0} then \
      {set-footer "you haven't selected a line to replace!"; return}
   set datalist [lreplace $datalist $selection $selection \
                     [list $moduletype $modulename $camaccrate $camacstation $vsn $subaddr $numchans $group $item $ipg $iincr $icomment]]
   make-datalist
   clear-footer
}

proc click-delete {} {
   global selection datalist
   if {$selection<0} then \
      {set-footer "you haven't selected a line to delete!"; return}
   set datalist [lreplace $datalist $selection $selection]
   set selection -1
   make-datalist
   clear-footer
}

proc click-clear {} {
   global datalist
   set datalist {}
   paint-datalist
}

proc click-default {} {
   global defaultlist datalist
#    this copies the list "defaultlist" into the format of the list "datalist"
   set modulename ""
   set channel ""
   set datalist {}
   foreach data $defaultlist {
      set moduletype [first $data]
      set camaccrate [second $data]
      set camacstation [third $data]
      set vsn [fourth $data]
      set subaddr [fifth $data]
      set numchans [sixth $data]
      set group [seventh $data]
      set item [eigth $data]
      set ipg [nineth $data]
      set iincr [tenth $data]
      set icom [eleventh $data]
      lappend datalist  \
    [list $moduletype $modulename $camaccrate $camacstation $vsn $subaddr $numchans $group $item $ipg $iincr $icom]
   }
   make-datalist
}

proc click-printconfig {} {
  global env debug datalist

  set info "[format "%10s %8s %8s %6s %10s %6s %6s"  \
          channel crate  station  vsn   subaddress  group   item]\n\n" 

  foreach data $datalist {
    set modulename [second $data]
    set camaccrate [third $data]
    set camacstation [fourth $data]
    set vsn [fifth $data]
    set subaddr [sixth $data]
    set numchans [seventh $data]
    set group [eigth $data]
    set item [nineth $data]
    set ipg [tenth $data]
    set iincr [eleventh $data]

    set items 0
    for {set i 0} {$i < $numchans} {incr i} {
        incr items

        append info "[format "%24s %4s %4s %6s %4s %6s %4s"  \
                $modulename.subaddr#$subaddr  $camaccrate  $camacstation  $vsn  $subaddr  $group $item]\n"

        incr subaddr
        incr item $iincr
        if {$items >= $ipg} then {set item [nineth $data]; incr group; set items 0}
    }
  }
  global _information_font
  set _information_font fixed
  midas-information $info
}

proc click-redisplay {} {
   global env debug
   if {$debug} then {insert-debug "click-redisplay"}
   :modify REDISPLAY :busy T
   paint-datalist
   repaint
   :modify REDISPLAY :busy F
}

proc click-datalist {c s} {
  global debug selection
  if {$debug} then {insert-debug "click-datalist with arguments \{$c $s\}"}
  case $c in {
    :list-select {set selection $s; load-data $selection}
    :list-deselect {set selection -1}
  }
  repaint
  clear-footer
}

proc double-click-datalist {c s} {
  global debug selection moduletype modulename tasknames modulechoice
  if {$debug} then {insert-debug "double-click-datalist with arguments \{$c $s\}"}
  set selection $s; load-data $selection
  set taskname [lindex $tasknames [lsearch $modulechoice $moduletype]]
  set n [second [split $modulename #]]
  inform $taskname set module $n
  inform $taskname start
}

proc click-module {n} {
  global debug modulechoice modulenames moduletype modulename
  if {$debug} then {insert-debug "click-module with argument \{$n\}"}
  set moduletype [lindex $modulechoice $n]
  set modulename [lindex $modulenames $n]
}
 
proc click-crate {n} {
  global cratechoice camaccrate
  set camaccrate [lindex $cratechoice $n]
}
 
proc click-station {n} {
  global stationchoice camacstation
  set camacstation [lindex $stationchoice $n]
}

proc click-vsn {n} {
  global numchoice vsn
  set vsn [lindex $numchoice $n]
}

proc click-sa {n} {
  global numchoice subaddr
  set subaddr [lindex $numchoice $n]
}

proc click-channels {n} {
  global numchoice numchans
  set numchans [lindex $numchoice $n]
}

proc click-group {n} {
  global numchoice group
  set group $n
}

proc click-item {n} {
  global numchoice item
  set item [lindex $numchoice $n]
}

proc click-ipg {n} {
  global numchoice ipg
  set ipg [lindex $numchoice $n]
}

proc click-iincr {n} {
  global numchoice iincr
  set iincr [lindex $numchoice $n]
}

proc click-comment {n} {
  global icomment
  set icomment $n
}

proc double-click arg {
   global debug
   if {$debug} then {insert-debug "double-click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   switch $w {

      CONFIG-DATALIST {double-click-datalist $c [lindex $arg 4]}
    
      default   {puts stdout $arg}
   }
}



proc click arg {
   global debug LUTpresent
   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"}  then {click-quit; return}
   if {$c==":Enter"} then {repaint-selection; return}
   switch $w  {

    MODULE    {click-module $v}
    CRATE    {click-crate  $v}
    STATION    {click-station $v}
    VSN     {click-vsn $v}
        SUBADDR {click-sa $v}
        CHANNELS {click-channels $v}
    GROUP   {click-group $v}
    ITEM    {click-item $v}
    IPG    {click-ipg $v}
        IINCR   {click-iincr $v}
        COMMENT {click-comment $v}

    ADD    {click-add}
    REPLACE    {click-replace}
    DELETE    {click-delete}
        CLEAR   {click-clear}
        DEFAULT {click-default}

        LOAD    {if {$LUTpresent != 0} then {load-configuration+LUT} else {load-configuration}}

    CONFIG-DATALIST {click-datalist $c [lindex $arg 4]}
    
        RESET   {DAQ-reset}
        GO      {DAQ-go}
        HALT    {DAQ-halt}

        SAVERESTORE    {inform camacsaverestore SSRstart}
        SR             {inform camacsaverestore CSRstart}
        PRINT   {
            set-footer "See Session Log"
            click-printconfig
        }

        REDISPLAY  {click-redisplay}

        default   {puts stdout $arg}
    }
}

proc start args {
   global  started
   if {$started == 0} then {load_preferences; make-frame; restore_configuration}
   open-frame
   initialise
   click-redisplay
}

proc autostart args {
   global  started
   if {$started == 0} then {start; close-frame} else {continue}
}

proc click-quit args {close-frame}

proc finish args {destroy-frame}

set cratechoice {}
for {set i 1} {$i <= 50} {incr i} {lappend cratechoice $i}

set stationchoice {}
for {set i 1} {$i <= 24} {incr i} {lappend stationchoice $i}

set numchoice {}
for {set i 0} {$i < 49} {incr i} {lappend numchoice $i}
lappend numchoice 255

proc make-frame {} {
   global env started
   global modulechoice cratechoice stationchoice numchoice
   global LUTpresent

  set started 1

  :frame [self frame-name] :label "[self frame-title]"     \
      :show-footer T  :resize F  \
      :icon-image "@$env(ICONHOME)/camac_crate" \
      :icon-name "[self icon-title]"

  :enable [self frame-name] Enter

   :panel CONFIGURATION  :layout T :padx 10 :pady 10
      :layout V; :row-gap 10; :col-gap 10
if {$LUTpresent != 0} then {set cols 130} else {set cols 90}
      :list M  :rows 3 :cols $cols :strings [list ""] :deaf T
      :list CONFIG-DATALIST  :rows 12 :cols $cols :exclusive T :strings [list ""] :double-click T
   :panel OPTIONS  :layout T :padx 10
      :layout H; :row-gap 10; :col-gap 10

      :choice-stack MODULE  :label "Module Type:"   :exclusive T  :strings "$modulechoice"  :value 0
      :choice-stack CRATE   :label "CAMAC Crate:"   :exclusive T  :strings "$cratechoice"   :value 0
      :choice-stack STATION :label "CAMAC Station:" :exclusive T  :strings "$stationchoice"   :value 0
      :choice-stack VSN     :label VSN:             :exclusive T  :strings "$numchoice"       :value 0
if {$LUTpresent != 0} {
      :choice-stack SUBADDR :label SubAddr:         :exclusive T  :strings "$numchoice"       :value 0
      :choice-stack CHANNELS :label Channels:       :exclusive T  :strings "$numchoice"       :value 0

      :next-row T
#      :choice-stack GROUP :label Group:            :exclusive T  :strings "$numchoice"    :value 1
      :text GROUP         :label Group:      :w 5    :value ""
      :choice-stack ITEM  :label Item:              :exclusive T  :strings "$numchoice"    :value 0
      :choice-stack IPG   :label "Items per group:" :exclusive T  :strings "$numchoice"    :value 2
      :choice-stack IINCR :label "Item increment:"  :exclusive T  :strings "$numchoice"    :value 2
}
if {$LUTpresent == 0} then {:next-row T}
      :text COMMENT       :label Comment:    :w 32   :value ""
 
   :panel CLICK :layout T :padx 10
      :layout H; :row-gap 10; :col-gap 15
      :vskip 10
      :button ADD     :label Add
      :button REPLACE :label Replace
      :button DELETE  :label Delete
      :button CLEAR   :label Clear
      :button LOAD    :label "Load Configuration into Register Server"
      :next-row T
      :button RESET   :label "RESET electronics"
      :button GO      :label "GO electronics"
      :button HALT    :label "HALT electronics"
if {$LUTpresent == 0} then {:next-row T}
      :button SAVERESTORE    :label "Save/Restore Setup"
      :button SR             :label "Save/Restore Configuration"
if {$LUTpresent != 0} {
      :button PRINT   :label "Print Configuration"
      :next-row T
}
      :hskip 40
      :button REDISPLAY :label Redisplay

   modify-frame

   :show T
}

proc modify-frame {} {}
