# camacdiag.tcl---CAMAC Diagnostics frame


set debug 0
set started 0

set frame CAMACDIAG
set frametitle "CAMAC Diagnostics"
set icontitle  "CAMACDiag"

package require ServerAccess 2.0    ;#  gets new version of EG command and Server Access procedures.

if {[info exists env(CAMACServer)] == 0} then {set MIDASAccessServer CAMAC}  \
        else {set MIDASAccessServer $env(CAMACServer)}

set maxbits 24
set names ""
for {set i 1} {$i<=$maxbits} {incr i} {
   lappend names "2^[expr $maxbits-$i]"
}
set names [quote-list $names]

set cchoice {"1" "2" "3" "4" "5" "6" "7" "8" "9" "10" "11" "12" "13" "14" "15" "16"}
set nchoice {"1" "2" "3" "4" "5" "6" "7" "8" "9" "10" "11" "12" "13" "14" "15" "16" \
   "17" "18" "19" "20" "21" "22" "23" "24"}
set achoice {"0" "1" "2" "3" "4" "5" "6" "7" "8" "9" "10" "11" "12" "13" "14" "15"}
set fchoice {"0" "1" "2" "3" "4" "5" "6" "7" "8" "9" "10" "11" "12" "13" "14" "15" "16" \
   "17" "18" "19" "20" "21" "22" "23" "24" "25" "26" "27" "28" "29" "30" "31"}
set wchoice {"16" "24"}
set pchoice {"rw" "ro" "wo"}
set lchoice {"0" "1" "2" "3" "4" "5" "6" "7" "8" "9" "10" "11" "12" "13" "14" "15" "16"}
set bchoice {"0" "1" "2" "3" "4" "5" "6" "7" "8" "9" "10" "11" "12" "13" "14" "15" "16"}
set zchoice {"x" "d" "b"}
set qchoice {"0" "1"}

set classchoice {"xCAMAC" "cCAMAC"}

set debugchoice {"0" "0x01" "0x02" "0x04" "0x08" "0x10" "-1"}

set camacdata "0x0000"
set regdata "0x0000"
set camacsdata "00"
set textdata ""


proc make-frame {} {
  global env
  global names cchoice nchoice achoice fchoice wchoice pchoice lchoice bchoice
  global zchoice qchoice classchoice debugchoice
  global textdata camacdata camacsdata regdata

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/default" \
      :icon-name "[self icon-title]" :resize F

   :panel BSD-PANEL :layout L :padx 10
      :layout H; :row-gap 10
      :vskip 10

      :choice BSD-SW  :label "Switches :" :strings $names :layout V :exclusive F :w 5 :deaf T
      :choice BSD-LED :label "LEDs :"     :strings $names :layout V :exclusive F :w 5

      :layout V; :row-gap 5; :next-row T
      :choice-stack BSD-C   :label "Crate"   :exclusive T :strings $cchoice :value 0
      :choice-stack BSD-N   :label "Station" :exclusive T :strings $nchoice :value 0
      :button BSD-DEFINE    :label "Define BSD" 
      :button BSD-REDISPLAY :label "Redisplay"
   
   :panel DIAG-PANEL :layout L :padx 10
      :layout H; :row-gap 10; :col-gap 5
      :vskip 10

      :message M1 :label "Camac.Address" :bold T
      :next-row T
      :choice-stack DIAG-C :label c :exclusive T :strings $cchoice :value 0
      :choice-stack DIAG-N :label n :exclusive T :strings $nchoice :value 0
      :choice-stack DIAG-A :label a :exclusive T :strings $achoice :value 0
      :choice-stack DIAG-F :label f :exclusive T :strings $fchoice :value 0
      :choice-stack DIAG-W :label w :exclusive T :strings $wchoice :value 0
      :next-row T
      :button ADDR-W :label "Write Camac.Address"
      :button ADDR-I :label "Initialise Camac.Address"
      :next-row T
      :text ADDR-RD :label "" :w 30 :value "$textdata" :deaf T
      :button ADDR-R :label "Read Camac.Address"
      :row-gap 15
      :next-row T
      :message M2 :label "Camac.Execute" :bold T
      :row-gap 5
      :next-row T
      :text EXEC-WD :label "" :w 10 :value "$camacdata"
      :button EXEC-W :label "Write Camac.Execute"
      :next-row T
      :text EXEC-RD :label "" :w 10 :value "$camacdata" :deaf T
      :button EXEC-R :label "Read Camac.Execute"
      :row-gap 10
      :next-row T
      :message M3 :label "Camac.Status" :bold T
      :row-gap 5
      :next-row T
      :text STATUS-RD :label "" :w 3 :value "$camacsdata" :deaf T
      :button STATUS-R :label "Read Camac.Status"
      :col-gap 80
      :choice-stack DEBUG :label "Camac.Debug" :exclusive T :strings $debugchoice :value 0
      :row-gap 10; :col-gap 5
      :next-row T
      :message M4 :label "Camac.Data" :bold T
      :row-gap 5
      :next-row T
      :text DATA-RD :label "" :w 10 :value "$camacdata" :deaf T
      :button DATA-R :label "Read Camac.Data"
      :row-gap 30
      :next-row T
      :text REG-NAME :label "Register Name:" :w 50 :value "$textdata"
      :row-gap 15
      :next-row T
      :choice-stack REG-CLASS :label "Class" :exclusive T :strings $classchoice :value 0
      :next-row T
      :message M5 :label "Register Attributes"  :bold T
      :row-gap 0; :col-gap 10
      :next-row T
      :choice-stack REG-C :label c :exclusive T :strings $cchoice :value 0
      :choice-stack REG-N :label n :exclusive T :strings $nchoice :value 0
      :choice-stack REG-A :label a :exclusive T :strings $achoice :value 0
      :choice-stack REG-F :label f :exclusive T :strings $fchoice :value 0
      :choice-stack REG-W :label w :exclusive T :strings $wchoice :value 0
      :row-gap 10
      :next-row T
      :choice-stack REG-P :label p :exclusive T :strings $pchoice :value 0
      :choice-stack REG-L :label l :exclusive T :strings $lchoice :value 0
      :choice-stack REG-B :label b :exclusive T :strings $bchoice :value 0
      :choice-stack REG-Z :label z :exclusive T :strings $zchoice :value 0
      :choice-stack REG-Q :label q :exclusive T :strings $qchoice :value 0
      :row-gap 15; :col-gap 0
      :next-row T
      :button REG-CR :label "Create Register"
      :button REG-WA :label "Write Register Attributes"
      :button REG-I  :label "Initialise Register"
      :row-gap 5
      :next-row T
      :text REG-RAD :label "" :w 60 :value "$textdata" :deaf T
      :next-row T
      :button REG-RA :label "Read Register Attributes"
      :next-row T
      :text REG-WDATA :label "" :w 10 :value "$regdata"
      :button REG-WD :label "Write Register"
      :col-gap 10
      :text REG-RDATA :label "" :w 10 :value "$textdata" :deaf T
      :button REG-RD :label "Read Register"
   
    :show T
}


set bsd_c 1
set bsd_n 1
set diag_c 1
set diag_n 1
set diag_a 0
set diag_f 0
set diag_w 16
set reg_name ""
set reg_class "xCAMAC"
set reg_c 1
set reg_n 1
set reg_a 0
set reg_f 0
set reg_w 16
set reg_p rw
set reg_l 0
set reg_b 0
set reg_z d
set reg_q 0


proc write-camac-register {r v} {return [first [MIDASWriteRegister Camac.$r $v]]}

proc read-camac-register {r} {return [tail [MIDASReadRegister Camac.$r]]}

proc initialise-camac-register {r} {return [first [MIDASInitialiseRegister Camac.$r]]}

proc write-register {r v} {return [first [MIDASWriteRegister $r $v]]}

proc write-registers {r v} {return [first [MIDASWriteRegisters $r $v]]}

proc read-register {r} {return [second [MIDASReadRegister $r]]}

proc create-register {r class} {return [first [MIDASDefineRegister $r $class "-c 0"]]}

proc delete-register {r} {return [first [MIDASDeleteRegister $r]]}

proc delete-registers {r} {return [first [MIDASDeleteRegisters $r]] }

proc write-attributes {r v} {return [first [MIDASWriteRegisterAttributes $r "$v"]]}

proc read-attributes {r} {return [tail [MIDASReadRegisterAttributes $r]]}

proc initialise-register {r} {return [first [MIDASInitialiseRegister $r]]}

proc initialise-registers {r} {return [first [MIDASInitialiseRegisters $r]]}

proc click-bsd-define {} {
   global bsd_c bsd_n
   set footer "defining register bsd.data"
   delete-registers bsd.data
   create-register bsd.data xCAMAC
   write-attributes bsd.data "-c $bsd_c -n $bsd_n -a 0 -f 0 -w 24 -p rw"
   clear-footer
}

proc click-write-attributes {} {
   global reg_class reg_name
   global reg_c reg_n reg_a reg_f reg_w reg_p reg_l reg_b reg_z reg_q
    switch $reg_class {
       xCAMAC  {
           write-attributes "$reg_name"  "-c $reg_c -n $reg_n -a $reg_a -f $reg_f -w $reg_w -p $reg_p -l $reg_l -b $reg_b -z $reg_z -q $reg_q"
       }
       cCAMAC  {
           write-attributes "$reg_name" "-c $reg_c -n $reg_n -a $reg_a -f $reg_f -q $reg_q"
       }
    }
}

proc click-bsd-redisplay {} {
  global maxbits
  :modify BSD-SW :value [invert $maxbits [read-register bsd.data]]
}

proc click-addr-redisplay {} {
  :modify ADDR-RD :value [read-camac-register "Address"]
}

proc click-exec-redisplay {} {
  :modify EXEC-RD :value [read-camac-register "Execute"]
}

proc click-status-redisplay {} {
  :modify STATUS-RD :value [read-camac-register "Status"]
}

proc click-data-redisplay {} {
  :modify DATA-RD :value [read-camac-register "Data"]
}

proc click-reg-redisplay {} {
  global reg_name
  :modify REG-RDATA :value "[read-register "$reg_name"]"
}

proc click-rega-redisplay {} {
  global reg_name
  :modify REG-RAD :value "[read-attributes "$reg_name"]"
}

proc click arg {
   global env debug resource maxbits bsd_c bsd_n diag_c diag_n diag_a diag_f diag_w
   global reg_name reg_class
   global reg_c reg_n reg_a reg_f reg_w reg_p reg_l reg_b reg_z reg_q
   global camacdata regdata
   global cchoice nchoice achoice fchoice wchoice pchoice lchoice bchoice zchoice qchoice
   global debugchoice classchoice
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch $w {

    BSD-C            {set bsd_c [lindex $cchoice $v]}
    BSD-N            {set bsd_n [lindex $nchoice $v]}
    BSD-DEFINE       {click-bsd-define}
    BSD-LED          {write-register bsd.data [invert $maxbits $v]}
    BSD-REDISPLAY    {click-bsd-redisplay}

    DIAG-C    {set diag_c [lindex $cchoice $v]}
    DIAG-N    {set diag_n [lindex $nchoice $v]}
    DIAG-A    {set diag_a [lindex $achoice $v]}
    DIAG-F    {set diag_f [lindex $fchoice $v]}
    DIAG-W    {set diag_w [lindex $wchoice $v]}

    ADDR-W    {write-camac-register "Address"  "-c $diag_c -n $diag_n -a $diag_a -f $diag_f -w $diag_w"}
    ADDR-I    {initialise-camac-register "Address"}
    ADDR-R    {click-addr-redisplay}

    EXEC-WD   {set camacdata [expr $v]}
    EXEC-W    {write-camac-register "Execute" "$camacdata"}
    EXEC-R    {click-exec-redisplay}

    STATUS-R  {click-status-redisplay}

    DATA-R    {click-data-redisplay}

    REG-NAME    {set reg_name $v}
    REG-CLASS   {set reg_class [lindex $classchoice $v]}
    REG-C    {set reg_c [lindex $cchoice $v]}
    REG-N    {set reg_n [lindex $nchoice $v]}
    REG-A    {set reg_a [lindex $achoice $v]}
    REG-F    {set reg_f [lindex $fchoice $v]}
    REG-W    {set reg_w [lindex $wchoice $v]}
    REG-P    {set reg_p [lindex $pchoice $v]}
    REG-L    {set reg_l [lindex $lchoice $v]}
    REG-B    {set reg_b [lindex $bchoice $v]}
    REG-Z    {set reg_z [lindex $zchoice $v]}
    REG-Q    {set reg_q [lindex $qchoice $v]}

    REG-CR    {create-register $reg_name $reg_class}
    REG-I     {initialise-register "$reg_name"}
    REG-WA    {click-write-attributes}
    REG-RA    {click-rega-redisplay}

    REG-WDATA {set regdata $v}
    REG-WD    {write-register "$reg_name" "$regdata"}
    REG-RD    {click-reg-redisplay}

    DEBUG    {write-camac-register Debug [lindex $debugchoice $v]}

      default    {puts stdout $arg}
      }
   }


proc start args {
   global started
   if {$started == 0} then {make-frame} else {open-frame}
   set started 1
}

proc click-quit {} {
   close-frame
}

proc finish args {destroy-frame}



set bashlimit 100

proc bash {} {
   global EGreport EGallow
   global bashlimit
  set EGreport -1
  set EGallow 0x2000a
  set total 0
  for {set count 0} {$count < $bashlimit} {incr count} {
     set data [read-register bsd.data]
     if {$data == 0xffffff} then {break}
     write-register bsd.data $data
     incr total
  }
  set-footer "$total camac write operations"
}


proc click-bsd-define {} {
   global  bsd_c bsd_n
   delete-registers bsd.*
   set register bsd.data
   set footer "defining register $register"
   create-register $register xCAMAC
   write-attributes $register "-c $bsd_c -n $bsd_n -a 0 -f 0 -w 24 -p rw"
   for {set i 1} {$i <= 32} {incr i} {
      set register bsd.data$i
      set footer "defining register $register"
      create-register $register xCAMAC
      write-attributes $register "-c $bsd_c -n $bsd_n -a 0 -f 0 -w 24 -p rw"
   }
   clear-footer
}


