#    common procedures used by CAMAC module tasks

if {[info exists env(CAMACServer)] == 0} then {set MIDASAccessServer CAMAC}  \
        else {set MIDASAccessServer $env(CAMACServer)}

set resource $MIDASAccessServer

set debug 0
set started 0

set camcrate "0"
set camstation "0"

if {![info exists module]} then {set module 1}
set adc  0
set channel 0

set alladcs 0
set allchans 0
set allmodules 0

set moduleprefix "#"
set adcsarecalled "adc"
set reset_cmd "reset"


proc write-camac-register {r v} {
       global resource
       EG  "register :name '/$resource/Camac.$r'  :value '$v'"
}

proc read-camac-register {r} {
       global resource
       EG "register :name '/$resource/Camac.$r'  ?value"
}

proc write-reg {r v} {
   global moduleprefix module allmodules mname
       global resource

         if {$allmodules} then {set lmodule *; set opt :pattern} \
                    else {set lmodule $module; set opt :name}
       EG  "register $opt '/$resource/$mname$moduleprefix$lmodule.$r' :value '$v'"
}

proc read-reg {r} {
   global moduleprefix module mname
       global resource

       EG "register :name '/$resource/$mname$moduleprefix$module.$r'  ?value"
}

proc read-adc-reg {r} {
   global module adc mname moduleprefix adcsarecalled
       global resource

       EG "register :name '/$resource/$mname$moduleprefix$module.$adcsarecalled$adc.$r'  ?value"
}

proc write-adc-reg {r v} {
   global module adc allmodules alladcs mname
   global moduleprefix adcsarecalled
       global resource

       set opt :name
       if {$allmodules} then {set lmodule *; set opt :pattern} \
                    else {set lmodule $module}
       if {$alladcs}    then {set ladc *; set opt :pattern} \
                    else {set ladc $adc}
       EG  "register $opt '/$resource/$mname$moduleprefix$lmodule.$adcsarecalled$ladc.$r' :value '$v'"
}

proc read-chan-reg {r} {
   global moduleprefix module chan mname
       global resource

       EG "register :name '/$resource/$mname$moduleprefix$module.chan$chan.$r'  ?value"
}

proc write-chan-reg {r v} {
   global moduleprefix module chan allmodules allchans mname
       global resource

       set opt :name
       if {$allmodules} then {set lmodule *; set opt :pattern} \
                    else {set lmodule $module}
       if {$allchans}    then {set lchan *; set opt :pattern} \
                    else {set lchan $chan}
       EG  "register $opt '/$resource/$mname$moduleprefix$lmodule.chan$lchan.$r' :value '$v'"
}

proc read-channel-reg {r} {
   global moduleprefix module channel mname
       global resource

       EG "register :name '/$resource/$mname$moduleprefix$module.c$channel.$r' ?value"
}

proc read-attributes {r} {
   global moduleprefix module mname
       global resource

       EG "register :name '/$resource/$mname$moduleprefix$module.$r' ?object"
}

proc camac-address {} {
   global resource mname module camcrate camstation
   global reset_cmd
   set attribs [read-attributes $reset_cmd]
   foreach f1 $attribs {
     if {[string first "-c" "$f1"] >= 0} then {
       set camcrate "[string range $f1 2 end]"
       :modify CAM-CRATE :value $camcrate 
     }
     if {[string first "-n" $f1] >= 0} then {
       set camstation "[string range $f1 2 end]"
       :modify CAM-STATION :value $camstation 
     }
   }
}
 
proc update-widget {register value} {
   global debug rrname wrname wname
   if {$debug} then {puts stderr "update-widget with arguments \{$register\} \{$value\}"}
   :modify $wname($register) :value $value
}

proc update-text-widget {register value} {
   global debug rrname wrname wname
   if {$debug} then {puts stderr "update-text-widget with arguments \{$register\} \{$value\}"}
   :modify $wname($register) :value $value
}

proc click-slider {w v} {
   global debug rrname wrname
   if {$debug} then {puts stderr "click-slider with arguments \{$w\} \{$v\}"}
   clear-footer
   set register $wrname($w)
   write-reg $register $v
   set register $rrname($w)
   update-widget $register [read-reg $register]
}

proc click-adc-slider {w v} {
   global debug rrname wrname
   if {$debug} then {puts stderr "click-adc-slider with arguments \{$w\} \{$v\}"}
   clear-footer
   set register $wrname($w)
   write-adc-reg $register $v
   set register $rrname($w)
   update-widget $register [read-adc-reg $register]
}

proc click-box {w v} {
   global debug rrname wrname
   if {$debug} then {puts stderr "click-box with arguments \{$w\} \{$v\}"}
   clear-footer
   set register $wrname($w)
   write-reg $register $v
   set register $rrname($w)
   update-widget $register [read-reg $register]
}

proc click-text {w v} {
   global debug rrname wrname
   if {$debug} then {puts stderr "click-text with arguments \{$w\} \{$v\}"}
   clear-footer
   set register $wrname($w)
   write-reg $register $v
   set register $rrname($w)
   update-text-widget $register [read-reg $register]
}

proc click-read-adc {w} {
  global rrname
  set register $rrname($w)
  update-text-widget $register [read-adc-reg $register]
}

proc click-read-data {w} {
  global rrname
  set register $rrname($w)
  update-text-widget $register [read-reg $register]
}

proc click-write-data {w v} {
   global wrname
   set register $wrname($w)
   write-reg $register $v
}

proc click-read-channels {w} {
   global rrname channel nchannels
   for {set i 0} {$i < $nchannels} {incr i} {
      set channel $i
      set register $rrname($w)
      update-text-widget $register$channel [expr [read-channel-reg $register]]
   }
}

proc click-action {w} {
   global rrname
   set register $rrname($w)
   read-reg $register
   redisplay-module
   redisplay-adc
}

proc click-select-module {v} {
   global debug module maxmodule
   if {$debug} then {puts stderr "click-select-module with argument \{$v\}"}
   set module $v
   if {$module<0} then {set module 0}
   if {$module>$maxmodule} then {set module $maxmodule}
   redisplay-module
   click-select-adc 0
   repaint
}

proc click-select-adc {v} {
   global debug adc maxadc
   if {$debug} then {puts stderr "click-select-adc with argument \{$v\}"}
   set adc $v
   if {$adc<0} then {set adc 0}
   if {$adc>$maxadc} then {set adc $maxadc}
   redisplay-adc
   repaint
}

proc click-all-adcs {v} {global alladcs; set alladcs $v}

proc click-all-chans {v} {global allchans; set allchans $v}

proc click-all-modules {v} {global allmodules; set allmodules $v}

proc repaint {} {
   global module adc
   :modify MODULE-NUM :value $module
   :modify ADC-NUM    :value $adc
}

proc redisplay-module {} {}
proc redisplay-adc {} {}
proc redisplay-channels {} {}

proc click-redisplay {} {
   global debug
   if {$debug} then {puts stderr "click-redisplay"}
   repaint
   redisplay-module
   redisplay-adc
   redisplay-channels
}

proc start args {
   global started
   if {$started == 0} then {make-frame; EGbindings} else {open-frame}
   set started 1
   click-redisplay
}

proc click-quit args {close-frame}

proc finish args {destroy-frame}

proc starttask {} {start; close-frame}





