# fdt.tcl---Fera-DT32 card control frame


set debug 0
set started 0

set frame FDT32
set frametitle "Fera-DT32 Card Setup"
set icontitle "FDT32"

# The following variables => /$resource/$modname#$num.$regname
# e.g. /camac/fdt32#1.reset
# The resource name - a CAMAC register server
if {[info exists env(CamacServer)] == 0} then {set resource camac}  \
   else {set resource $env(CamacServer)}
# The I/F card name
set modname "fdt32"
# The I/F card number
set num 1

set rocoNumber 47
set rocochoice {"47" "1" "2" "3" "4" "5" "6" "7"}

set laststatus 0
set lastQ 0
set lastX 0

set fifonotempty 1
set fifonothfull 0
set fifonotfull 1

set fdt32dir "/eg/experiments"
catch {set fdt32dir "$env(EXPERIMENTBASE)/fdt32"}

set lutdirectory $fdt32dir
set lutfile ""
set lutcurrent ""
set lutdefault "$lutdirectory/LUT.default"
set lutcurrent "$lutdirectory/LUT.current"

set cnvfile ""

set delaydir $fdt32dir/Timing-Dir
set file(ftval) "ftval#$num"
set file(gated) "gated#$num"
set file(gatel) "gatew#$num"
set file(rqren) "rqren#$num"

set def(ftval) "ftval#$num"
set def(gated) "gated#$num"
set def(gatel) "gatew#$num"
set def(rqren) "rqren#$num"

set mode(ftval) R
set mode(gated) R
set mode(gatel) R
set mode(rqren) R

set getdel(ftval) raw-delay
set getdel(gated) raw-delay
set getdel(gatel) raw-delay
set getdel(rqren) raw-delay

set eqtns(ftval) {"1657 - 6.059*$d + 0.008513*pow($d,2 \
                  "541.3 - 0.1395*$d + 0.2129*1e-4*pow($d,2 \
                  "333.1 - 0.006411*$d + 1e-6*pow($d,2 }
set eqtns(gated) {"826.2 - 0.4623*$d + 0.0002645*pow($d,2 \
                  "653.5 - 0.02718*$d+ 0.3406*1e-5*pow($d,2 \
                  "604.6 - 0.001830*$d" \
                  "916.5 - 0.01622*$d" }
set eqtns(gatel) {"1708 - 18.34*$d + 0.06262*pow($d,2 \
           "317 - 0.4473*$d + 0.2669*1e-3*pow($d,2) - 0.5432*1e-7*pow($d,3 \
                  "81.89 - 0.01675*$d + 0.7560*1e-6*pow($d,2 }
set eqtns(rqren) {"1546 - 8.537*$d + 0.01768*pow($d,2 \
            "566 - 0.4316*$d+ 0.2646*1e-3*pow($d,2) - 0.5566*1e-7*pow($d,3 \
                  "351.6 - 0.02119*$d + 0.1383*1e-5*pow($d,2 \
                  "65.9 + 0.05024*$d - 0.3206*1e-5*pow($d,2 }

set range(ftval) {130 350 3100 20600}
set range(gated) {120 560 4000 20400 23000}
set range(gatel) {50 150 1300 7200}
set range(rqren) {75 250 1200 7300 11000}

set value(ftval) {}
set value(gated) {}
set value(gatel) {}
set value(rqren) {}

set delay(ftval) {}
set delay(gated) {}
set delay(gatel) {}
set delay(rqren) {}

set delayw(ftval) VALDELAY
set delayw(gated) GATEDELAY
set delayw(gatel) GATELENGTH
set delayw(rqren) RENDELAY

set delayl(ftval) "FT->Validation"
set delayl(gated) "Validation->Gate"
set delayl(gatel) "Gate Width"
set delayl(rqren) "ADC Request->REN"

set wrname(ENROC)	control.en_roco
set wrname(GOADC)	control.go_adc
set wrname(ENROCADC)	control
set wrname(TEST)	control.test
set wrname(GATEDELAY)	gate.delay
set wrname(GATELENGTH)	gate.length
set wrname(VALDELAY)	val.delay
set wrname(RENDELAY)	ren.delay
set wrname(LEMO1)	test.#1
set wrname(LEMO2)	test.#2
set wrname(LEMO3)	test.#3
set wrname(RESET)	reset
set wrname(LUT-ZERO)	clearaddr
set wrname(FIFOCLR)	clearfifo
set wrname(LAM)		ablelam
set wrname(LUT-LMENU)	writelut
set wrname(LUT-SMENU)	readlut

set rrname(DTPAUSE)	status.dtpause
set rrname(DTSTOP)	status.dtstop
set rrname(DTREN)	status.dtren
set rrname(DTPASS)	status.dtpass
set rrname(RDENROC)	status.en_roco
set rrname(FPASS)	status.pass_fera
set rrname(LAM)		testlam
set rrname(LUT-CUR)	writelut

for {set sid [array startsearch wrname]} {[array anymore wrname $sid]} { } {
	set element [array nextelement wrname $sid]
	set wname($wrname($element)) $element
}
for {set sid [array startsearch rrname]} {[array anymore rrname $sid]} { } {
	set element [array nextelement rrname $sid]
	set wname($rrname($element)) $element
}
set wname(control.en_roco) RDENROC
set wname(ablelam) LAM

#defines whether the xv call needs a string or number value for the widget
set xvtype(DTPAUSE)	xvval
set xvtype(DTSTOP)	xvval
set xvtype(DTREN)	xvval
set xvtype(DTPASS)	xvval
set xvtype(RDENROC)	xvval
set xvtype(FPASS)	xvval
set xvtype(LAM)		xvval
set xvtype(LUT-CUR)	xvstr

set insp1 \
  {"gate" "FastTrigger" "FT->Validation" "REQ" "Validation"
   "CLI" "TriggerOK" "WAK" "WST(ADC)" "PASS(ADC)" "InhibitRequest"}
set test(1) {0 1 2 3 4 5 6 7 15 23 31}

set insp2 \
  {"validok" "Endsubevent" "/OPBM" "/OPBMR"
   "DTPAUSE" "DTREN" "DTPASS"}
set test(2) {1 2 3 4 5 6 7}

set insp3 \
  {"/OE" "/WE" "/RD" "/WR" "/HF" "/FULL" "/EMPTY" }
set test(3) {1 2 3 4 5 6 7}

proc xvval v {
   return $v
}

proc xvstr v {
   return '$v'
}

proc dirnames l {
   foreach f $l {lappend ll [first $f]}
   return $ll
   }

proc strip-dots l {
   foreach n $l {if {$n!="." && $n!=".."} then {lappend ll $n}}
   return $ll
   }

proc strip-to-dot s {string range $s [expr 1+[string first . $s]] end }

proc fopen {name {attr r}} {
   set fl ""
   set err [catch {set fl [open $name $attr] } m]
   if {$err} {
	set-footer $m
   }
   return $fl
}

proc setupdelay {t {m L} } {
   global debug cnvframe delaydir
   global def file mode getdel value delay delayw delayl range

   set value($t) {}
   set delay($t) {}

   set mode($t) $m
   clear-footer
# if setting up for Raw "delays", do it and return
   if {$m == "R"} then {
      set getdel($t) raw-delay
      :modify $delayw($t) :label "$delayl($t):" :min 0 :max 1023
      set-footer $cnvframe "$delayl($t) set to Raw mode - no conversion"
      set-footer "$delayl($t) set to Raw mode - no conversion"
      return
   }

# if setting up for Calculated (by function) delays, do it and return
   if {$m == "C"} then {
      set getdel($t) calc-delay
      :modify $delayw($t) :label "$delayl($t) (ns):" \
          :min [first $range($t)] :max [last $range($t)]
      set-footer $cnvframe "$delayl($t) set to Calculated mode"
      set-footer "$delayl($t) set to Calculated mode"
      return
   }

# must be setting up for Lookup delays, try to do it set up Calculated if fail
   set f [fopen $delaydir/$file($t) r]
   if {$f != ""} then {
      set getdel($t) lookup-delay
      while 1 {
         set l [gets $f]
         if {[eof $f]} {break}
         lappend value($t) [first $l]
         lappend delay($t) [second $l]
      }
      close $f
      :modify $delayw($t) :label "$delayl($t) (ns):" \
          :min [first $delay($t)] :max [last $delay($t)]
      set-footer $cnvframe "$delayl($t) set to Lookup mode from $delaydir/$file($t)"
      set-footer "$delayl($t) set to Lookup mode from $delaydir/$file($t)"
   }  else  {
      set mode($t) "C"
      set getdel($t) calc-delay
      :modify $delayw($t) :label "$delayl($t) (ns):" \
          :min [first $range($t)] :max [last $range($t)]
      set-footer $cnvframe "$delayl($t) set to Calculated mode"
      set-footer "$delayl($t) set to Calculated mode"
   }
}

proc click-defdelays {} {
   global debug
   global def file mode getdel value delay delayw delayl

   set file(ftval) $def(ftval)
   set file(gatel) $def(gatel)
   set file(gated) $def(gated)
   set file(rqren) $def(rqren)

   set msg "will use calculated values for "
   setupdelay ftval
   if {$mode(ftval) == "C"} then {set msg [concat $msg "$delayl(ftval), "]}
   setupdelay gatel
   if {$mode(gatel) == "C"} then {set msg [concat $msg "$delayl(gatel), "]}
   setupdelay gated
   if {$mode(gated) == "C"} then {set msg [concat $msg "$delayl(gated), "]}
   setupdelay rqren
   if {$mode(rqren) == "C"} then {set msg [concat $msg "$delayl(rqren), "]}
   if {$msg!=""} then {
      set-footer $msg
   }
}

proc raw-delay {t d} {
   return $d
}

proc calc-delay {t d} {
   global debug eqtns range
   global def file mode getdel value delay delayw delayl
   set eqn $eqtns($t)
   set rng $range($t)
   set len [expr [llength $rng] - 1]

   if {$d < [first $rng]} {
      set d [first $rng]
   } else {
      if {$d > [last $rng]} {
         set d [last $rng]
      }
   }
   set rng [lrange $rng 1 end]
   set len [llength $rng]
   for {set i 0} {$i<$len} {incr i} {
      if {$d<[lindex $rng $i]} {
         return [expr round([lindex $eqn $i])]
      }
   }
# shouldn't be able to get here
   return -1
}

proc lookup-delay {t d} {
   global debug
   global def file mode getdel value delay delayw delayl

   set len [llength $delay($t)]
   set i 0
   set low 0
   set high [expr $len - 1]
   set foundit 0

   while {$low <= $high} {
      set i [expr ($low + $high) / 2]
      if {$i>=$len || $i<0} then {
         break
      }
      if {$d < [lindex $delay($t) $i]} then {
         set high  [incr i -1]
      } else {
         if {$d > [lindex $delay($t) $i]} then {
            set low [incr i]
         } else {
            set foundit 1
            break
         }
      }
   }
   if {$foundit} then  {
      set val [lindex $value($t) $i]
      if {$debug} then {
         puts stdout "$d found => $val"
      }
   } else {
      if {$i>=$len} then {
         set val [lindex $value($t) [expr $len-1]]
         if {$debug} then {
            puts stdout "$d too big => $val"
         }
      } else {
         if {$i<0} then {
            set val [lindex $value($t) 0]
         if {$debug} then {
            puts stdout "$d too small => $val"
         }
         } else {
            set diff [expr ([lindex $delay($t) $high] - [lindex $delay($t) $low])]
            set off [expr ($d - [lindex $delay($t) $low])]
            set prop [expr $off.0 / $diff]
            set val [lindex $value($t) $low]
            set vdiff [expr [lindex $value($t) $high] - $val]
            set voff [expr round($prop * $vdiff)]
            if {$high < $low} then {
               incr val $voff
            } else {
               incr val -$voff
            }
            if {$debug} then {
               puts stdout "$d not found, interpolated => $val"
            }
         }
      }
   }
   return $val
}

proc get-delay {t v} {
   global debug
   global def file mode getdel value delay delayw delayl

   set r [$getdel($t) $t $v]
   return $r
}

proc write-camreg {r v} {
   global resource num  debug
   if {$debug} then {
      puts stderr "(register :name '/$resource/Camac.$r'  :value '$v')"
   }
   eg "(register :name '/$resource/Camac.$r'  :value '$v')"
}

proc read-camreg {r} {
   global resource num  debug
   eg "(register :name '/$resource/Camac.$r'  ?value)"
   }

proc dbg {v} {write-camreg Debug $v}
proc dbgv {} {dbg 1}
proc dbgi {} {dbg 2}
proc dbgx {} {dbg 4}
proc dbgc {} {dbg 8}
proc dbgq {} {dbg 16}
proc dbgvi {} {dbg 3}
proc dbgvx {} {dbg 5}
proc dbgix {} {dbg 6}
proc dbgvix {} {dbg 7}
proc dbgall {} {dbg 0x1f}
proc dbgoff {} {dbg 0}

proc init-reg r {
   global resource num modname debug
   set rr /$resource/$modname#$num.$r
   set-footer "Initialising register $rr ..."
   if {$debug} then {puts stderr "Initialising register $rr"}
   set z [catch {eg "(register :name '$rr' !initialise)" } m]
   if {$z!=0} then {
      display-eg-error-report self $m "Attempting to initialise register" $rr
      error $m
   }
   clear-footer
}

proc write-reg {r v} {
   global resource num modname  laststatus lastQ lastX debug
   set rr /$resource/$modname#$num.$r
   if {$debug} then {puts stderr "(register :name '$rr' :value '$v')"}
   set z [catch {eg "(register :name '$rr' :value '$v')" } m]
   if {$z!=0} then {
      display-eg-error-report self $m "Attempting to write register" $rr
      error $m
   }
   set laststatus [read-camreg Status]
   if {$laststatus != ""} then {
      set lastQ [string index $laststatus 1]
      set lastX [string index $laststatus 2]
   }
}

proc read-reg {r} {
   global resource num modname laststatus lastQ lastX debug
   set rr /$resource/$modname#$num.$r
   set z [catch {eg "(register :name '$rr' ?value)" } m]
   if {$z!=0} then {
      display-eg-error-report self $m "Attempting to read register" $rr
      error $m
   }
   if {$debug} then {puts stderr "(register :name '$rr' ?value) => $m"}
   clear-footer
   set laststatus [second $m]
   if {$laststatus != ""} then {
      set lastQ [string index $laststatus 1]
      set lastX [string index $laststatus 2]
   }

   set v [first $m]
   if {[string index $v 1] == "x"} then {
      scan [string range $v 2 end] "%x" v
   }

   return $v
   }

proc read-regs r {
   global resource num modname debug
   set pattern /$resource/$modname#$num.$r
   set-footer "reading registers $pattern ..."
   if {$debug} then {puts stderr "reading registers $pattern"}
   set z [catch {eg "(register :pattern '$pattern' ?names-and-values)" } m]
   if {$z!=0} then {
      display-eg-error-report self $m "Attempting to read registers" $pattern
      error $m
   }
   clear-footer
   return $m
}

proc redisplay-lemos {} {
   global test
   for {set c 1} {$c < 4} {incr c} {
      set v [read-reg test.#$c]
      set i [lsearch $test($c) $v]
      :modify LEMO$c :value $i
   }
}

proc read-fifo-status {} {
   global debug fifonotempty fifonothfull fifonotfull
   set fifonotempty [read-reg status.notempty]
   set fifonothfull [read-reg status.nothfull]
   set fifonotfull [read-reg status.notfull]
   }

proc update-fifo-status {} {
   global debug fifonotempty fifonothfull fifonotfull
   set value 0
   set error 0
   set setit 0
   if {!$fifonotempty} then {
      set value 0
      set setit 1
      if {!$fifonothfull || !$fifonotfull} then {
         set error 1
      }
   }
   if {!$error && !$fifonothfull} then {
      set value 2
      set setit 1
      if {!$fifonotempty || !$fifonotfull} then {
         set error 1
      }
   }
   if {!$error && !$fifonotfull} then {
      set value 3
      set setit 1
      if {!$fifonothfull || !$fifonotempty} then {
         set error 1
      }
   }
   if {!$setit} then {
      set value 1
   }
   if {$error} then {
      display-error-report self "" "FIFO flags inconsistent" \
           "notempty: $fifonotempty, nothfull: $fifonothfull, notfull: $fifonotfull"
   } else {
      if {$debug}  then {
         puts stderr "widget: FIFOSTA, val $value"
      }
      :modify FIFOSTA :value $value
   }
}

proc update-widget {reg val} {
   global wname xvtype debug

   if {$debug}  then {
      puts stderr "widget: $wname($reg), reg: $reg, val $val"
   }
   if {$reg=="testlam"} then {set val [string index $val 1]}
   set z [catch {:modify $wname($reg) :value [$xvtype($wname($reg)) $val]} m]
   if {$z!=0} then {
      display-error-report self $m "Could not update widget" $wname($reg)
      error $m
   }
}

proc click-enroco {w v} {
   global rrname wrname wname debug
   clear-footer
   set reg $wrname($w)
   write-reg $reg $v
# en_roco control bit is echoed in en_roco status bit - update it on display
   set reg $rrname($wname($reg))
   update-widget $reg [read-reg $reg]
}

proc click-enrocadc {w v} {
   global rrname wrname wname debug
   clear-footer
   set reg $wrname($w)
   if {$v==1} then {
      set v 9
   }
   write-reg $reg $v
# en_roco control bit is echoed in en_roco status bit - update it on display
   set reg $rrname($wname(control.en_roco))
   update-widget $reg [read-reg $reg]
}

proc click-write {w v} {
   global rrname wrname debug
   clear-footer
   set reg $wrname($w)
   write-reg $reg $v
}

proc click-delay {w v t} {
   global def file mode getdel value delay delayw delayl
   global wrname

   set v [get-delay $t $v]
   write-reg $wrname($delayw($t)) $v
#   puts stdout "Writing $v to $wrname($delayw($t)
}

proc click-lemo {w n v} {
   global debug rrname wrname debug test
   clear-footer
   set reg $wrname($w)
   write-reg $reg [lindex $test($n) $v]
}

proc click-lam {w v} {
   global rrname wrname wname debug
   clear-footer
   set reg $wrname($w)
   if {$v} then {
      write-reg en$reg $v
    } else {
      write-reg dis$reg $v
    }
   set reg $rrname($wname($reg))
   update-widget $reg [read-reg $reg]
}

proc click-reset {w} {
   global rrname wrname debug
   clear-footer
   set reg $wrname($w)
   write-reg $reg 0
   click-redisplay
}

proc click-clrfifo {w} {
   global rrname wrname debug
   clear-footer
   set reg $wrname($w)
   write-reg $reg 0
   read-fifo-status
   update-fifo-status
}

proc click-zero {w} {
   global rrname wrname debug
   clear-footer
   set reg $wrname($w)
   write-reg $reg 0
# zeroing the LUT address turns out to be more complicated than a single write
# to the counter you also need to do a write to the LUT to transfer the
# (now zeroed) address counter to the LUT's :address register" (according to
# the documentation)
   set reg lutitem
   write-reg $reg 0
}

proc click-redisplay {} {
   global rrname cnvvisible
   clear-footer
   lut-paint-directory
   lut-repaint

   foreach w [array names rrname] {
      set reg $rrname($w)
      update-widget $reg [read-reg $reg]
   }
   read-fifo-status
   update-fifo-status

#   redisplay-lemos
   if {$cnvvisible} then {click-cnv-redisplay}
}

proc click-lut-redisplay {} {
   global debug
   lut-repaint
   lut-paint-directory
   update-widget writelut [read-reg writelut]
}

proc lut-paint-directory {} {
   global lutdirectory
   set z [catch {eg "(directory :path '$lutdirectory' ?files)"} m]
   if {$z!=0} then {
     display-eg-error-report self $m "Attempting to read directory" $lutdirectory
     error $m
   }
   set dirlist [lsort [strip-dots [dirnames $m]]]
   :modify LUT-LIST :empty T :strings $dirlist :see 0
}
                 
proc lut-repaint {} {
   global lutdirectory lutfile 
   :modify LUT-DIRECTORY :value "$lutdirectory"
   :modify LUT-FILE      :value "$lutfile"
}

proc click-lut-list {c file} {
   global lutfile
   switch $c {
      :list-select {click-lut-file $file}
      :list-deselect {click-lut-file ""}
      }
   }

proc load-lut {} {click-lut-load LUT-LMENU Default}

proc click-lut-load {w m} {
   global lutfile lutdirectory lutdefault wrname
   global rocoNumber
   set reg $wrname($w)

#    first zero the LUT address register
   click-zero LUT-ZERO

#    load LUT from file  8k - 32 words are loaded
   if {$m=="Default" || $lutfile==""} then {
      set-footer "writing LUT from $lutdefault"
      write-reg $reg $lutdefault
   } else {
      set-footer "writing LUT from $lutdirectory/$lutfile"
      write-reg $reg $lutdirectory/$lutfile
   }

#    load last 32 words of the LUT
   set lutdata [expr [expr [expr $rocoNumber << 8] + 255] << 2]
   for {set i 0} {$i < 8} {incr i} {
     for {set j 0} {$j < 4} {incr j} {
      write-reg lutitem [expr $lutdata + $j]
     }
   }

   click-lut-redisplay
}


proc save-lut {} {click-lut-save LUT-SMENU Default}

proc click-lut-save {w m} {
   global lutfile lutdirectory lutcurrent wrname
   set reg $wrname($w)

#    first zero the LUT address register
   click-zero LUT-ZERO

   if {$m=="Default" || $lutfile==""} then {
      set-footer "writing LUT to $lutcurrent"
      write-reg $reg $lutcurrent
   } else {
      set-footer "writing LUT to $lutdirectory/$lutfile"
      write-reg $reg $lutdirectory/$lutfile
   }

   click-lut-redisplay
}

proc click-lut-file name {
   global lutdirectory lutfile 
   set lutfile $name
   lut-repaint          
   if {$name!=""&&[string match d* [first [exec ls -ld $lutdirectory/$name]]]} then {
      click-lut-directory $lutdirectory/$name
   }
}

proc click-lut-directory dir {
   global lutdirectory lutfile
   set lutdirectory $dir
   lut-paint-directory
   set lutfile ""
   lut-repaint
}

proc click-cnv {} {
   global cnvvisible cnvframe
   open-frame $cnvframe
   set cnvvisible 1
   click-cnv-redisplay
}   

proc click arg {
   global debug
   global rocoNumber rocochoice
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit $f; return}
   switch $w {
      ENROC        {click-enroco $w $v}
      GOADC        {click-write $w $v}
      ENROCADC     {click-enrocadc $w $v}
      TEST         {click-write $w $v}

      GATEDELAY    {click-delay $w $v gated}
      GATELENGTH   {click-delay $w $v gatel}
      VALDELAY     {click-delay $w $v ftval}
      RENDELAY     {click-delay $w $v rqren}

      CONVFILES    {click-cnv }

      FIFOCLR      {click-clrfifo $w}

      LUT-DIRECTORY {click-lut-directory $v}
      LUT-FILE     {click-lut-file $v}
      LUT-LIST     {click-lut-list $c $v}
      LUT-ZERO     {click-zero $w}
      LUT-LOAD     -
      LUT-LMENU    {click-lut-load $w $v}
      LUT-SAVE     -
      LUT-SMENU    {click-lut-save $w $v}

      ROCO         {set rocoNumber [lindex $rocochoice $v]}

      LEMO1        {click-lemo $w 1 $v}
      LEMO2        {click-lemo $w 2 $v}
      LEMO3        {click-lemo $w 3 $v}

      RESET        {click-reset $w}
      LAM          {click-lam $w $v}

      REDISPLAY    {click-redisplay}

      CNV-DIRECTORY {click-cnv-directory $v}
      CNV-FILE     {click-cnv-file $v}
      CNV-LIST     {click-cnv-list $c $v}

      SET-GT       -
      SET-FT       -
      SET-RQ       -
      SET-WI       {}
      GT-MENU      -
      FT-MENU      -
      RQ-MENU      -
      WI-MENU      {click-cnv-set $w $v}

      CNV-SET-GT   -
      CNV-SET-FT   -
      CNV-SET-RQ   -
      CNV-SET-WI   {}
      CNV-GT-MENU  -
      CNV-FT-MENU  -
      CNV-RQ-MENU  -
      CNV-WI-MENU  {click-cnv-set $w $v}

      DEFAULTS       -
      CNV-DEFAULTS  {click-defdelays}

      CNV-REDISPLAY {click-cnv-redisplay}

      default    {puts stderr $arg}
   }
}

proc set-experiment dir {
   global env fdt32dir lutdirectory lutdefault
   set fdt32dir "$env(EXPERIMENTBASE)/fdt32"
   set lutdirectory $fdt32dir
   set lutdefault "$lutdirectory/LUT.default"
   set lutcurrent "$lutdirectory/LUT.current"
   click-redisplay
}

proc cnv-paint-directory {} {
	global delaydir
	set z [catch {eg "(directory :path '$delaydir' ?files)"} m]
	if {$z!=0} then {
		display-eg-error-report self $m "Attempting to read directory" $delaydir
		error $m
	}
	set dirlist [lsort [strip-dots [dirnames $m]]]
	:modify CNV-LIST :empty T :strings $dirlist :see 0
}
                 
proc cnv-repaint {} {
   global delaydir cnvfile 
   :modify CNV-DIRECTORY :value "$delaydir"
   :modify CNV-FILE :value "$cnvfile"
}

proc click-cnv-list {c file} {

   switch $c {
      :list-select {click-cnv-file $file}
      :list-deselect {click-cnv-file ""}
   }
}

proc click-cnv-set {w m} {
   global debug cnvfile delaydir cnvvisible
   global def file mode getdel value delay delayw delayl

   switch $w {
      GT-MENU  -
      CNV-GT-MENU  {set t gated}

      WI-MENU  -
      CNV-WI-MENU  {set t gatel}

      FT-MENU  -
      CNV-FT-MENU  {set t ftval}

      RQ-MENU  -
      CNV-RQ-MENU  {set t rqren}

      default {return}
   }

   switch $m  {
      {Calculated}	{
         set rcl C
      }
      {Current File}	{
         if {$cnvfile==""} then {
            set file($t) $def($t)
            set cnvfile $def($t)
         } else {
            set file($t) $cnvfile
         }
         set rcl L
      }
      {Default File}	{
         set file($t) $def($t)
         set cnvfile $def($t)
         set rcl L
      }
      {Raw}	{
         set rcl R
      }
      default {return}
   }

   setupdelay $t $rcl
   if {$cnvvisible} then {click-cnv-redisplay}
}

proc click-cnv-redisplay {} {
   global delaydir cnvfile debug
   cnv-repaint
   cnv-paint-directory
   }

proc click-cnv-file name {
	global delaydir cnvfile 
	set cnvfile $name
	cnv-repaint          
	if {$name!=""&&[string match d* [first [exec ls -ld $delaydir/$name]]]} then {
      click-cnv-directory $delaydir/$name
	}
}

proc click-cnv-directory dir {
   global delaydir cnvfile
   set delaydir $dir
   cnv-paint-directory
   set cnvfile ""
   cnv-repaint
   }


#----------------------------------------------------------------------------
# Some utility procs to set the slot number (subaddress) of the fdt32 interface 
# called by "setslot <value>" - no graphical interface
# finds each fdt32 register and sets its -n attribute to <value>

proc getregnames {} {
   global resource num modname debug
   set rr /$resource/$modname#$num.*
   set err [catch {eg "(register :pattern '$rr' ?names)" } m]
   if {$err} { display-eg-error-report self $m }
   return $m
}

proc set-attr {r v} {
   global resource num modname debug
   set rr /$resource/$r
   if {$debug} then {puts stderr "(register :name '$rr' :value '$v')"}
   set z [catch {eg "(register :name '$rr' :object '$v')" } m]
   if {$z!=0} then {
      display-eg-error-report self $m "Attempting to write attributes to register" $rr
      error $m
   }
}

proc setslot {slot} {
   foreach i [getregnames] {set-attr $i "-n $slot"}
}


proc make-frame {} {
  global env
  global insp1 insp2 insp3 lutdirectory lutfile rocochoice

set col1 25c

  :frame [self frame-name] :label "[self frame-title]"  :show-footer T \
      :icon-image "@$env(ICONHOME)/camac_module" \
      :icon-name "[self icon-title]" :resize F

   :panel STATUS :layout T :padx 5 :pady 5
     :layout H;  :row-gap 5
      :message M1 :label "Status Registers:" :bold T
      :next-row T
      :checkbox DTPAUSE :label "DTPAUSE:"          :deaf T :valuex 12c
      :checkbox DTSTOP  :label "DTSTOP:"           :deaf T :valuex 10c
      :checkbox DTREN   :label "DTREN:"            :deaf T :valuex 10c
      :checkbox DTPASS  :label "DTPASS:"           :deaf T :valuex 10c
      :checkbox RDENROC :label "ROCO Enabled:"     :deaf T :valuex 15c
      :checkbox FPASS  :label "Fera PASS:"         :deaf T :valuex 12c
   
   :panel CONTROL :layout T :padx 5 :pady 5
     :layout H; :row-gap 5
      :message M2 :label "Control Registers:" :bold T 
      :next-row T
      :checkbox ENROC    :label "Enable ROCO:"       :valuex 20c
      :checkbox GOADC    :label "Enable ADC:"        :valuex 15c
      :checkbox ENROCADC :label "Enable ROCO & ADC:" :valuex 20c
      :checkbox TEST     :label "Test Mode:"         :valuex 15c
   
   :panel P3 :layout T :padx 5 :pady 5
     :layout H; :row-gap 10; :col-gap 20
      :message M3 :label "Timings:" :bold T 
      :next-row T
      :slider GATEDELAY :label "Validation->Gate:" :min 99 :max 23000 :valuex $col1
      :menubutton GT-MENU :label "Val->Gate" \
         :strings [list "Default File" "Calculated" "Raw"] 
      :next-row T
      :slider GATELENGTH :label "Gate Width:" :min 50 :max 7000 :valuex $col1
      :menubutton WI-MENU :label "Gate Width" \
         :strings [list "Default File" "Calculated" "Raw"]
      :next-row T
      :slider VALDELAY :label "FT->Validation:" :min 99 :max 20000 :valuex $col1
      :menubutton FT-MENU :label "FT->Val" \
         :strings [list "Default File" "Calculated" "Raw"] 
      :hskip 30
      :button DEFAULTS :label "Default Conversions" 
      :next-row T
      :slider RENDELAY :label "ADC Request->REN:" :min 75 :max 11000 :valuex $col1
      :menubutton RQ-MENU :label "REQ->REN" \
         :strings [list "Default File" "Calculated" "Raw"]
      :hskip 10
      :button CONVFILES :label "Setup Conversions..." 
      :next-row T
      :message M4 :label "FIFO:" :bold T
      :next-row T
      :choice FIFOSTA :label "Status:" :deaf T :valuex $col1 \
         :strings [list "Empty" "Not Half Full" "Half Full" "Full"] 
      :button FIFOCLR :label "Clear FIFO"
   
   :panel LUT :layout T :padx 5 :pady 5
     :layout H; :row-gap 10; :col-gap 20
      :message M5 :label "Lookup Table:" :bold T
      :hskip 350
      :list LUT-LIST :rows 7 :cols 20
      :vskip -100
      :next-row T
      :text LUT-DIRECTORY :label "Directory:" :w 50  :valuex 15c :value "$lutdirectory"
      :next-row T
      :text LUT-FILE :label "File:" :w 30 :valuex 15c :value "$lutfile"
      :next-row T
      :text LUT-CUR :label "Current:" :w 50 :valuex 15c :read-only T
      :next-row T
      :hskip 50
      :menubutton LUT-LMENU :label "Load LUT"  :strings [list "Default" "Selected File"]
      :choice-stack ROCO :label "ROCO Number" :exclusive T :strings $rocochoice :value 0
      :next-row T
      :hskip 50
      :button RESET :label "Reset Module" 
      :choice LAM :label "LAM:" :exclusive T :strings [list "Disabled" "Enabled"]
   
   :panel P5 :layout T :padx 5 :pady 5
     :layout H; :row-gap 10; :col-gap 50
     :message M6 :label "Inspection Lemos:" :bold T
     :choice-stack LEMO1 :label "1"  :strings $insp1 
     :choice-stack LEMO2 :label "2"  :strings $insp2 
     :choice-stack LEMO3 :label "3"  :strings $insp3 
     :next-row T
     :hskip 50
     :button REDISPLAY :label "Redisplay" 
     :button HELP :label "Help..."

   :show T
   
}

set cnvframe "[self frame-name].cnv"
set cnvvisible 0

proc make-cnvframe {} {
  global env cnvframe
  global delaydir cnvfile

set col0 100

   :frame $cnvframe :label "Fera-DT32 Delay Conversion Management" \
      :show-footer T :resize F \
      :icon-image "@$env(ICONHOME)/camac_module" \
      :icon-name "[self icon-title]Cnv"

  close-frame $cnvframe

   :panel P1 :layout L :padx 5 :pady 5
    :layout V; :row-gap 10; :col-gap 10

      :text CNV-DIRECTORY :label "Directory:" :w 50  :valuex 10c :value "$delaydir"
      :text CNV-FILE :label "File:" :w 30 :valuex 10c :value "$cnvfile"
      :message M1 :label "Setup Conversion Table:" :bold T
      :menubutton CNV-GT-MENU :label "Val->Gate" \
         :strings [list "Current File" "Default File" "Calculated" "Raw"]
   :layout H
      :menubutton CNV-FT-MENU :label "FT->Val"   \
         :strings [list "Current File" "Default File" "Calculated" "Raw"]
      :menubutton CNV-RQ-MENU :label "REQ->REN"  \
         :strings [list "Current File" "Default File" "Calculated" "Raw"]
   :next-row T
      :menubutton CNV-WI-MENU :label "Gate Width" \
         :strings [list "Current File" "Default File" "Calculated" "Raw"]
   :col-gap 50
      :button CNV-DEFAULTS :label "Default Conversions"
   :next-row T
      :button CNV-REDISPLAY :label "Redisplay"
      :hskip 80
      :button CNV-HELP :label "Help..."

   :panel P2 :layout L :padx 5 :pady 5
      :list CNV-LIST :rows 10 :cols 20

}


proc start args {
   global started
   if $started then {open-frame} else {
     make-frame
     make-cnvframe
     click-defdelays
     set started 1
   }
   click-redisplay
}

proc click-quit f {
  global cnvvisible cnvframe
   close-frame $f
   if {$cnvvisible == 1} then {close-frame $cnvframe; set cnvvisible 0}
}

proc finish args {destroy-frame}
